/******************************************************************************
	Fileio.C
******************************************************************************/
#include	<sys/time.h>
#include	<X11/keysym.h>
#include	"typedef.h"
#if 1 /* 20000214 ROM-FILE */
#include	"xim.h"
#include	"xui.h"
#else /* 20000214 */
#include	"vjectrl.h"
#include	"vjeenv.h"
#endif/* 20000214 */
#include	"macro.h"
#include	"fileio.h"
#include	"env.h"
#include	"getstr.h"
#include	"strutl.h"

/*=============================================================================
	Local	Data
=============================================================================*/
/*-----------------------------------------------------------------------------
	Fix	String
-----------------------------------------------------------------------------*/
static	TCHAR	sztCfgF[]	= TEXT("vje.cfg");
static	TCHAR	sztVje[]	= TEXT("Vje");
static	TCHAR	sztAtw[]	= TEXT("AtwApp");

#if BSD
static	TCHAR	sztEtcD[]	= TEXT("/usr/local/etc/vje30");
#else
static	TCHAR	sztEtcD[]	= TEXT("/etc/vje30");
#endif
static	TCHAR	sztInsD[]	= TEXT("/usr/local/vje30");
static	TCHAR	sztComD[]	= TEXT("com");
static	TCHAR	sztEnvD[]	= TEXT("env");
static	TCHAR	sztDicD[]	= TEXT("dic");
static	TCHAR	sztUsrD[]	= TEXT(".vje");
static	TCHAR	sztGeneral[]	= TEXT("general");
static	TCHAR	sztIni[]	= TEXT(".ini");
static	TCHAR	sztSty[]	= TEXT(".sty");
static	TCHAR	sztClr[]	= TEXT(".clr");
static	TCHAR	sztRom[]	= TEXT(".rom");
static	TCHAR	sztKey[]	= TEXT(".key");
static	TCHAR	sztAch[]	= TEXT(".ach");
static	TCHAR	sztKch[]	= TEXT(".kch");
static	TCHAR	sztYen[]	= TEXT("/");
#if 0 /* Delete:Set */
static	TCHAR	sztSet[]	= TEXT(".set");
#endif /* Delete:Set */
static	TCHAR	sztDat[]	= TEXT(".dat");
static	TCHAR	sztDic[]	= TEXT(".dic");
static	TCHAR	sztKig[]	= TEXT(".kig");
static	TCHAR	sztTbn[]	= TEXT(".tbn");
static	TCHAR	sztNull[]	= TEXT("");

static	TCHAR	sztUsrDic[]	= TEXT("vjed95u");


/*=============================================================================
	Local	Function
=============================================================================*/
static	int	GetNameR1 (LPTSTR lptFull, LPTSTR lptBase, LPTSTR lptExt);
static	int	GetNameW1 (LPTSTR lptFull, LPTSTR lptBase, LPTSTR lptExt);


/*=============================================================================
	GetUserID
=============================================================================*/
LPSTR	GetUserID ()
{
static	TCHAR	id[MAX_PATH+1];
struct	passwd	*usr_d;
	LPTSTR	p;

	if (id[0]) {
		return ((LPTSTR )id);
	}
	p	= (LPTSTR )getenv ("USER");
	if (p != NULL) {
		lstrncpy (id, p, lstrlen (p));
		return ((LPTSTR )id);
	}
	usr_d	= getpwuid (getuid ());
	if (usr_d != NULL) {
		lstrncpy (id, usr_d->pw_name, lstrlen (usr_d->pw_name));
		return ((LPTSTR )id);
	}
	return ((LPTSTR )0);
}

/*=============================================================================
	GetHomeDir
=============================================================================*/
LPSTR	GetHomeDir ()
{
static	TCHAR	dir[MAX_PATH+1];
struct	passwd	*usr_d;
	LPTSTR	p;

	if (dir[0]) {
		return ((LPTSTR )dir);
	}
	p	= (LPTSTR )getenv ("HOME");
	if (p != NULL) {
		lstrncpy (dir, p, lstrlen (p));
		return ((LPTSTR )dir);
	}
	usr_d	= getpwuid (getuid ());
	if (usr_d != NULL) {
		lstrncpy (dir, usr_d->pw_dir, lstrlen (usr_d->pw_dir));
		return ((LPTSTR )dir);
	}
	return ((LPTSTR )0);
}

/*=============================================================================
	GetUserDir
=============================================================================*/
LPSTR	GetUserDir ()
{
static	TCHAR	dir[MAX_PATH+1];
	LPTSTR	p;

	if (dir[0]) {
		return ((LPTSTR )dir);
	}
	p	= GetHomeDir ();
	if (p != (LPTSTR )0) {
		lstrncpy (dir, p, lstrlen (p));
		lstrcat (dir, sztYen);
		lstrcat (dir, sztUsrD);
		return ((LPTSTR )dir);
	}
	return ((LPTSTR )0);
}

/*=============================================================================
	IsExistEx
=============================================================================*/
int	IsExistEx
(
	LPTSTR fileName
)
{
	return (access (fileName, F_OK) == 0 ? TRUE : FALSE);
}

/*=============================================================================
	IsReadEnable
=============================================================================*/
int	IsReadEnable
(
	LPTSTR	fileName
)
{
	return (access (fileName, R_OK) == 0 ? TRUE : FALSE);
}

/*=============================================================================
	IsWriteEnable
=============================================================================*/
int	IsWriteEnable
(
	LPTSTR	fileName
)
{
	return (access (fileName, W_OK) == 0 ? TRUE : FALSE);
}

/*=============================================================================
	IsDirectory
=============================================================================*/
int	IsDirectory
(
	LPTSTR	fileName
)
{
struct	stat	sts;

	if (stat (fileName, &sts)) {
		return (FALSE);
	}
	return (S_ISDIR (sts.st_mode) ? TRUE : FALSE);
}

/*=============================================================================
	ParseFileName
=============================================================================*/
int	ParseFileName
(
	LPTSTR	fileName,
	LPTSTR	path
)
{
	LPTSTR	lslash;

	lstrcpy (path, fileName);
	lslash	= (LPTSTR )strrchr (path, '/');
	if (lslash == NULL) {
		lstrcpy (path, "./");
	} else if (lslash == path) {
		lstrcpy (path, "/");
	} else {
		*lslash	= '\0';
	}
	return (TRUE);
}

/*=============================================================================
	CopyFile
=============================================================================*/
int	CopyFile
(
	LPTSTR	srcFileName,
	LPTSTR	dstFileName,
	int	failIfExists
)
{
	TCHAR	path[MAX_PATH+1];
	TCHAR	cmd[MAX_PATH+1];
	int	rc;

	if (IsDirectory (srcFileName)
	 || !IsReadEnable (srcFileName)) {
		return (FALSE);
	}
	if (failIfExists != FALSE) {
		if (IsExistEx (dstFileName)) {
			return (FALSE);
		}
	}
	ParseFileName (dstFileName, path);
	if (!IsWriteEnable (path)) {
		return (FALSE);
	}
#if 0
	printf ("copy %s -> %s\n", srcFileName, dstFileName);
#endif
	sprintf (cmd, "cp -f %s %s", srcFileName, dstFileName);
	rc	= system (cmd);
	if (rc != 0) {
		return (FALSE);
	}
	return (TRUE);
}

/*=============================================================================
	MoveFile
=============================================================================*/
int	MoveFile
(
	LPTSTR	srcFileName,
	LPTSTR	dstFileName
)
{
	TCHAR	path[MAX_PATH+1];

	if (IsDirectory (srcFileName)
	 || !IsReadEnable (srcFileName)) {
		return (FALSE);
	}
	if (IsExistEx (dstFileName)) {
		return (FALSE);
	}
	ParseFileName (dstFileName, path);
	if (!IsWriteEnable (path)) {
		return (FALSE);
	}
#if 0
	printf ("rename %s -> %s\n", srcFileName, dstFileName);
#endif
	return (rename (srcFileName, dstFileName) == 0 ? TRUE : FALSE);
}

/*=============================================================================
	DeleteFile
=============================================================================*/
int	DeleteFile
(
	LPTSTR	fileName
)
{
	if (IsDirectory (fileName)
	 || !IsWriteEnable (fileName)) {
		return (FALSE);
	}
#if 0
	printf ("delete %s\n", fileName);
#endif
	return (unlink (fileName) == 0 ? TRUE : FALSE);
}

/*=============================================================================
	CreateFile
=============================================================================*/
HANDLE	CreateFile
(
	LPTSTR	file,
	int	flag,
	mode_t	mode
)
{
static	HANDLE	fd;

	if (mode) {
		fd	= open (file, flag, mode);
	} else {
		fd	= open (file, flag);
	}
	if (fd < 0) {
		return (INVALID_HANDLE_VALUE);
	}
	return	((HANDLE )fd);
}

/*=============================================================================
	ReadFile
=============================================================================*/
int	ReadFile
(
	HANDLE	fd,
	LPVOID	buf,
	LPDWORD	size
)
{
	int	sz;

	sz	= read (fd, buf, *size);
	if (sz < 0) {
		return (FALSE);
	}
	*size	= sz;
	return (TRUE);
}

/*=============================================================================
	WriteFile
=============================================================================*/
int	WriteFile
(
	HANDLE	fd,
	LPVOID	buf,
	LPDWORD	size
)
{
	int	sz;

	sz	= write (fd, buf, *size);
	if (sz < 0) {
		return (FALSE);
	}
	*size	= sz;
	return (TRUE);
}

/*=============================================================================
	CloseHandle
=============================================================================*/
int	CloseHandle
(
	HANDLE	fd
)
{
	return (close (fd) == 0 ? TRUE : FALSE);
}

/*=============================================================================
	FindNextFile
=============================================================================*/
int	FindNextFile
(
	LPDIR		dirHandle,
	LPDIRENT	findData
)
{
	LPDIRENT	fd;

	fd	= readdir (dirHandle);
	if (fd == NULL) {
		return (FALSE);
	}
	lmemcpy ((LPSTR )findData, (LPSTR )fd, (int )sizeof (DIRENT));
	return (TRUE);
}

/*=============================================================================
	FindFirstFile
=============================================================================*/
LPDIR	FindFirstFile
(
	LPTSTR		fileName,
	LPDIRENT	findData
)
{
	TCHAR	path[MAX_PATH+1];
	LPDIR	dir;
	int	rc;

	if (!IsDirectory (fileName)) {
		ParseFileName (fileName, path);
	} else {
		lstrcpy (path, fileName);
	}
	if (!IsReadEnable (path)) {
		return ((LPDIR )0);
	}
	dir	= opendir (path);
	if (dir == NULL) {
		return ((LPDIR )0);
	}
	rc	= FindNextFile (dir, findData);
	if (!rc) {
		return ((LPDIR )0);
	}
	return ((LPDIR )dir);
}

/*=============================================================================
	FindClose
=============================================================================*/
int	FindClose
(
	LPDIR	dirHandle
)
{
	return (closedir (dirHandle) == 0 ? TRUE : FALSE);
}

/*=============================================================================
	CreateDirectory
=============================================================================*/
int	CreateDirectory
(
	LPTSTR	dirName
)
{
	TCHAR	path[MAX_PATH+1];
	TCHAR	path2[MAX_PATH+1];
	TCHAR	cmd[MAX_PATH+1];
	int	rc;

	if (IsExistEx (dirName)) {
		return (FALSE);
	}
	ParseFileName (dirName, path);
	if (*(dirName + lstrlen (dirName) - 1) == TEXT ('/')) {
		ParseFileName (path, path2);
	} else {
		lstrcpy (path2, path);
	}
	if (!IsWriteEnable (path2)) {
		return (FALSE);
	}
#if 0
	printf ("mkdir %s\n", dirName);
#endif
	sprintf (cmd, "mkdir %s", dirName);
	rc	= system (cmd);
	if (rc != 0) {
		return (FALSE);
	}
	return (TRUE);
}

/*=============================================================================
	IsIncludePath
=============================================================================*/
int	IsIncludePath
(
	LPTSTR lptBase
)
{
	int	rc = FALSE;
	int	len;
	int	i;

	len	= lstrlen (lptBase);
	for (i=0; i<len; i++) {
		if (IsDBCS (lptBase[i])) {
			i++;
		} else if (lptBase[i] == TEXT ('/')) {
			return (TRUE);
		}
	}
	return (FALSE);
}

/*=============================================================================
	SearchFile
=============================================================================*/
DWORD	SearchFile
(
	LPTSTR lptDir,
	LPTSTR lptFile
)
{
	TCHAR	fileName[MAX_PATH+1];

	lstrcpy (fileName, lptDir);
	lstrcat (fileName, lptFile);
	if (IsDirectory (fileName)) {
		return (FILE_ATTRIBUTE_DIRECTORY);
	}
	return (FILE_ATTRIBUTE_NORMAL);
}

/*=============================================================================
	IsExist
=============================================================================*/
int	IsExist
(
	LPTSTR lptFile
)
{
	if (IsDirectory (lptFile)
	 || !IsReadEnable (lptFile)) {
		return (FALSE);
	}
	return (TRUE);
}

/*=============================================================================
	GetEnv
=============================================================================*/
LPENV	WINAPI	GetEnv
(
	int	iMode
)
{
static	ENV	envO;	/* ưδĶǡ	*/
static	ENV	envC;	/* ߤδĶǡ	*/

	return (iMode ? &envO : &envC);
}

/*=============================================================================
	LoadEnv
=============================================================================*/
int	WINAPI	LoadEnv
(
	LPENV		lpEnv,
	LPENV		lpEnvO
)
{
	LPENVCLR	clr;
	LPENVKBD	kbd;
	LPENVROM	rom;
#if 0 /* Delete:Set */
	LPENVSET	set;
#endif /* Delete:Set */
	TCHAR		sztTemp[MAX_PATH + 1];

	InLog(__FILE__ "/LoadEnv");
	clr	= &lpEnv->clr;
	kbd	= &lpEnv->kbd;
	rom	= &lpEnv->rom;
#if 0 /* Delete:Set */
	set	= &lpEnv->set;
#endif /* Delete:Set */
	/*
		Sty File
	*/
	GetNameR1(sztTemp,sztNull,sztSty);
	LoadStyFile(lpEnv,sztTemp);
	/*
		Clr File
	*/
	clr->wClrFrom	= GetNameR1(sztTemp,clr->sztClrFile,sztClr);
	LoadClrFile(lpEnv,sztTemp);
	clr->wClrUpdate	= NO;
	/*
		Key File
	*/
	kbd->wKeyFrom	= GetNameR1(sztTemp,kbd->sztKeyFile,sztKey);
	LoadKeyFile(lpEnv,sztTemp);
	kbd->wKeyUpdate	= NO;
	/*
		Ach File
	*/
	kbd->wAchFrom	= GetNameR1(sztTemp,kbd->sztAchFile,sztAch);
	LoadAchFile(lpEnv,sztTemp);
	kbd->wAchUpdate	= NO;
	/*
		Kch File
	*/
	kbd->wKchFrom	= GetNameR1(sztTemp,kbd->sztKchFile,sztKch);
	LoadKchFile(lpEnv,sztTemp);
	kbd->wKchUpdate	= NO;
	/*
		Rom File
	*/
	rom->wRomFrom	= GetNameR1(sztTemp,rom->sztRomFile,sztRom);
	LoadRomFile(lpEnv,sztTemp);
	rom->wRomUpdate	= NO;
	/*
		Set File
	*/
#if 0 /* Delete:Set */
	set->wSetFrom	= GetNameR1(sztTemp,set->sztSetFile,sztSet);
	LoadSetFile(lpEnv,sztTemp);
	set->wSetUpdate	= NO;
#endif /* Delete:Set */
	/*
		Check
	*/
	MakeDefaultEnv(lpEnvO);
	CheckEnv(lpEnv,lpEnvO);
	*lpEnvO	= *lpEnv;
	OutLog(__FILE__ "/LoadEnv");
	return	TRUE;
}

/*=============================================================================
	SaveRomaEnv
=============================================================================*/
int	WINAPI	SaveRomaEnv
(
	LPENV	lpEnv
)
{
	TCHAR		sztFile[MAX_PATH + 1];
	LPTSTR		lptRoma;

	GetNameW1(sztFile,sztNull,sztSty);
	lptRoma = (lpEnv->mod.wRoman == YES) ? TEXT("YES") : TEXT("NO");
	WritePPString (TEXT("Mode"),TEXT("Roman"),lptRoma,sztFile);
	return	TRUE;
}

/*=============================================================================
	SaveEnv
=============================================================================*/
int	WINAPI	SaveEnv
(
	LPENV		lpEnv,
	LPENV		lpEnvO
)
{
	LPENVINI	ini,iniO;
	TCHAR		sztF[MAX_PATH + 1];
	TCHAR		sztG[MAX_PATH + 1];
	TCHAR		sztH[MAX_PATH + 1];
	int		iFrom;
	int		i,j,k;

	InLog(__FILE__ "/SaveEnv");
	ini	= &lpEnv->ini;
	iniO	= &lpEnvO->ini;
	if	( *ini->sztInsDir ) {
		/*
			Sty File
		*/
		if	( memcmp(lpEnv,lpEnvO,sizeof(ENV)) ) {
			GetNameW1(sztF,sztNull,sztSty);
#ifdef	DEBUG_ENV
			dprintf("Delete %s\n",sztF);
#endif
			DeleteFile(sztF);
			SaveStyFile(lpEnv,sztF);
		}
		/*
			Clr File
		*/
		if	( lpEnv->clr.wClrUpdate != NO ) { /* YES or RELOAD */
			iFrom	= GetNameW1(sztF,lpEnv->clr.sztClrFile,sztClr);
			if	( lpEnv->clr.wClrFrom	== LOAD_USER	) {
				if	( IsExist(sztF) ) {
#ifdef	DEBUG_ENV
					dprintf("Delete %s\n",sztF);
#endif
					DeleteFile(sztF);
				}
				lpEnv->clr.wClrFrom	= LOAD_PROG;
			}
			if	( lpEnv->clr.wClrUpdate == YES		) {
				SaveClrFile(lpEnv,sztF);
				lpEnv->clr.wClrFrom	= iFrom;
			}
			lpEnv->clr.wClrUpdate	= NO;
#if	VERSION == 25
			lpEnv->clr.wClrFrom	= GetNameR1(
							sztF,
							lpEnv->clr.sztClrFile,
							sztClr
						);
#endif
		}
		/*
			Key File
		*/
		if	( lpEnv->kbd.wKeyUpdate != NO ) { /* YES or RELOAD */
			iFrom	= GetNameW1(sztF,lpEnv->kbd.sztKeyFile,sztKey);
			if	( lpEnv->kbd.wKeyFrom	== LOAD_USER	) {
				if	( IsExist(sztF) ) {
#ifdef	DEBUG_ENV
					dprintf("Delete %s\n",sztF);
#endif
					DeleteFile(sztF);
				}
				lpEnv->kbd.wKeyFrom	= LOAD_PROG;
			}
			if	( lpEnv->kbd.wKeyUpdate == YES		) {
				SaveKeyFile(lpEnv,sztF);
				lpEnv->kbd.wKeyFrom	= iFrom;
			}
			lpEnv->kbd.wKeyUpdate	= NO;
#if	VERSION == 25
			lpEnv->kbd.wKeyFrom	= GetNameR1(
							sztF,
							lpEnv->kbd.sztKeyFile,
							sztKey
						);
#endif
		}
		/*
			Ach File
		*/
		if	( lpEnv->kbd.wAchUpdate != NO ) { /* YES or RELOAD */
			iFrom	= GetNameW1(sztF,lpEnv->kbd.sztAchFile,sztAch);
			if	( lpEnv->kbd.wAchFrom   == LOAD_USER	) {
				if	( IsExist(sztF) ) {
#ifdef	DEBUG_ENV
					dprintf("Delete %s\n",sztF);
#endif
					DeleteFile(sztF);
				}
				lpEnv->kbd.wAchFrom	= LOAD_PROG;
			}
			if	( lpEnv->kbd.wAchUpdate == YES		) {
				SaveAchFile(lpEnv,sztF);
				lpEnv->kbd.wAchFrom	= iFrom;
			}
			lpEnv->kbd.wAchUpdate	= NO;
#if	VERSION == 25
			lpEnv->kbd.wAchFrom	= GetNameR1(
							sztF,
							lpEnv->kbd.sztAchFile,
							sztAch
						);
#endif
		}
		/*
			Kch File
		*/
		if	( lpEnv->kbd.wKchUpdate != NO ) { /* YES or RELOAD */
			iFrom	= GetNameW1(sztF,lpEnv->kbd.sztKchFile,sztKch);
			if	( lpEnv->kbd.wKchFrom   == LOAD_USER	) {
				if	( IsExist(sztF) ) {
#ifdef	DEBUG_ENV
					dprintf("Delete %s\n",sztF);
#endif
					DeleteFile(sztF);
				}
				lpEnv->kbd.wKchFrom	= LOAD_PROG;
			}
			if	( lpEnv->kbd.wKchUpdate == YES		) {
				SaveKchFile(lpEnv,sztF);
				lpEnv->kbd.wKchFrom	= iFrom;
			}
			lpEnv->kbd.wKchUpdate	= NO;
#if	VERSION == 25
			lpEnv->kbd.wKchFrom	= GetNameR1(
							sztF,
							lpEnv->kbd.sztKchFile,
							sztKch
						);
#endif
		}
		/*
			Rom File
		*/
		if	( lpEnv->rom.wRomUpdate != NO ) { /* YES or RELOAD */
			iFrom	= GetNameW1(sztF,lpEnv->rom.sztRomFile,sztRom);
			if	( lpEnv->rom.wRomFrom	== LOAD_USER	) {
				if	( IsExist(sztF) ) {
#ifdef	DEBUG_ENV
					dprintf("Delete %s\n",sztF);
#endif
					DeleteFile(sztF);
				}
				lpEnv->rom.wRomFrom	= LOAD_PROG;
			}
			if	( lpEnv->rom.wRomUpdate == YES		) {
				SaveRomFile(lpEnv,sztF);
				lpEnv->rom.wRomFrom	= iFrom;
			}
			lpEnv->rom.wRomUpdate	= NO;
#if	VERSION == 25
			lpEnv->rom.wRomFrom	= GetNameR1(
							sztF,
							lpEnv->rom.sztRomFile,
							sztRom
						);
#endif
		}
		/*
			Set File
		*/
#if 0 /* Delete:Set */
		if	( lpEnv->set.wSetUpdate	!= NO ) { /* YES or RELOAD */
			iFrom	= GetNameW1(sztF,lpEnv->set.sztSetFile,sztSet);
#if	0							/* -960902 */
#ifdef	DEBUG_ENV
			dprintf("Delete %s\n",sztF);
#endif
			DeleteFile(sztF);
			SaveSetFile(lpEnv,sztF);
			lpEnv->set.wSetFrom	= iFrom;
#else								/* +960902 */
			if	( lpEnv->set.wSetFrom	== LOAD_USER	) {
#ifdef	DEBUG_ENV
				dprintf("Delete %s\n",sztF);
#endif
				DeleteFile(sztF);
				lpEnv->set.wSetFrom	= LOAD_PROG;
			}
			if	( lpEnv->set.wSetUpdate == YES		) {
				SaveSetFile(lpEnv,sztF);
				lpEnv->set.wSetFrom	= iFrom;
			}
#endif
			lpEnv->set.wSetUpdate	= NO;
		}
#endif /* Delete:Set */
		FlushPP();
	}
	*lpEnvO	= *lpEnv;
done:
	OutLog(__FILE__ "/SaveEnv");
	return	TRUE;
}

/*=============================================================================
	LoadCfgFile
=============================================================================*/
LPSTR	LoadCfgFile
(
	LPENV		lpEnv,
	LPTSTR		lptFile
)
{
	LPTSTR		k1, k2;
	LPENVINI	ini;
	LPTSTR		lptOut;

	k1		= lptFile;
	ini		= &lpEnv->ini;

	k2		= sztVje;
	lptOut		= GPStr (k1, k2, TEXT ("Path"), sztNull);
	if (!IsDirectory (lptOut)) {
		lstrxcat (ini->sztInsDir, sztInsD, sztYen, NULL);
	} else {
		if (*(lptOut + lstrlen (lptOut) - 1) == TEXT ('/')) {
			lstrcpy (ini->sztInsDir, lptOut);
		} else {
			lstrxcat (ini->sztInsDir, lptOut, sztYen, NULL);
		}
	}

	k2		= sztAtw;
	lptOut		= GPStr (k1, k2, TEXT ("Title"), sztNull);
	lstrcpy (ini->sztAtwTitle, lptOut);
}

/*=============================================================================
	GetBasicInfo
=============================================================================*/
int	GetBasicInfo
(
	LPENV		lpEnv
)
{
	LPENVINI	ini;
	TCHAR		sztTemp[MAX_PATH + 1];

	InLog (__FILE__ "/GetBasicInfo");
	ini = &lpEnv->ini;

	/*
		User ID
	*/
	lstrcpy (ini->sztUsr, GetUserID ());

	/*
		Etc Dir
	*/
	lstrxcat (ini->sztEtcDir, sztEtcD, sztYen, NULL);

	/*
		Cfg File
	*/
	lstrxcat (sztTemp, ini->sztEtcDir, sztCfgF, NULL);
	LoadCfgFile (lpEnv, sztTemp);

	lstrxcat (ini->sztComDir, ini->sztInsDir, sztComD, sztYen, NULL);
	lstrxcat (ini->sztEnvDir, ini->sztInsDir, sztEnvD, sztYen, NULL);
	lstrxcat (ini->sztDicDir, ini->sztInsDir, sztDicD, sztYen, NULL);
	lstrxcat (ini->sztUsrDir, GetUserDir (), sztYen, NULL);
	lstrcpy (ini->sztDatBase, sztGeneral);

	OutLog (__FILE__ "/GetBasicInfo");
	return (TRUE);
}

/*=============================================================================
	MakeNewDic
=============================================================================*/
int	MakeNewDic
(
	LPTSTR		oDic
)
{
	TCHAR		sztSrc[MAX_PATH+1];
	LPENV		lpEnv;
	LPENVINI	ini;
	int		ret = 0;

	lpEnv		= GetEnv(0);
	ini		= &lpEnv->ini;

	lstrxcat (sztSrc, ini->sztComDir, sztUsrDic, sztDic, NULL);
	CopyFile (sztSrc, oDic, 0);

	return (ret);
}

/*=============================================================================
	BuildDicName
=============================================================================*/
int	WINAPI	BuildDicName
(
	LPTSTR		lptFull,
	LPTSTR		lptBase,
	WORD		wDicMode
)
{
	LPENV		lpEnv;
	LPENVINI	ini;
	int		ret;

	lpEnv	= GetEnv(0);
	ini	= &lpEnv->ini;
	ret	= 0;
	while	(1) {
		if	( IsIncludePath(lptBase) ) {
			lstrcpy(lptFull,lptBase);
			if	( IsExist(lptFull) ) {
				ret	= LOAD_FULL;
			}
			else	{
				if	( wDicMode & MOD_DIC_FOR_GAKU ) {
					MakeNewDic(lptFull);
					ret	= LOAD_FULL;
				}
			}
			break;
		}
		lstrxcat(lptFull,ini->sztUsrDir,lptBase,NULL);
		if	( IsExist(lptFull) ) {
			ret	= LOAD_USER;
			break;
		}
		lstrxcat(lptFull,ini->sztDicDir,lptBase,NULL);
		if	( IsExist(lptFull) ) {
			ret	= LOAD_PROG;
			break;
		}
		if	( wDicMode & MOD_DIC_FOR_GAKU ) {
			lstrxcat(lptFull,ini->sztUsrDir,lptBase,NULL);
			MakeNewDic(lptFull);
			ret	= LOAD_USER;
		}
		break;
	}
	return	ret;
}

/*=============================================================================
	GetDicTitle
=============================================================================*/
void	GetDicTitle
(
	LPTSTR	lptDesc,
	LPTSTR	lptFile
)
{
	HANDLE	hFile;
	HEADER	header;
	DWORD	dwT;

	*lptDesc	= (TCHAR )0;
	hFile		= CreateFile (lptFile, O_RDONLY,0);
	if (hFile != INVALID_HANDLE_VALUE) {
		dwT	= sizeof (HEADER);
		ReadFile (hFile, (LPVOID) &header, &dwT);
		if (dwT == (DWORD )sizeof (HEADER)) {
			lstrcpy (
				lptDesc,
				(char *)sjis2euc ((char *)header.title)
			);
		}
		CloseHandle (hFile);
	}
	return;
}

/*=============================================================================
	OpenVjeFiles
=============================================================================*/
void	OpenVjeFiles
(
#if 1 /* 20000214 ROM-FILE */
	FXim_Client	*client,
#else
	HBASEDT		bebuf,
	PBD		bdata,
#endif
	LPENV		lpEnv
)
{
#if 1 /* 20000214 ROM-FILE */
	PBD		bdata;
	HBASEDT		bebuf;
#endif
	TCHAR		sztFile[MAX_PATH + 1];
	LPENVDIC	dic;
	WORD		wMode;
	int		i;

#if 1 /* 20000214 ROM-FILE */
	bdata	= (PBD     )GetBDATA (client);
	bebuf	= (HBASEDT )GetBEBUF (client);
#endif

	bdata->wVkey = F_INIT;
	BasicEdit (bebuf, bdata);
	/*
		Dic
	*/
	for	( i = 0 ; i < MAX_DIC ; i++ ) {
		dic		= &lpEnv->dic[i];
		if	(!dic->sztDicFile[0]) continue;
		wMode	= 0;
		if	( dic->wLearnMode == YES    ) {
			wMode	|= MOD_GAKU_BUN|MOD_GAKU_DON;
		}
		else if	( dic->wLearnMode == DON    ) {
			wMode	|= MOD_GAKU_DON;
		}
		if	( dic->wLearnType == MYSELF ) {
			wMode	|= MOD_GAKU_MYSELF;
		}
		if	( dic->wParallel  == YES    ) {
			wMode	|= MOD_PARA_ACCESS;
		}
		if	( i == (MAX_DIC - 1) ) {
			wMode	|= MOD_DIC_FOR_GAKU;
		}
		if	(BuildDicName (sztFile, dic->sztDicFile, wMode)) {

			GetDicTitle (dic->sztDicDesc, sztFile);
	
			bdata->wVkey		= F_DIC_OPEN;
			bdata->wOptDicNo	= i;
			bdata->wOptDicMode	= wMode;
			bdata->lpchDicName	= sztFile;
			BasicEdit (bebuf, bdata);
		}
	}
#if 1 /* 20000214 ROM-FILE */
	if (client == root_client) {
#endif
		/*
			Rom
		*/
		bdata->wVkey		= F_ROM_OPEN;
		GetNameR1 (sztFile, lpEnv->rom.sztRomFile, sztRom);
		bdata->lpchRomFlName	= sztFile;
		bdata->lpchRomTbName	= TEXT("ROMA");
		BasicEdit (bebuf, bdata);
		/*
			Lim
		*/
		bdata->wVkey		= F_LIM_OPEN;
		GetNameR1 (sztFile, lpEnv->dci.sztDicDat, sztDat);
		bdata->lpchDicName	= sztFile;
		BasicEdit (bebuf, bdata);
#if 1 /* 20000214 ROM-FILE */
	}
#endif
	return ;
}

/*=============================================================================
	IsDicAlive
=============================================================================*/
BOOL	IsDicAlive
(
	WORD	wDicNo
)
{
	LPENV	lpEnv;

	lpEnv	= (LPENV )GetEnv (0);
	return ((BOOL)(wDicNo < MAX_DIC && lpEnv->dic[wDicNo].sztDicFile[0]));
}

/*=============================================================================
	CloseVjeFiles
=============================================================================*/
void	CloseVjeFiles
(
#if 1 /* 20000214 ROM-FILE */
	FXim_Client	*client
#else
	HBASEDT		bebuf,
	PBD		bdata
#endif
)
{
#if 1 /* 20000214 ROM-FILE */
	PBD		bdata;
	HBASEDT		bebuf;
#endif
	int		i;

#if 1 /* 20000214 ROM-FILE */
	bdata	= (PBD     )GetBDATA (client);
	bebuf	= (HBASEDT )GetBEBUF (client);
#endif

#if 1 /* 20000214 ROM-FILE */
	if (client == root_client) {
#endif
		bdata->wVkey	= F_LIM_CLOSE;
		BasicEdit (bebuf, bdata);
		bdata->wVkey	= F_ROM_CLOSE;
		BasicEdit (bebuf, bdata);
#if 1 /* 20000214 ROM-FILE */
	}
#endif

	for (i=0; i < MAX_DIC; i++) {
		bdata->wVkey		= F_DIC_CLOSE;
		bdata->wOptDicNo	= i;
		BasicEdit (bebuf, bdata);
	}
	return ;
}

/*=============================================================================
	CopyDirectory
=============================================================================*/
int	CopyDirectory
(
	LPTSTR		lptSrc,
	LPTSTR		lptDes,
	int		failIfExists
)
{
	TCHAR		sztTmp1[MAX_PATH+1];
	TCHAR		sztTmp2[MAX_PATH+1];
	LPDIR		hFind;
	DIRENT		fd;
	LPTSTR		lptExt;
	int		i;

	InLog (__FILE__ "/CopyDirectory");
	lstrxcat (sztTmp1, lptSrc, TEXT ("*.*"), NULL);
	hFind	= FindFirstFile (sztTmp1, &fd);
	if (hFind != (LPDIR )0) {
		do {
			i	= lstrlen (fd.d_name);
			i	= (i>4) ? (i-4) : 0;
			lptExt	= &fd.d_name[i];
#if !PCUNIX
			if( !lstrcmpi (lptExt, sztSty)
			 || !lstrcmpi (lptExt, sztClr)
			 || !lstrcmpi (lptExt, sztKey)
			 || !lstrcmpi (lptExt, sztAch)
			 || !lstrcmpi (lptExt, sztKch)
			 || !lstrcmpi (lptExt, sztRom)
			 || !lstrcmpi (lptExt, sztSet)
			 || !lstrcmpi (lptExt, sztDat)
			 || !lstrcmpi (lptExt, sztDic)
			 || !lstrcmpi (lptExt, sztIni)
			 || !lstrcmpi (lptExt, sztKig)
			 || !lstrcmpi (lptExt, sztTbn)) {
#else /* !PCUNIX */
			if( !lstrcmp (lptExt, sztSty)
			 || !lstrcmp (lptExt, sztClr)
			 || !lstrcmp (lptExt, sztKey)
			 || !lstrcmp (lptExt, sztAch)
			 || !lstrcmp (lptExt, sztKch)
			 || !lstrcmp (lptExt, sztRom)
#if 0 /* Delete:Set */
			 || !lstrcmp (lptExt, sztSet)
#endif /* Delete:Set */
			 || !lstrcmp (lptExt, sztDat)
			 || !lstrcmp (lptExt, sztDic)
			 || !lstrcmp (lptExt, sztIni)
			 || !lstrcmp (lptExt, sztKig)
			 || !lstrcmp (lptExt, sztTbn)) {
#endif /* !PCUNIX */
				lstrxcat (sztTmp1, lptSrc, fd.d_name, NULL);
				lstrxcat (sztTmp2, lptDes, fd.d_name, NULL);
				CopyFile (sztTmp1, sztTmp2, failIfExists);
			}
		} while (FindNextFile (hFind, &fd));
		FindClose (hFind);
	}
	OutLog (__FILE__ "/CopyDirectory");
	return (TRUE);
}

/*=============================================================================
	GetNameR1
=============================================================================*/
static	int	GetNameR1
(
	LPTSTR		lptFull,
	LPTSTR		lptBase,
	LPTSTR		lptExt
)
{
	LPENV		lpEnv;
	LPENVINI	ini;
	int		ret;

	InLog(__FILE__ "/GetNameR1");
	lpEnv	= GetEnv(0);
	ini	= &lpEnv->ini;
	do	{
		if	( IsIncludePath(lptBase) ) {
			lstrcpy(lptFull,lptBase);
			ret	= LOAD_FULL;
			break;
		}
#if	VERSION == 25
		/* 桼ξ */
		lstrxcat(lptFull,ini->sztUsrDir,lptBase,NULL);
		if	( IsExist(lptFull) ) {
			ret	= LOAD_USER;
			break;
		}
#endif
		lstrxcat(lptFull,ini->sztUsrDir,ini->sztDatBase,lptExt,NULL);
		if	( IsExist(lptFull) ) {
			ret	= LOAD_USER;
			break;
		}
		lstrxcat(lptFull,ini->sztEnvDir,lptBase,NULL);
		if	( IsExist(lptFull) ) {
			ret	= LOAD_PROG;
			break;
		}
#if 1
		lstrxcat(lptFull,ini->sztDicDir,lptBase,NULL);
		if	( IsExist(lptFull) ) {
			ret	= LOAD_PROG;
			break;
		}
#endif
		lstrxcat(lptFull,ini->sztUsrDir,ini->sztDatBase,lptExt,NULL);
		ret	= LOAD_USER;
	}	while(0);
	OutLog(__FILE__ "/GetNameR1");
	return	ret;
}

/*=============================================================================
	GetNameW1
=============================================================================*/
static	int	GetNameW1
(
	LPTSTR		lptFull,
	LPTSTR		lptBase,
	LPTSTR		lptExt
)
{
	LPENV		lpEnv;
	LPENVINI	ini;
	int		ret;

	InLog(__FILE__ "/GetNameW1");
	lpEnv	= GetEnv(0);
	ini	= &lpEnv->ini;
	if	( IsIncludePath(lptBase) ) {
		lstrcpy(lptFull,lptBase);
		ret	= LOAD_FULL;
	}
	else	{
		lstrxcat(lptFull,ini->sztUsrDir,ini->sztDatBase,lptExt,NULL);
		ret	= LOAD_USER;
	}
	OutLog(__FILE__ "/GetNameW1");
	return	ret;
}

/*=============================================================================
	IsCharVirt
=============================================================================*/
int	WINAPI	IsCharVirt
(
	WORD	wVirt
)
{
static	WORD	wCharVirt[]	= {
		XK_KP_Divide,	XK_KP_Multiply,	XK_KP_Subtract,	XK_KP_Add,
		XK_KP_Equal,	XK_KP_Separator,XK_KP_Decimal,
		XK_exclam,	XK_quotedbl,	XK_numbersign,	XK_dollar,
		XK_percent,	XK_ampersand,	XK_apostrophe,	XK_parenleft,
		XK_parenright,	XK_asciitilde,
		XK_minus,	XK_asciicircum,
#if DEF_BACKSLASH_AND_YEN
		XK_yen,
#endif
		XK_at,		XK_bracketleft,	XK_semicolon,	XK_colon,
		XK_bracketright,XK_comma,	XK_period,	XK_slash,
		XK_backslash,
		XK_equal,	XK_brokenbar,	XK_bar,		XK_grave,
		XK_braceleft,	XK_plus,	XK_asterisk,	XK_braceright,
		XK_less,	XK_greater,	XK_question,	XK_underscore,
		};
	LPENV	lpEnv;
	int	i;
	int	ret;

	ret	= FALSE;
	lpEnv	= GetEnv(0);
	wVirt	= ConvVK (wVirt);
	if	( (XK_0    <= wVirt && wVirt <= XK_9    )
	||	  (XK_a    <= wVirt && wVirt <= XK_z    )
	||	  (XK_A    <= wVirt && wVirt <= XK_Z    )
	||	  (XK_KP_0 <= wVirt && wVirt <= XK_KP_9 ) ) {
		ret	= TRUE;
	}
	else	{
		for	( i = 0 ; i < recof(wCharVirt) ; i++ ) {
			if	( wCharVirt[i] == wVirt ) {
				ret	= TRUE;
				break;
			}
		}
	}
	return	ret;
}

/*=============================================================================
	CheckUserEnv
=============================================================================*/
int	CheckUserEnv
(
	LPENV		lpEnv
)
{
	TCHAR		sztGen[MAX_PATH+1];
	LPENVINI	ini;

	ini	= &lpEnv->ini;
	lstrxcat (sztGen, ini->sztUsrDir, sztGeneral, sztSty, NULL);
	if (!IsExist (sztGen)) {
		CreateDirectory (ini->sztUsrDir);
		CopyDirectory (ini->sztComDir, ini->sztUsrDir, TRUE);
	}
	if (!IsExist (sztGen)) {
		return (FALSE);
	}
	return (TRUE);
}

