/******************************************************************************
	strutl.c
******************************************************************************/
#if !PCUNIX
#include	"define.h"
#ifndef	DPMI
#include	<windows.h>
#include	<stdlib.h>				/* for qsort,bsearch */
#else
#include	"typedef.h"
#include	"vjeio.h"
#include	"xmacro.h"
#endif
#include	<stdarg.h>				/* for va_*	     */
#include	"strutl.h"
#else /* !PCUNIX */
#include	<stdlib.h>
#include	<stdarg.h>
#include	"common.h"
#include	"typedef.h"
#include	"vjeenv.h"
#include	"macro.h"
#include	"fileio.h"
#include	"strutl.h"
#endif /* PCUNIX */

/*=============================================================================
	define/typedef
=============================================================================*/
#if PCUNIX
#undef	CR
#undef	LF
#endif /* PCUNIX */
#if	!USE_WINAPI
#ifndef	DPMI
#define	MAX_BUFF	16384
#define	MAX_LINE	1024
#else
#define	MAX_BUFF	8192
#define	MAX_LINE	512
#endif
#define	CR		TEXT('\r')
#define	LF		TEXT('\n')
#define	SPACE		TEXT(' ' )
#define	EQ		TEXT('=' )
#define	OF_XREADWRITE	(OF_READWRITE|OF_SHARE_EXCLUSIVE)
#endif
#define	recof(x)	((sizeof x)/(sizeof x[0]))

/*=============================================================================
	local	data
=============================================================================*/
#if	!USE_WINAPI
static	TCHAR		chFileName[MAX_PATH + 1];
static	TCHAR		chSectName[MAX_PATH + 1];
static	TCHAR	FAR	sztBuf	  [MAX_BUFF + 1];
static	int	FAR	inoLin	  [MAX_LINE + 1];
static	LPINT	FAR	lpiSrt    [MAX_LINE + 1];
static	int	lenSrt;
#define	sizSrt	(sizeof(LPINT))
static	int	bUpdate	= FALSE;
static	int	lenBuf	= 0;
static	int	lenLin;
static	int	begLin;
static	int	endLin;
#endif
static	TCHAR	sztNull[]	= TEXT("");

/******************************************************************************
	for [Get|Write]PrivateProfileString Compatible Function
******************************************************************************/
/*=============================================================================
	keylen
=============================================================================*/
static	int	keylen(LPTSTR lptStr)
{
	int	ret;

	ret	= 0;
	while	( *lptStr != TEXT('\0')
	&&	  *lptStr != TEXT('=' )
	&&	  *lptStr != TEXT('\r')
	&&	  *lptStr != TEXT('\n') ) {
		ret++;
		lptStr++;
	}
	return	ret;
}

/*=============================================================================
	lpicmpq		compare for qsort
=============================================================================*/
#if	!USE_WINAPI && !defined(DPMI)
int	_cdecl	lpicmpq(const void *arg1,const void *arg2)
{
	LPTSTR	lptBuf1,lptBuf2;
	int	lenBuf1,lenBuf2;
	int	ret;
	int	n;

	lptBuf1	= &sztBuf[**(LPINT FAR *)arg1];
	lptBuf2	= &sztBuf[**(LPINT FAR *)arg2];
	lenBuf1	= keylen(lptBuf1);
	lenBuf2	= keylen(lptBuf2);
	n	= min(lenBuf1,lenBuf2);
#if !PCUNIX
	ret	= lstrncmpi(lptBuf1,lptBuf2,n);
#else /* !PCUNIX */
	ret	= lstrncmp(lptBuf1,lptBuf2,n);
#endif /* !PCUNIX */
	if	( !ret ) {
		ret	= lenBuf1 - lenBuf2;
	}
	return	ret;
}
#endif

/*=============================================================================
	lpicmpb		compare for bsearch
=============================================================================*/
#if	!USE_WINAPI
int	_cdecl	lpicmpb(const void *arg1,const void *arg2)
{
	LPTSTR	lptKey,lptBuf;
	int	lenKey,lenBuf;
	int	ret;
	int	n;

	lptKey	= (LPTSTR) arg1;
	lptBuf	= &sztBuf[**(LPINT FAR *)arg2];
	lenKey	= lstrlen(lptKey);
	lenBuf	= keylen(lptBuf);
	n	= min(lenKey,lenBuf);
#if !PCUNIX
	ret	= lstrncmpi(lptKey,lptBuf,n);
#else /* !PCUNIX */
	ret	= lstrncmp(lptKey,lptBuf,n);
#endif /* !PCUNIX */
	if	( !ret ) {
		ret	= lenKey - lenBuf;
	}
	return	ret;
}
#endif

/*=============================================================================
	FlushBuff
=============================================================================*/
#if	!USE_WINAPI && !defined(DPMI)
static	void	FlushBuff(void)
{
	HANDLE	hFile;
	DWORD	dwT;

	while	( bUpdate ) {
		bUpdate	= FALSE;
#if !PCUNIX
		hFile	= CreateFile(
				chFileName,
				GENERIC_WRITE,
				0,
				(LPSECURITY_ATTRIBUTES) NULL,
				OPEN_ALWAYS,
				FILE_FLAG_SEQUENTIAL_SCAN|
				FILE_ATTRIBUTE_NORMAL,
				(HANDLE) NULL
			);
#else /* !PCUNIX */
		hFile	= CreateFile (chFileName,O_CREAT|O_WRONLY|O_TRUNC,0644);
#endif /* !PCUNIX */
		if	( hFile == INVALID_HANDLE_VALUE ) {
			break;
		}
#if !PCUNIX
		WriteFile(hFile,(LPVOID)sztBuf,(DWORD)lenBuf,&dwT,NULL);
#else /* !PCUNIX */
		dwT = lenBuf;
		WriteFile (hFile, (LPVOID )sztBuf, &dwT);
#endif /* !PCUNIX */
		SetEndOfFile(hFile);
		CloseHandle(hFile);
	}
	return;
}
#endif

/*=============================================================================
	FlushBuff
=============================================================================*/
#if	!USE_WINAPI && defined(DPMI)
static	void	FlushBuff(void)
{
	short	fd;

	while	( bUpdate ) {
		bUpdate	= FALSE;
		fd	= LowOpen(chFileName,O_RDWR|O_BINARY);
		if	( fd == -1 ) {
			break;
		}
		LowWrite(fd,sztBuf,lenBuf);
		LowWrite(fd,sztBuf,0);			/* Adjust File Size */
		LowClose(fd);
	}
	return;
}
#endif

/*=============================================================================
	LoadBuff
=============================================================================*/
#if	!USE_WINAPI && !defined(DPMI)
static	void	LoadBuff(void)
{
	HANDLE		hFile;
	DWORD		dwT;

	do	{
#if !PCUNIX
		hFile	= CreateFile(
				chFileName,
				GENERIC_READ,
				FILE_SHARE_READ,
				(LPSECURITY_ATTRIBUTES) NULL,
				OPEN_EXISTING,
				FILE_FLAG_SEQUENTIAL_SCAN|
				FILE_ATTRIBUTE_NORMAL,
				(HANDLE) NULL
			);
#else /* !PCUNIX */
		hFile	= CreateFile (chFileName, O_RDONLY,0);
#endif /* !PCUNIX */
		if	( hFile == INVALID_HANDLE_VALUE ) {
			break;
		}
#if !PCUNIX
		ReadFile(
			hFile,
			(LPVOID) sztBuf,
			MAX_BUFF - 1,
			&dwT,
			(LPOVERLAPPED) NULL
		);
#else /* !PCUNIX */
		dwT = MAX_BUFF - 1;
		ReadFile ( hFile, (LPVOID) sztBuf, &dwT);
#endif /* !PCUNIX */
		lenBuf	= (int) dwT;
		CloseHandle(hFile);
	}	while(0);
	return;
}
#endif

/*=============================================================================
	LoadBuff
=============================================================================*/
#if	!USE_WINAPI && defined(DPMI)
static	void	LoadBuff(void)
{
	short	fd;

	do	{
		fd	= LowOpen(chFileName,O_RDONLY|O_BINARY);
		if	( fd == -1 ) {
			break;
		}
		lenBuf	= LowRead(fd,sztBuf,MAX_BUFF-1);
		LowClose(fd);
	}	while(0);
	return;
}
#endif

/*=============================================================================
	AdjustBuff
=============================================================================*/
#if	!USE_WINAPI
static	void	AdjustBuff(LPTSTR lpchNewFile,LPTSTR lpchNewSect)
{
	int		n;
	int		i,j,k;
	LPTSTR		p;

#if !PCUNIX
	if	( lstrcmpi(chFileName,lpchNewFile) ) {
#else /* !PCUNIX */
	if	( lstrcmp(chFileName,lpchNewFile) ) {
#endif /* !PCUNIX */
		FlushBuff();
		lenBuf	= 0;
		lstrcpy(chFileName,lpchNewFile);
		LoadBuff();
		for	( k = i = 0 ; i < lenBuf ; i++ ) {
			if	( sztBuf[i] == LF ) {
				k	= i + 1;
			}
#if !DEF_ENV_EUC
			else if ( sztBuf[i] == CR ) {
				if	(sztBuf[i + 1] != LF) {
					sztBuf[i]	= SPACE;
				}
			}
#endif
			else if ( sztBuf[i] <= SPACE ) {
				sztBuf[i]	= SPACE;
			}
		}
		lenBuf		= k;
		sztBuf[lenBuf]	= (TCHAR) 0;
		lenLin 		= 0;
		inoLin[lenLin]	= 0;
		for	(i = 0 ; i < lenBuf && lenLin < MAX_LINE - 1 ; i++) {
			if	( sztBuf[i] == LF ) {
				inoLin[++lenLin]	= i + 1;
			}
		}
		*chSectName	= (TCHAR) 0;
	}
#if !PCUNIX
	if	( lstrcmpi(chSectName,lpchNewSect) ) {
#else /* !PCUNIX */
	if	( lstrcmp(chSectName,lpchNewSect) ) {
#endif /* !PCUNIX */
		lstrcpy(chSectName,lpchNewSect);
		begLin	= 0;
		endLin	= 0;
		j	= lstrlen(chSectName);
		k	= 0;
		for	( i = 0 ; i < lenLin ; i++ ) {
#if !DEF_ENV_EUC
			for	( n = 0 ; sztBuf[inoLin[i] + n] != CR ; n++ );
#else
			for	( n = 0 ; sztBuf[inoLin[i] + n] != LF ; n++ );
#endif
			if	( (n                         >= 3        )
			&&	  (sztBuf[inoLin[i]        ] == TEXT('['))
			&&	  (sztBuf[inoLin[i] + n - 1] == TEXT(']')) ) {
				if	( k == 0 ) {
					p	= &sztBuf[inoLin[i]+1];
					if	((n != j + 2		   )
#if !PCUNIX
					||	 (lstrncmpi(p,chSectName,j))) {
#else /* !PCUNIX */
					||	 (lstrncmp(p,chSectName,j))) {
#endif /* !PCUNIX */
						continue;
					}
					begLin	= i + 1;
					k	= 1;
				}
				else if	( k == 1 ) {
					k	= 2;
					break;
				}
			}
		}
		if	( k == 0 ) {
			n			= lstrlen(chSectName);
			sztBuf[lenBuf++]	= '[';
			while	( n-- ) sztBuf[lenBuf++] = *lpchNewSect++;
			sztBuf[lenBuf++]	= ']';
#if !DEF_ENV_EUC
			sztBuf[lenBuf++]	= CR;
#endif
			sztBuf[lenBuf++]	= LF;
			sztBuf[lenBuf  ]	= (TCHAR) 0;
			inoLin[++lenLin]	= lenBuf;
			begLin			= lenLin;
			endLin			= lenLin;
		}
		else {
			endLin			= i;
		}
		for	( i = begLin,k = 0 ; i < endLin ; i++,k++ ) {
			lpiSrt[k]	= &inoLin[i];
		}
		lenSrt	= k;
#ifndef	DPMI
		qsort((LPVOID)lpiSrt,lenSrt,sizSrt,lpicmpq);
#endif
	}
	return;
}
#endif

/*=============================================================================
	ScanElement
=============================================================================*/
#if	!USE_WINAPI
static	int	ScanElement(LPTSTR Key,int n)
{
	LPVOID	lpv;
	int	ret;

	ret	= endLin;
#ifndef	DPMI
	lpv	= bsearch((LPVOID) Key,(LPCVOID)lpiSrt,lenSrt,sizSrt,lpicmpb);
#else
	lpv	= lsearch((LPVOID) Key,(LPCVOID)lpiSrt,lenSrt,sizSrt,lpicmpb);
#endif
	if	( lpv != NULL ) {
		ret	= (int)( (*(LPINT FAR *)lpv) - (&inoLin[0]) );
	}
	return	ret;
}
#endif

/*=============================================================================
	GetElement
=============================================================================*/
#if	!USE_WINAPI
static	int	GetElement(LPTSTR Key,LPTSTR lpchValue,int cchValue)
{
	int	ret;
	int	n;
	int	i,j;

	ret	= FALSE;
	n	= lstrlen(Key);
	i	= ScanElement(Key,n);
	if	( i < endLin ) {
		j	= inoLin[i] + n + 1;
		cchValue--;	//For Null
#if !DEF_ENV_EUC
		while	(sztBuf[j] != CR && sztBuf[j] != LF && cchValue) {
#else
		while	(sztBuf[j] != LF && cchValue) {
#endif
			*lpchValue++	= sztBuf[j++];
			cchValue--;
		}
		ret		= TRUE;
	}
	*lpchValue	= (TCHAR) 0;
	return	ret;
}
#endif

/*=============================================================================
	SetElement
=============================================================================*/
#if	!USE_WINAPI
static	void	SetElement(LPTSTR Key,LPTSTR lpchValue)
{
	int	i,j,k;
	int	m,n;

	n	= lstrlen(Key);
	i	= ScanElement(Key,n);
	if	( lpchValue ) {
		m	= lstrlen(lpchValue);
		if	( i == endLin ) {
			for	( j = 0 ; j < lenSrt ; j++ ) {
				if	(lpicmpb(Key,&lpiSrt[j]) <= 0) break;
			}
			for	( k = lenSrt ; k > j ; k-- ) {
				lpiSrt[k]	= lpiSrt[k - 1];
			}
			lpiSrt[k]	= &inoLin[i];
			lenSrt		= lenSrt + 1;
			lenLin	= lenLin + 1;
			for	( j = lenLin ; j > endLin ; j-- ) {
				inoLin[j]	= inoLin[j - 1];
			}
			endLin	= endLin + 1;
		}
#if !DEF_ENV_EUC
		k	= (n + 1 + m + 2) - (inoLin[i+1] - inoLin[i]);
#else
		k	= (n + 1 + m + 1) - (inoLin[i+1] - inoLin[i]);
#endif
		if	( k != 0 ) {
			if	( k > 0 ) {
				for	(j=lenBuf-1 ; j >= inoLin[i+1] ; j--) {
					sztBuf[j+k]	= sztBuf[j];
				}
			}
			else	{
				for	(j=inoLin[i+1] ; j < lenBuf ; j++) {
					sztBuf[j+k]	= sztBuf[j];
				}
			}
			lenBuf	+= k;
			for	( j = i + 1 ; j <= lenLin ; j++ ) {
				inoLin[j]	+= k;
			}

		}
		j	= inoLin[i];
		while	( n-- )	sztBuf[j++] = *Key++;
				sztBuf[j++] = '=';
		while	( m-- )	sztBuf[j++] = *lpchValue++;
#if !DEF_ENV_EUC
				sztBuf[j++] = CR;
#endif
				sztBuf[j++] = LF;
	}
	else	{
		if	( i < endLin ) {
			for	( j = 0 ; j < lenSrt ; j++ ) {
				if	(lpicmpb(Key,&lpiSrt[j]) <= 0) break;
			}
			lenSrt		= lenSrt - 1;
			for	( k = 0 ; k < lenSrt ; k++ ) {
				if	( k >= j ) {
					lpiSrt[k]	= lpiSrt[k + 1];
				}
				if	( lpiSrt[k] > &inoLin[i] ) {
					lpiSrt[k]--;
				}
			}
			k	= inoLin[i+1] - inoLin[i];
			for	( j = inoLin[i+1] ; j < lenBuf ; j++ ) {
				sztBuf[j - k]	= sztBuf[j];
			}
			lenBuf	= lenBuf - k;
			for	( j = i + 1 ; j <= lenLin ; j++ ) {
				inoLin[j - 1]	= inoLin[j] - k;
			}
			endLin	= endLin - 1;
			lenLin	= lenLin - 1;
		}
	}
	sztBuf[lenBuf]	= (TCHAR) 0;
	bUpdate	= TRUE;
	return;
}
#endif

/*=============================================================================
	GetPPString
=============================================================================*/
int	WINAPI	GetPPString
(
	LPTSTR	lpchSection,
	LPTSTR	lpchTitle,
	LPTSTR	lpchDefault,
	LPTSTR	lpchValue,
	int	iBufferMax,
	LPTSTR	lpchFile
)
{
#if	!USE_WINAPI
	AdjustBuff(lpchFile,lpchSection);
	if	( !GetElement(lpchTitle,lpchValue,iBufferMax) ) {
		lstrcpy(lpchValue,lpchDefault);
	}
	return	lstrlen(lpchValue);
#else
	return	GetPrivateProfileString(lpchSection,
					lpchTitle,
					lpchDefault,
					lpchValue,
					iBufferMax,
					lpchFile	);
#endif
}

/*=============================================================================
	WritePPString
=============================================================================*/
int	WINAPI	WritePPString
(
	LPTSTR	lpchSection,
	LPTSTR	lpchTitle,
	LPTSTR	lpchValue,
	LPTSTR	lpchFile
)
{
#if	!USE_WINAPI
	AdjustBuff(lpchFile,lpchSection);
	SetElement  (lpchTitle,lpchValue);
	return	TRUE;
#else
	return	WritePrivateProfileString(lpchSection,
					  lpchTitle,
					  lpchValue,
					  lpchFile	);
#endif
}

/*=============================================================================
	FlushPP
=============================================================================*/
void	WINAPI	FlushPP
(
	void
)
{
#if	!USE_WINAPI
	FlushBuff();
	*chFileName = *chSectName = (TCHAR) 0;
#endif
	return;
}

/*=============================================================================
	BeginFetchPP
=============================================================================*/
#if	!USE_WINAPI
LPVOID	WINAPI	BeginFetchPP
(
	LPTSTR	lpchSection,
	LPTSTR	lpchFile
)
{
	AdjustBuff(lpchFile,lpchSection);
	return	(LPVOID) begLin;
}
#endif

/*=============================================================================
	FetchPPString
=============================================================================*/
#if	!USE_WINAPI
LPVOID	WINAPI	FetchPPString
(
	LPVOID	lpHandle,
	LPTSTR	lptTitle,
	int	maxTitle,
	LPTSTR	lptValue,
	int	maxValue
)
{
	LPTSTR	lptLin;
	int	i;
	int	j;
	LPVOID	ret;

	i	= (int)((DWORD)lpHandle);
	ret	= NULL;
	if	( i < endLin ) {
		lptLin	= &sztBuf[inoLin[i]];
		for	( (j = 0					)
#if !DEF_ENV_EUC
		;	  (j < maxTitle && *lptLin!=CR && *lptLin!=EQ	)
#else
		;	  (j < maxTitle && *lptLin!=LF && *lptLin!=EQ	)
#endif
		;	  (j++						) ) {
			*lptTitle++	= *lptLin++;
		}
		*lptTitle	= (TCHAR) 0;
		if	( *lptLin == EQ ) lptLin++;
		for	( (j = 0					)
#if !DEF_ENV_EUC
		;	  (j < maxValue && *lptLin!=CR			)
#else
		;	  (j < maxValue && *lptLin!=LF			)
#endif
		;	  (j++						) ) {
			*lptValue++	= *lptLin++;
		}
		*lptValue	= (TCHAR) 0;
		i++;
		ret	= (LPVOID) i;
	}
	return	ret;
}
#endif

/******************************************************************************
	Profile	Service Functions
******************************************************************************/
/*=============================================================================
	GPStr
=============================================================================*/
LPTSTR	WINAPI	GPStr
(
	LPTSTR	lptFile,
	LPTSTR	lptSect,
	LPTSTR	lptKey,
	LPTSTR	lptDef
)
{
static	TCHAR	sztHold[2048];

	sztHold[0]			= (TCHAR) 0;
	GetPPString(lptSect,lptKey,lptDef,sztHold,(sizeof sztHold)-1,lptFile);
	if	( !(*sztHold) ) {
		lstrncpy(sztHold,lptDef,(sizeof sztHold)-1);
	}
	return	(LPTSTR) sztHold;
}

/*=============================================================================
	GPInt
=============================================================================*/
int	WINAPI	GPInt
(
	LPTSTR	lptFile,
	LPTSTR	lptSect,
	LPTSTR	lptKey,
	int	iDef
)
{
	LPTSTR	lptStr;
	TCHAR	c;
	int	ret;

	lptStr	= GPStr(lptFile,lptSect,lptKey,sztNull);
	if	( *lptStr ) {
		ret	= 0;
		while	( *lptStr ) {
			c	= *lptStr++;
			if	( TEXT('0') <= c && c <= TEXT('9') ) {
				ret	= ret * 10 + (c - TEXT('0'));
			}
			else	{
				ret	= iDef;
				break;
			}
		}
	}
	else	{
		ret	= iDef;
	}
	return	ret;
}

/*=============================================================================
	GPSel
=============================================================================*/
int	WINAPI	GPSel
(
	LPTSTR	lptFile,
	LPTSTR	lptSect,
	LPTSTR	lptKey,
	int	iDef,
	LPTSTR	lptSel
)
{
	LPTSTR	lptBuff;
	int	lenBuff;
	LPTSTR	lptOut;
	int	ret;
	int	i;

	lptBuff	= GPStr(lptFile,lptSect,lptKey,sztNull);
	lenBuff	= lstrlen(lptBuff);
	if	( !lenBuff ) {
		ret	= iDef;
	}
	else	{
		lptOut	= lptSel;
		for	( i = 0 ; lptOut = lstrtok(lptOut,TEXT(' ')) ; i++ ) {
#if !PCUNIX
			if	( !lstrcmpi(lptOut,lptBuff) ) {
#else /* !PCUNIX */
			if	( !lstrcmp(lptOut,lptBuff) ) {
#endif /* !PCUNIX */
				ret	= i;
				break;
			}
		}
		if	( !lptOut ) {
			ret	= iDef;
		}
	}
	return	ret;
}

/*=============================================================================
	GPHex
=============================================================================*/
int	WINAPI	GPHex
(
	LPTSTR	lptFile,
	LPTSTR	lptSect,
	LPTSTR	lptKey,
	int	iDef
)
{
	LPTSTR	lptOut;
	int	ret;
	int	c;

	lptOut	= GPStr(lptFile,lptSect,lptKey,sztNull);
	if	( *lptOut ) {
		ret	= 0;
		while	( *lptOut ) {
			c	= *lptOut++;
			if	( TEXT('0') <= c && c <= TEXT('9') ) {
				c	= c - TEXT('0');
			}
			else if	( TEXT('a') <= c && c <= TEXT('f') ) {
				c	= c - TEXT('a') + 0x0a;
			}
			else if	( TEXT('A') <= c && c <= TEXT('F') ) {
				c	= c - TEXT('A') + 0x0a;
			}
			else	{
				c	= 0;
			}
			ret	= (ret << 4) | c & 0x0f;
		}
	}
	else	{
		ret	= (int) iDef;
	}
	return	(WORD) ret;
}

/*=============================================================================
	WPStr
=============================================================================*/
void	WINAPI	WPStr
(
	LPTSTR	lptFile,
	LPTSTR	lptSect,
	LPTSTR	lptKey,
	LPTSTR	lptDat
)
{
	WritePPString(lptSect,lptKey,lptDat,lptFile);
	return;
}

/*=============================================================================
	WPInt
=============================================================================*/
void	WINAPI	WPInt
(
	LPTSTR	lptFile,
	LPTSTR	lptSect,
	LPTSTR	lptKey,
	int	iDat
)
{
	WPStr(lptFile,lptSect,lptKey,litos(iDat,10));
	return;
}

/*=============================================================================
	WPSel
=============================================================================*/
void	WINAPI	WPSel
(
	LPTSTR	lptFile,
	LPTSTR	lptSect,
	LPTSTR	lptKey,
	int	iDat,
	LPTSTR	lptSel
)
{
	LPTSTR	lptOut;
	int	i;

	lptOut	= lptSel;
	for	( i = 0 ; lptOut = lstrtok(lptOut,TEXT(' ')) ; i++ ) {
		if	( i == (int) iDat ) {
			break;
		}
	}
	WPStr(lptFile,lptSect,lptKey,lptOut);
	return;
}

/*=============================================================================
	WPHex
=============================================================================*/
void	WINAPI	WPHex
(
	LPTSTR	lptFile,
	LPTSTR	lptSect,
	LPTSTR	lptKey,
	int	iDat
)
{
	WPStr(lptFile,lptSect,lptKey,litos(iDat,16));
	return;
}

/******************************************************************************
	for lstr* Function
******************************************************************************/
/*=============================================================================
	lstrcpy
=============================================================================*/
/* #ifdef	DPMI */
#if PCUNIX || defined(DPMI)
LPTSTR	WINAPI	lstrcpy(LPTSTR dest,LPTSTR src)
{
	LPTSTR	ret;

	ret	= dest;
	while	( *dest++ = *src++ );
	return	ret;
}
#endif

/*=============================================================================
	lstrcat
=============================================================================*/
/* #ifdef	DPMI */
#if PCUNIX || defined(DPMI)
LPTSTR	WINAPI	lstrcat(LPTSTR dest,LPTSTR src)
{
	LPTSTR	ret;

	ret	= dest;
	while	( *dest ) dest++;
	while	( *dest++ = *src++ );
	return	ret;
}
#endif

/*=============================================================================
	lstrcmp
=============================================================================*/
/* #ifdef	DPMI */
#if PCUNIX || defined(DPMI)
int	WINAPI	lstrcmp(LPTSTR dest,LPTSTR src)
{
	int	ret;

	ret	= 0;
	while	( *dest || *src ) {
		if	( *dest != *src ) {
			ret	= (*dest - *src);
			break;
		}
		dest++;
		src++;
	}
	return	ret;
}
#endif

/*=============================================================================
	lstrcmpi
=============================================================================*/
/* #ifdef	DPMI */
#if PCUNIX || defined(DPMI)
int	WINAPI	lstrcmpi(LPTSTR dest,LPTSTR src)
{
	int	ret;
	UCHAR	cd,cs;

	ret	= 0;
	while	( *dest || *src ) {
/* #ifndef	DPMI */
#if !PCUNIX && !defined(DPMI)
		cd	= CharUpper((LPTSTR)(WORD)*dest);
		cs	= CharUpper((LPTSTR)(WORD)*src );
#else
		cd	= upper(*dest);
		cs	= upper(*src );
#endif
		if	( cd != cs ) {
			ret	= cd - cs;
			break;
		}
		dest++;
		src++;
	}
	return	ret;
}
#endif

/*=============================================================================
	lstrlen
=============================================================================*/
/* #ifdef	DPMI */
#if PCUNIX || defined(DPMI)
int	WINAPI	lstrlen(LPTSTR src)
{
	int	ret;

	ret	= 0;
	while	( *src++ ) ret++;
	return	ret;
}
#endif

/*=============================================================================
	sstrlen
=============================================================================*/
#if !PCUNIX
#ifdef	DPMI
int	_cdecl	strlen(LPTSTR src)
{
	return	lstrlen(src);
}
#endif
#endif /* PCUNIX */

/*=============================================================================
	lstrncpy	Win32's lstrcpyn has Fatal Bug.
=============================================================================*/
LPTSTR	WINAPI	lstrncpy(LPTSTR lptDes,LPTSTR lptSrc,int maxDes)
{
	LPTSTR	ret;

	ret	= lptDes;
	while	( maxDes && *lptSrc ) {
#ifndef	UNICODE
		if	( IsDBCSLeadByte(*lptSrc) ) {
			if	( maxDes <= 1 ) {
				break;
			}
			*lptDes++	= *lptSrc++;
			maxDes--;
		}
#endif
		*lptDes++	= *lptSrc++;
		maxDes--;
	}
	*lptDes	= (TCHAR) 0;
	return	ret;
}

/*=============================================================================
	lstrcpy2		楳ɤȴƥԡ
=============================================================================*/
LPTSTR	WINAPI	lstrcpy2(LPTSTR lptDes,LPTSTR lptSrc)
{
	LPTSTR	ret;
	TCHAR	c;

	ret	= lptDes;
	while	( (c = *lptSrc++) ) {
		if	( c >= TEXT(' ') && c != 0x7f ) {
			*lptDes++	= c;
		}
	}
	*lptDes	= (TCHAR) 0;
	return	ret;
}

/*=============================================================================
	lmemcpy
=============================================================================*/
LPTSTR	WINAPI	lmemcpy(LPSTR lptDes,LPSTR lptSrc,int maxDes)
{
	LPTSTR	ret;

	ret	= lptDes;
	while	( maxDes-- ) {
		*lptDes++	= *lptSrc++;
	}
	return	ret;
}

/*=============================================================================
	lstrncmp
=============================================================================*/
int	WINAPI	lstrncmp(LPTSTR lptDes,LPTSTR lptSrc,int maxCmp)
{
	int	ret	= 0;

	while	( maxCmp-- ) {
		if	( *lptDes != *lptSrc ) {
			ret	= (*lptDes - *lptSrc);
			break;
		}
		lptDes++;
		lptSrc++;
	}
	return	ret;
}

/*=============================================================================
	lstrncmpi
=============================================================================*/
int	WINAPI	lstrncmpi(LPTSTR lptDes,LPTSTR lptSrc,int maxCmp)
{
	int	ret	= 0;

	while	( maxCmp-- ) {
/* #ifndef	DPMI */
#if !PCUNIX && !defined(DPMI)
		if	( CharUpper((LPTSTR)(WORD)(*lptDes))
		!=	  CharUpper((LPTSTR)(WORD)(*lptSrc)) )
#else
		if	( upper(*lptDes) != upper(*lptSrc) )
#endif
		{
			ret	= (*lptDes - *lptSrc);
			break;
		}
		lptDes++;
		lptSrc++;
	}
	return	ret;
}

/*=============================================================================
	lstrchr
=============================================================================*/
LPTSTR	WINAPI	lstrchr(LPTSTR lptSrc,TCHAR tch)
{
	LPTSTR	ret;

	ret	= NULL;
	for	( ; *lptSrc ; lptSrc++ ) {
		if	( *lptSrc == tch ) {
			ret	= lptSrc;
			break;
		}
	}
	return	ret;
}

/*=============================================================================
	jstrlen
=============================================================================*/
int	WINAPI	jstrlen(LPTSTR lptSrc)
{
	int	ret;
	TCHAR	c;

#ifdef	UNICODE
	ret	= lstrlen(lptSrc);
#else
	ret	= 0;
	while	( *lptSrc ) {
		c	= *lptSrc++;
		if	( IsDBCSLeadByte(c) ) {
			c	= *lptSrc++;
		}
		ret++;
	}
#endif
	return	ret;
}

/*=============================================================================
	lstrtok
=============================================================================*/
LPTSTR	WINAPI	lstrtok(LPTSTR lptStr,TCHAR sztSep)
{
static	TCHAR	sztOut[128];
static	LPTSTR	lptSav;
	TCHAR	c1,c2;
	LPTSTR	ret;

	c1	= sztSep;
	c2	= (TCHAR)(( c1 != TEXT(' ') ) ? c1 : TEXT('\t'));
	if	( lptStr == sztOut ) {		/* ܰʹߤθƤӽФ	*/
		lptStr	= lptSav;
	}
	while	( *lptStr && (*lptStr == c1 || *lptStr == c2) ) {
		lptStr++;
	}
	if	( *lptStr ) {
		ret	= sztOut;
		while	( *lptStr && (*lptStr != c1 && *lptStr != c2) ) {
			*ret++	= *lptStr++;
		}
		*ret	= (TCHAR) 0;
		lptSav	= lptStr;
		ret	= sztOut;
	}
	else	{
		ret	= NULL;
	}
	return	ret;
}

/*=============================================================================
	latoi
=============================================================================*/
int	WINAPI	latoi(LPTSTR lptStr)
{
	int	ret;
	TCHAR	c;

	ret	= 0;
	while	( lptStr && *lptStr ) {
		c	= *lptStr++;
		if	( TEXT('0') <= c && c <= TEXT('9') ) {
			ret	= ret * 10 + (c - TEXT('0'));
		}
		else	{
			break;
		}
	}
	return	ret;
}

/*=============================================================================
	litos
=============================================================================*/
LPTSTR	WINAPI	litos(int v,int r)
{
static	TCHAR	t[36];
	int	c;
	int	i;
	int	s;

	i	= recof(t) - 1;
	t[i]	= 0x00;
	s	= v;
	if	( v < 0 ) {
		v	= -v;
	}
	if	( !(2 <= r && r <= 36) ) {
		r	= 10;
	}
	do	{
		c	= v % r;
		c	= (c > 9) ? (c + '7') : (c + '0');
		t[--i]	= (TCHAR) c;
		v	= v / r;
	}	while	( i > 0 && v > 0 );
	if	( s < 0 ) {
		t[--i]	= '-';
	}
	return	&t[i];
}

/*=============================================================================
	lstrxcat
=============================================================================*/
LPTSTR	_cdecl	lstrxcat(LPTSTR lptOut, ...)
{
	va_list	ap;
	LPTSTR	lptEach;
	LPTSTR	ret;

	ret	= lptOut;
	*lptOut	= (TCHAR) 0;
	va_start(ap,lptOut);
	while	(1) {
		lptEach	= va_arg(ap,LPTSTR);
		if	( !lptEach ) break;
		while	( *lptEach ) *lptOut++ = *lptEach++;
	}
	va_end  (ap);
	*lptOut	= (TCHAR) 0;
	return	ret;
}

/*=============================================================================
	lchrcat
=============================================================================*/
LPTSTR	WINAPI	lchrcat(LPTSTR lptDes,int iChar)
{
	LPTSTR	lptTmp;

	lptTmp	= lptDes;
#ifdef	UNICODE
	*lptTmp++	= (TCHAR) iChar;
#else
	if	( iChar & 0xff00 ) {
		*lptTmp++	= (TCHAR) HIBYTE(iChar);
	}
	*lptTmp++	= (TCHAR) LOBYTE(iChar);
#endif
	*lptTmp		= (TCHAR) 0;
	return	lptDes;
}

/*=============================================================================
	lstrtyp		ʸlptSrc iPosܤʸλ֤

	͡	0:оݤλ	ANK Ǥ
		1:		裱ХȤǤ
		2:		裲ХȤǤ
=============================================================================*/
int	WINAPI	lstrtyp(LPTSTR lptSrc,int iPos)
{
	int	iCount;
	int	iState;
	TCHAR	c;

	for ( iCount = iState = 0 ; iCount <= iPos ; iCount++ ) {
		c = *lptSrc++;
		if	(iState == 1)
			iState = 2;
		else if ((0x80 <= c && c <= 0x9f) || (0xe0 <= c && c <= 0xfc))
			iState = 1;
		else
			iState = 0;
	}
	return	iState;
}

/*=============================================================================
	lsearch
=============================================================================*/
#ifdef	DPMI
void *	_cdecl	lsearch
(
	void *	key,
	void *	base,
	int	num,
	int	width,
	CMPFUNC	compare
)
{
	int	place;

	place	= 0;
	while	(place < num) {
		if	(!(*compare)(key,base)) {
			return	base;
		}
		else {
			base	= (char *)base + width;
			place++;
		}
	}
	return	NULL;
}
#endif
