/******************************************************************************
	Xdraw.C
******************************************************************************/
#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include "typedef.h"
#include "xim.h"
#include "xui.h"
#include "macro.h"
#include "fileio.h"
#include "env.h"
#include "getstr.h"
#include "strutl.h"
#include "getvcode.h"


/*=============================================================================
	Define
=============================================================================*/
#define	IsEucKanji(s) ((s != EUC_kanashift) && (s & 0x80))
#define	IsEucKana(s)  (s == EUC_kanashift)


/*=============================================================================
	DrawBorder
=============================================================================*/
void	DrawBorder
(
	Window	window,
	GC	gc[],
	int	x,
	int	y,
	int	w,
	int	h
)
{
#define	LINE	4
	int	x1=x, x2=x1+w-1;

	y += h/LINE-1;
	XDrawLine (display, window, gc[0], x1, y, x2, y);
	y++;
	XDrawLine (display, window, gc[2], x1, y, x2, y);
	y++;
	XDrawLine (display, window, gc[1], x1, y, x2, y);
	y++;
	XDrawLine (display, window, gc[0], x1, y, x2, y);
#undef	LINE
}

/*=============================================================================
	DrawRectangle
=============================================================================*/
void	DrawRectangle
(
	Window	window,
	GC	gc,
	int     x,
	int     y,
	int     w,
	int     h
)
{
	int	x1=x, y1=y;
	int	x2=x1+w-1, y2=y1+h-1;

	XDrawLine (display, window, gc, x1, y1, x1, y2);
	XDrawLine (display, window, gc, x1, y1, x2, y1);
	XDrawLine (display, window, gc, x2, y1, x2, y2);
	XDrawLine (display, window, gc, x1, y2, x2, y2);
	x1++; x2--; y1++; y2--;
	XDrawLine (display, window, gc, x1, y1, x1, y2);
	XDrawLine (display, window, gc, x1, y1, x2, y1);
	XDrawLine (display, window, gc, x2, y1, x2, y2);
	XDrawLine (display, window, gc, x1, y2, x2, y2);

	return;
}

/*=============================================================================
	DrawPopButton1
=============================================================================*/
void	DrawPopButton1
(
	Window	window,
	GC	gc[],
	int     x,
	int     y,
	int     w,
	int     h
)
{
	int	x1=x, y1=y;
	int	x2=x1+w-1, y2=y1+h-1;

	XDrawLine (display, window, gc[1], x1, y1, x1, y2);
	XDrawLine (display, window, gc[1], x1, y1, x2, y1);
	XDrawLine (display, window, gc[2], x2, y1, x2, y2);
	XDrawLine (display, window, gc[2], x1, y2, x2, y2);

	return;
}

/*=============================================================================
	DrawPushButton1
=============================================================================*/
void	DrawPushButton1
(
	Window	window,
	GC	gc[],
	int     x,
	int     y,
	int     w,
	int     h
)
{
	int	x1=x, y1=y;
	int	x2=x1+w-1, y2=y1+h-1;

	XDrawLine (display, window, gc[2], x1, y1, x1, y2);
	XDrawLine (display, window, gc[2], x1, y1, x2, y1);
	XDrawLine (display, window, gc[1], x2, y1, x2, y2);
	XDrawLine (display, window, gc[1], x1, y2, x2, y2);

	return;
}

/*=============================================================================
	DrawPopButton
=============================================================================*/
void	DrawPopButton
(
	Window	window,
	GC	gc[],
	int     x,
	int     y,
	int     w,
	int     h
)
{
	int	x1=x, y1=y;
	int	x2=x1+w-1, y2=y1+h-1;

	XDrawLine (display, window, gc[1], x1, y1, x1, y2);
	XDrawLine (display, window, gc[1], x1, y1, x2, y1);
	XDrawLine (display, window, gc[2], x2, y1, x2, y2);
	XDrawLine (display, window, gc[2], x1, y2, x2, y2);
	x1++; x2--; y1++; y2--;
	XDrawLine (display, window, gc[0], x1, y1, x1, y2);
	XDrawLine (display, window, gc[0], x1, y1, x2, y1);
	XDrawLine (display, window, gc[0], x2, y1, x2, y2);
	XDrawLine (display, window, gc[0], x1, y2, x2, y2);

	return;
}

/*=============================================================================
	DrawPushButton
=============================================================================*/
void	DrawPushButton
(
	Window	window,
	GC	gc[],
	int     x,
	int     y,
	int     w,
	int     h
)
{
	int	x1=x, y1=y;
	int	x2=x1+w-1, y2=y1+h-1;

	XDrawLine (display, window, gc[2], x1, y1, x1, y2);
	XDrawLine (display, window, gc[2], x1, y1, x2, y1);
	XDrawLine (display, window, gc[1], x2, y1, x2, y2);
	XDrawLine (display, window, gc[1], x1, y2, x2, y2);
	x1++; x2--; y1++; y2--;
	XDrawLine (display, window, gc[0], x1, y1, x1, y2);
	XDrawLine (display, window, gc[0], x1, y1, x2, y1);
	XDrawLine (display, window, gc[0], x2, y1, x2, y2);
	XDrawLine (display, window, gc[0], x1, y2, x2, y2);

	return;
}

/*=============================================================================
	DrawPopScrollButton
=============================================================================*/
void	DrawPopScrollButton
(
	Window	window,
	GC	gc[],
	int     x,
	int     y,
	int     w,
	int     h
)
{
	int	x1=x, y1=y;
	int	x2=x1+w-1, y2=y1+h-1;

	XDrawLine (display, window, gc[1], x1, y1, x1, y2);
	XDrawLine (display, window, gc[1], x1, y1, x2, y1);
	XDrawLine (display, window, gc[2], x2, y1, x2, y2);
	XDrawLine (display, window, gc[2], x1, y2, x2, y2);
	x1++; x2--; y1++; y2--;
	XDrawLine (display, window, gc[0], x1, y1, x1, y2);
	XDrawLine (display, window, gc[0], x1, y1, x2, y1);
	XDrawLine (display, window, gc[0], x2, y1, x2, y2);
	XDrawLine (display, window, gc[0], x1, y2, x2, y2);

	return;
}

/*=============================================================================
	DrawPushScrollButton
=============================================================================*/
void	DrawPushScrollButton
(
	Window	window,
	GC	gc[],
	int     x,
	int     y,
	int     w,
	int     h
)
{
	int	x1=x, y1=y;
	int	x2=x1+w-1, y2=y1+h-1;

	XDrawLine (display, window, gc[2], x1, y1, x1, y2);
	XDrawLine (display, window, gc[2], x1, y1, x2, y1);
	XDrawLine (display, window, gc[1], x2, y1, x2, y2);
	XDrawLine (display, window, gc[1], x1, y2, x2, y2);
	x1++; x2--; y1++; y2--;
	XDrawLine (display, window, gc[0], x1, y1, x1, y2);
	XDrawLine (display, window, gc[0], x1, y1, x2, y1);
	XDrawLine (display, window, gc[0], x2, y1, x2, y2);
	XDrawLine (display, window, gc[0], x1, y2, x2, y2);

	return;
}

/*=============================================================================
	DrawPanel
=============================================================================*/
void	DrawPanel
(
	Window	window,
	GC	gc[],
	int     x,
	int     y,
	int     w,
	int     h
)
{
	int	x1=x, y1=y;
	int	x2=x1+w-1, y2=y1+h-1;

#if 1
	XDrawLine (display, window, gc[0], x1, y1, x1, y2);
	XDrawLine (display, window, gc[0], x1, y1, x2, y1);
	XDrawLine (display, window, gc[4], x2, y1, x2, y2);
	XDrawLine (display, window, gc[4], x1, y2, x2, y2);
	x1++; x2--; y1++; y2--;
	XDrawLine (display, window, gc[1], x1, y1, x1, y2);
	XDrawLine (display, window, gc[1], x1, y1, x2, y1);
	XDrawLine (display, window, gc[2], x2, y1, x2, y2);
	XDrawLine (display, window, gc[2], x1, y2, x2, y2);
#else
	XDrawLine (display, window, gc[0], x1, y1, x1, y2);
	XDrawLine (display, window, gc[0], x1, y1, x2, y1);
	XDrawLine (display, window, gc[2], x2, y1, x2, y2);
	XDrawLine (display, window, gc[2], x1, y2, x2, y2);
	x1++; x2--; y1++; y2--;
	XDrawLine (display, window, gc[1], x1, y1, x1, y2);
	XDrawLine (display, window, gc[1], x1, y1, x2, y1);
	XDrawLine (display, window, gc[2], x2, y1, x2, y2);
	XDrawLine (display, window, gc[2], x1, y2, x2, y2);
#endif

	return;
}

/*=============================================================================
	DrawOutline
=============================================================================*/
void	DrawOutline
(
	Window	window,
	GC	gc,
	int	x,
	int	y,
	int	w,
	int	h
)
{
	int	xx=x, yy=y, ww=w, hh=h;

	XDrawRectangle (display, window, gc, xx, yy, ww, hh);
	xx++; yy++; ww-=2; hh-=2;
	XDrawRectangle (display, window, gc, xx, yy, ww, hh);
}

/*=============================================================================
	VJEDrawString
=============================================================================*/
int	VJEDrawString
(
	Display		*d,
	Window		w,
	GC		gc,
	FXimFont	*font,
	int		x,
	int		y,
	unsigned char	text[],
	char		attr[],
	int		len,
	int		mode,		/* 0=Ascii 1=Kanji*/
	int		ip		/* */
)
{
	XFontStruct 	*font_struct;
	int		height, width=0, attribute;
	int		n, start;
	int		width2;
	int		eucCnt,jisCnt;
	UCHAR		sjisBuff[MAX_COMP_SIZE+1];
	UCHAR		eucBuff[MAX_COMP_SIZE+1];
	UCHAR		jisBuff[MAX_COMP_SIZE+1];
	short		flag;
	short 		i, j;

	height = font->max_font_height;
	for(n=0;n<len;) {
		start = n;
		attribute = attr[start];
		flag = 0;
		if (VjeKanji(text[start])) {
			n += 2;
			flag = 1;
		} else
			n ++;

        	for (i=0,j=start; i< n-start; i++,j++) {
			sjisBuff[i]= text[j];
		}
		sjisBuff[i] = 0;
		eucCnt = ShiftjisToEuc(eucBuff,sjisBuff);
		jisCnt = ShiftjisToJis(jisBuff,sjisBuff);
                width2 = FXChTextWidth(font,eucBuff,eucCnt);

		if (!(attribute & CL_NODISP)){
			attribute &= 0x7f;
			SetVclrAttr(d,gc,attribute,1);
			XFillRectangle(d,w,gc,x+width,y-font->max_font_ascent,
				width2, font->max_font_height+2);
			SetVclrAttr(d,gc,attribute,0);

		  	if (flag == 0) {
				font_struct = font->ascii_kana_font_struct;
				XSetFont(d,gc,font_struct->fid);
				XDrawImageString(d,w,gc,x+width,y,
					(char *)jisBuff,jisCnt);
			} else {
				font_struct = font->kanji_font_struct;
				XSetFont(d,gc,font_struct->fid);
				XDrawImageString16(d,w,gc,x+width,y,
					(XChar2b *)jisBuff,jisCnt);
			}
		}
		width += width2;
	}
	XFlush(d);
	return ( width );
}

/*=============================================================================
	zenhanString
=============================================================================*/
void	zenhanString
(
	Display		*d,
	Window		w,
	FXimFont	*font,
	int		x,
	int		y,
	unsigned char 	text[],
	char		attr[],
	int		len,
	int		ip		/* WindowIP */
)
{
	int	i,j,start,width;
	GC	gc;
	char	flag;			/* 0=ASCII Code 1=*/

	for(i=0,width=0;i<len;){
		if(iskanji(text[i])) {
			flag = 1;	/*  */
			gc = font->reverse_gc;
		} else {
			flag = 0;	/* ASCII */
			gc = font->normal_gc;
		}
		start = i;
		for(j=0;i<len;i++,j++){
			if(iskanji(text[i]) != flag)
				break;
			else if(flag) {
				i++; j++;
			}
		}
		if(j) {
			width	+= VJEDrawString (
					d,w,gc,font,
					x+width,y,&text[start],
					&attr[start],j,flag,ip
				);
		}
	}
}

/*=============================================================================
	DrawEucTextAttrGcString
=============================================================================*/
void	DrawEucTextAttrGcString
(
	Display		*d,
	Window		w,
	FXimFont	*font,
	int		x,
	int		y,
	unsigned char	buf[],
	GC		gc,
	int		len
)
{
 	XFontStruct	*font_struct;
	int		width, start;
        UCHAR		eucBuff[MAX_COMP_SIZE+1];
	UCHAR		jisBuff[MAX_COMP_SIZE+1];
        int		jisCnt;
	char		flag;
        short		ii, jj;                                      
	int		i, j, k, n;

	for (i=0,width=0; i < len;) {
		if (IsEucKanji (buf[i])) {
			flag = 1;	/*  */
		} else {
			flag = 0;	/* ASCII */
		}
		start = i;
		for (j=0; i < len; i++,j++) {
			if ((IsEucKanji (buf[i]) ? 1 : 0) != flag) {
				break;
			} else if (IsEucKana  (buf[i])
				|| IsEucKanji (buf[i])) {
				i++; j++;
			}
		}

                for (ii=0,jj=start; ii < j; ii++,jj++) {
			eucBuff[ii] = buf[jj];
                } 
                eucBuff[ii] = 0;

		jisCnt = euc2jis (jisBuff, eucBuff);

  		if (flag == 0) {
			font_struct = font->ascii_kana_font_struct;
			XSetFont (d, gc, font_struct->fid);
			XDrawImageString (
				d, w, gc, x + width, y,
				(char *)jisBuff, jisCnt
			);
		} else {
			font_struct = font->kanji_font_struct;
			XSetFont (d, gc, font_struct->fid);
			XDrawImageString16 (
				d, w, gc,x + width, y,
				(XChar2b *)jisBuff, jisCnt
			);
		}
                width += FXChTextWidth (font, eucBuff, lstrlen (eucBuff));
	}
}

/*=============================================================================
	DrawEucTextAttrNoString
=============================================================================*/
void	DrawEucTextAttrNoString
(
	Display		*d,
	Window		w,
	FXimFont	*font,
	int		x,
	int		y,
	unsigned char	buf[],
	int		attr,
	int		len
)
{
	GC		gc;
 	XFontStruct	*font_struct;
	int		width, start;
        UCHAR		eucBuff[MAX_COMP_SIZE+1];
	UCHAR		jisBuff[MAX_COMP_SIZE+1];
        int		jisCnt;
	char		flag;
        short		ii, jj;                                      
	int		i, j, k, n;

	for (i=0,width=0; i < len;) {
		if (IsEucKanji (buf[i])) {
			flag = 1;	/*  */
			gc   = font->reverse_gc;
		} else {
			flag = 0;	/* ASCII */
			gc   = font->normal_gc;
		}
		start = i;
		for (j=0; i < len; i++,j++) {
			if ((IsEucKanji (buf[i]) ? 1 : 0) != flag) {
				break;
			} else if (IsEucKana  (buf[i])
				|| IsEucKanji (buf[i])) {
				i++; j++;
			}
		}
		if(attr == 1){/* focus line */
			XSetForeground(d,gc,GetVclrPixel (CLR_WHITE));
			XSetBackground(d,gc,GetVclrPixel (CLR_BLACK));
		} else if (attr == 0) {/* no focus line */
			XSetForeground(d,gc,GetVclrPixel (CLR_BLACK));
			XSetBackground(d,gc,GetVclrPixel (CLR_WHITE));
		} else if (attr == 2) {
			XSetForeground(d,gc,GetVclrPixel (CLR_BLACK));
			XSetBackground(d,gc,GetVclrPixel (CLR_GRAY ));
		} else if (attr == 3) {
			XSetForeground(d,gc,GetVclrPixel (CLR_YELLOW));
			XSetBackground(d,gc,GetVclrPixel (CLR_TICKER_GRAY));
		} else if (attr == 4) {
			XSetForeground(d,gc,GetVclrPixel (CLR_YELLOW));
			XSetBackground(d,gc,GetVclrPixel (CLR_WHITE));
		} else if (attr == 5) {
			XSetForeground(d,gc,GetVclrPixel (CLR_TICKER_GRAY));
			XSetBackground(d,gc,GetVclrPixel (CLR_GRAY ));
		} else if (attr == 6) {
			XSetForeground(d,gc,GetVclrPixel (CLR_BLACK));
			XSetBackground(d,gc,GetVclrPixel (CLR_LIGHT_YELLOW));
		}

                for (ii=0,jj=start; ii < j; ii++,jj++) {
			eucBuff[ii] = buf[jj];
                } 
                eucBuff[ii] = 0;

		jisCnt = euc2jis (jisBuff, eucBuff);

  		if (flag == 0) {
			font_struct = font->ascii_kana_font_struct;
			XSetFont (d, gc, font_struct->fid);
			XDrawImageString (
				d, w, gc, x + width, y,
				(char *)jisBuff, jisCnt
			);
		} else {
			font_struct = font->kanji_font_struct;
			XSetFont (d, gc, font_struct->fid);
			XDrawImageString16 (
				d, w, gc,x + width, y,
				(XChar2b *)jisBuff, jisCnt
			);
		}
                width += FXChTextWidth (font, eucBuff, lstrlen (eucBuff));
	}
}

/*=============================================================================
	DrawEucTextAttrString
=============================================================================*/
void	DrawEucTextAttrString
(
	Display		*d,
	Window		w,
	FXimFont	*font,
	int		x,
	int		y,
	unsigned char	buf[],
	int		attr,
	int		len
)
{
	GC		gc;
 	XFontStruct	*font_struct;
	int		i, j, k, width, start, n;
	char		flag;
        unsigned char	eucBuff[MAX_COMP_SIZE+1];
	unsigned char	jisBuff[MAX_COMP_SIZE+1];
        int		jisCnt;
        short		ii, jj;                                      

	width	= 0;
	for (i=0; i < len;) {
		if (IsEucKanji (buf[i])) {
			flag = 1; /*   */
			gc   = font->reverse_gc;
		} else {
			flag = 0; /* ASCII */
			gc   = font->normal_gc;
		}
		start = i;
		for (j=0; i < len; i++, j++) {
			if ((IsEucKanji (buf[i]) ? 1 : 0) != flag) {
				break;
			} else if (IsEucKana  (buf[i])
				|| IsEucKanji (buf[i])) {
				i++; j++;
			}
		}
		SetVclrAttr (d, gc, attr, 0);

                for (ii=0,jj=start; ii < j; ii++,jj++) {
			eucBuff[ii] = buf[jj];
                } 
                eucBuff[ii] = 0;

		jisCnt = euc2jis (jisBuff, eucBuff);

  		if (flag == 0) {
			font_struct = font->ascii_kana_font_struct;
			XSetFont (d, gc, font_struct->fid);
			XDrawImageString (
				d, w, gc, x + width, y,
				(char *)jisBuff, jisCnt
			);
		} else {
			font_struct = font->kanji_font_struct;
			XSetFont (d, gc, font_struct->fid);
			XDrawImageString16 (
				d, w, gc, x + width, y,
				(XChar2b *)jisBuff, jisCnt
			);
		}
                width += FXChTextWidth (font, eucBuff, lstrlen (eucBuff));
	}
}

/*=============================================================================
	DataKakutei
=============================================================================*/
void	DataKakutei
(
	FXim_Client	*client
)
{
	short	i, j;

	if (client == root_client) {
		return;
	}
	/*
	 * ʸУ̤
	 */
	*(client->bdata.lpchResult + client->bdata.cchResult)	= (char )NULL;
	j	= ShiftjisToUshortEuc (
			client->output_buff,
			client->bdata.lpchResult
		);
	client->reverse_colums	= j;
	client->last_colums	= j;
	client->current_colums	= j;
	for (i=0; i<client->last_colums; i++) {
		if ((client->output_buff[i] >> 8) == 0x8e) {
			client->output_buff[i]	&= 0x00ff;
		}
	}
	SendBuffByLength (
		client->window,
		client->output_buff,
		client->reverse_colums
	);
	sleep (0);

	/*
	 * ʬξʤɡʸϡѴʸƱȯ
	 * 礬ޤ
	 */
	if (client->bdata.cchDisplay != 0) {
		j	= ShiftjisToUshortEuc (
				client->output_buff,
				client->bdata.lpchDisplay
			);
        	client->reverse_colums	= 0;
        	client->last_colums	= j;
		Clear_NODISP(client);
		client->input_flag	= FINF_MODY;
	} else {
		client->reverse_colums	= 0;
		client->last_colums	= 0;
		client->current_colums	= 0;
	}
}

/*=============================================================================
	VJEDisplayPut
=============================================================================*/
void	VJEDisplayPut
(
	FXim_Client	*client
)
{
	PBD	bdata;
	LPVKP	lpVKp;

	bdata	= (PBD   )GetBDATA (client);
	lpVKp	= (LPVKP )GetVKp (client);

	/*---------------------------------------------------------------------
		ʸ
	---------------------------------------------------------------------*/
	if ((bdata->wDataUpdate & VJE_RESULT)
	 && (bdata->wDataStatus & VJE_RESULT)) {
		if (bdata->cchResult) {
			DataKakutei (client);
			bdata->cchResult = 0;
		}
	}

	/*---------------------------------------------------------------------
		ϡѴʸ
	---------------------------------------------------------------------*/
	if (bdata->wDataUpdate & VJE_DISPLAYDATA){ 
		DisplayOutputBuff ();
	}

	/*---------------------------------------------------------------------
		ꥹȤ
	---------------------------------------------------------------------*/
	if ((bdata->wMainStage & PRC_DON)
	 || (bdata->wMainStage & PRC_CODE)) {
		if (lpVKp->Visible != 1
	 	 || bdata->wMainStage & PRC_CODE) {
			KohoWindowOpen (client);
		}
		if ((bdata->wDataUpdate & VJE_LBDATA)
		 && (bdata->wDataStatus & VJE_LBDATA)) {
			if (bdata->cchLBListLen > 0) {
				KohoWindowDisplay (client);
			}
		}
	} else {
		if (lpVKp->Visible == 1) {
			KohoWindowClose (client);
		}
	}

	/*---------------------------------------------------------------------
		ơɽ
	---------------------------------------------------------------------*/
	if ((bdata->wDataUpdate & VJE_MODEDATA)
	 && (bdata->wDataStatus & VJE_MODEDATA)) {
		DisplayStatus (client);
	}

}

/*=============================================================================
	DrawStatusButtonInit
=============================================================================*/
void	DrawStatusButtonInit
(
	FXim_Client	*client,
	LPVWIN		lpVWin
)
{
	LPVSPLT		lpVSplt;
	Window		window;
	GC		*gc;
	int		x, y, w, h;

	lpVSplt	= (LPVSPLT )GetVSplt (lpVWin);

	window	= root_client->status_window;
	gc	= lpVWin->GCList;
	x	= lpVSplt->wBtnX[lpVSplt->wBtnPosPrev]
		- DRAW_STATUS_WINDOW_BUTTON_BORDER;
	y	= lpVSplt->wBtnY[lpVSplt->wBtnPosPrev]
		- DRAW_STATUS_WINDOW_BUTTON_BORDER;
	w	= lpVSplt->wBtnSize
		+ DRAW_STATUS_WINDOW_BUTTON_BORDER * 2;
	h	= lpVSplt->wBtnSize
		+ DRAW_STATUS_WINDOW_BUTTON_BORDER * 2;

	DrawRectangle (window, gc[0], x, y, w, h);
}

/*=============================================================================
	DrawStatusButton
=============================================================================*/
void	DrawStatusButton
(
	FXim_Client	*client,
	LPVWIN		lpVWin
)
{
	LPVSPLT		lpVSplt;
	Window		window;
	GC		*gc;
	int		x, y, w, h;

	lpVSplt	= (LPVSPLT )GetVSplt (lpVWin);

	if (lpVSplt->wBtnPos >= lpVSplt->wBtnCount) {
		return;
	}

	if (lpVSplt->wBtnPosPrev != lpVSplt->wBtnPos) {
		DrawStatusButtonInit (client, lpVWin);
		lpVSplt->wBtnPosPrev	= lpVSplt->wBtnPos;
	}

	window	= root_client->status_window;
	gc	= lpVWin->GCList;
	x	= lpVSplt->wBtnX[lpVSplt->wBtnPos]
		- DRAW_STATUS_WINDOW_BUTTON_BORDER;
	y	= lpVSplt->wBtnY[lpVSplt->wBtnPos]
		- DRAW_STATUS_WINDOW_BUTTON_BORDER;
	w	= lpVSplt->wBtnSize
		+ DRAW_STATUS_WINDOW_BUTTON_BORDER * 2;
	h	= lpVSplt->wBtnSize
		+ DRAW_STATUS_WINDOW_BUTTON_BORDER * 2;

	switch (lpVSplt->wBtnMode) {
	case STS_BTN_MODE_POP:
		DrawPopButton  (window, gc, x, y, w, h);
		break;
	case STS_BTN_MODE_PSH:
		DrawPushButton (window, gc, x, y, w, h);
		break;
	}
	XFlush (display);
}

/*=============================================================================
	DrawStatusPalette
=============================================================================*/
void	DrawStatusPalette
(
	FXim_Client	*client,
	LPVWIN		lpVWin
)
{
	LPVSPLT		lpVSplt;
	Pixmap		pPixBtn;
	TCHAR		sztText[MAX_LONG_STR + MAX_LONG_STR + 1];
	int		iIndex, iPlus;
	int		i, iCount;

	lpVSplt	= (LPVSPLT )GetVSplt (lpVWin);

	if (lpVSplt->wBtnPos == WIN_STS_BTN_MAX) {
		i	= 0;
		iCount	= lpVSplt->wBtnCount;
		XFillRectangle (
			display,
			lpVSplt->pPixBtn,
			lpVWin->GCList[0],
			0,
			0,
			lpVWin->StatusM.width,
			lpVWin->StatusM.height
		);
	} else {
		i	= lpVSplt->wBtnPos;
		iCount	= lpVSplt->wBtnPos + 1;
	}

	if (lpVSplt->wBtnMode != STS_BTN_MODE_PSH
	 || lpVSplt->wBtnPos  == WIN_STS_BTN_MAX) {
		iPlus	= 0;
	} else {
		iPlus	= 1;
	}
	iPlus  += (lpVSplt->wBtnSize - lpVSplt->wPixBtnSize) / 2;

	for (; i < iCount; i++) {
		iIndex	= GetFuncState (
				client,
				lpVSplt->wBtnFunc[i],
				sztText
			);
		iIndex	+= lpVSplt->wBtnIndex[i];

		if (!IsScreenMode (client)) {
			if (lpVSplt->wBtnFunc[i] == F_SCRN) {
				pPixBtn	= lpVSplt->pPixBtnOn;
			} else {
				pPixBtn	= lpVSplt->pPixBtnOff;
			}
		} else {
			pPixBtn	= lpVSplt->pPixBtnOn;
		}

		XFillRectangle (
			display,
			lpVSplt->pPixBtn,
			lpVWin->GCList[0],
			lpVSplt->wBtnX[i] - DRAW_STATUS_WINDOW_BUTTON_BORDER,
			lpVSplt->wBtnY[i] - DRAW_STATUS_WINDOW_BUTTON_BORDER,
			lpVSplt->wBtnSize + DRAW_STATUS_WINDOW_BUTTON_BORDER*2,
			lpVSplt->wBtnSize + DRAW_STATUS_WINDOW_BUTTON_BORDER*2
		);
		XCopyArea (
			display,
			pPixBtn,
			lpVSplt->pPixBtn,
			lpVWin->GCList[5],
			iIndex * lpVSplt->wPixBtnSize,
			0,
			lpVSplt->wPixBtnSize,
			lpVSplt->wPixBtnSize,
			lpVSplt->wBtnX[i] + iPlus,
			lpVSplt->wBtnY[i] + iPlus
		);
	}
	XCopyArea (
		display,
		lpVSplt->pPixBtn,
		root_client->status_window,
		lpVWin->GCList[5],
		0,
		0,
		lpVWin->StatusM.width,
		lpVWin->StatusM.height,
		0,
		0
	);

	XFlush (display);
}

/*=============================================================================
	DrawStatusWindow
=============================================================================*/
void	DrawStatusWindow
(
	Window	window,
	LPVWIN	lpVWin
)
{
	DrawPanel (
		window,
		lpVWin->GCList,
		0,
		0,
		lpVWin->StatusR.width,
		lpVWin->StatusR.height
	);
}

/*=============================================================================
	DrawPopUpMenuFrame
=============================================================================*/
void	DrawPopUpMenuFrame
(
	Window	window,
	GC	gc[],
	int	w,
	int	h
)
{
	DrawPanel (window, gc, 0, 0, w, h);
}

/*=============================================================================
	DrawPopUpMenu
=============================================================================*/
void	DrawPopUpMenu
(
	Window	window,
	GC	gc[],
	int	w,
	int	h
)
{
	XFillRectangle (display, window, gc[0], 0, 0, w, h);
	DrawPopUpMenuFrame (window, gc, w, h);
}

/*=============================================================================
	DrawPopUpMenuMessageBorder
=============================================================================*/
void	DrawPopUpMenuMessageBorder
(
	Window	window,
	GC	gc[],
	int	x,
	int	y,
	int	w,
	int	h
)
{
#define	LINE	2
	int	x1=x, x2=x1+w-1;

	y += h/LINE-1;
	XDrawLine (display, window, gc[2], x1, y, x2, y);
	y++;
	XDrawLine (display, window, gc[1], x1, y, x2, y);
#undef	LINE
}

/*=============================================================================
	DrawPopUpMenuMessageArea
=============================================================================*/
void	DrawPopUpMenuMessageArea
(
	Window	window,
	GC	gc[],
	int	focus,
	int	chk,
	int	cld,
	int	x,
	int	y,
	int	w,
	int	h
)
{
#define	NUM_POINTS	3
	XPoint		points[NUM_POINTS];
	int		xx, yy;

	if (focus == 0) {
		DrawRectangle (window, gc[0], x, y, w, h);
	} else {
		DrawPopButton (window, gc, x, y, w, h);
	}

	yy	= y+h/2 + h%2;
	if (chk != 0) {
		xx	= DRAW_POPUP_MENU_BORDER
			+ DRAW_POPUP_MENU_FONT_BORDER
			+ DRAW_POPUP_MENU_CHECK_MARK
			/ 2;
		points[0].x = xx-4;	points[0].y = yy+1;
		points[1].x = xx-2;	points[1].y = yy+4;
		points[2].x = xx;	points[2].y = yy-4;
		/* black */
		XDrawLine (
			display, window, gc[4],
			points[0].x, points[0].y, points[1].x, points[1].y
		);
		/* black */
		XDrawLine (
			display, window, gc[4],
			points[1].x, points[1].y, points[2].x, points[2].y
		);
	}
	if (cld != 0) {
		xx	= w - 1;
		points[0].x = xx-4;	points[0].y = yy-4;
		points[1].x = xx-4; 	points[1].y = yy+4;
		points[2].x = xx;	points[2].y = yy;
		/* black */
		XFillPolygon (
			display, window, gc[4],
			points, NUM_POINTS, Nonconvex, CoordModeOrigin
		);
	}
#undef	NUM_POINTS
}

/*=============================================================================
	DrawPreeditRootWindow
=============================================================================*/
void	DrawPreeditRootWindow
(
	Window	window,
	GC	gc[],
	short	w1,
	short	h1,
	short	x2,
	short	y2,
	short	w2,
	short	h2
)
{
	XFillRectangle (display, window, gc[0], 0, 0, w1, h1);
	DrawPanel (window, gc, 0, 0, w1, h1);

	/* white */
	XFillRectangle (display, window, gc[3],
		x2 - DRAW_PREEDIT_ROOT_WINDOW_FONT_BORDER,
		y2 - DRAW_PREEDIT_ROOT_WINDOW_FONT_BORDER,
		w2 + DRAW_PREEDIT_ROOT_WINDOW_FONT_BORDER * 2,
		h2 + DRAW_PREEDIT_ROOT_WINDOW_FONT_BORDER * 2);
}

/*=============================================================================
	DrawKohoHScrPtr
=============================================================================*/
void	DrawKohoHScrPtr
(
	FXim_Client	*client,
	LPVWIN		lpVWin
)
{
	LPVKP		lpVKp;
	UCHAR		buf[3];
	Window		window;
	GC		*gc;
	int		xx, yy, ww, hh;

	lpVKp	= (LPVKP )GetVKp (client);

	window	= client->koho_window;
	gc	= lpVWin->GCList;

	xx	= lpVKp->HScr.x
		+ lpVKp->HScr.height;
	yy	= lpVKp->HScr.y;
	ww	= lpVKp->HScr.width
		- lpVKp->HScr.height * 2;
	hh	= lpVKp->HScr.height;

	XFillRectangle (display, window, gc[1], xx, yy, ww, hh);
	GetKohoHScrPtr (client, &xx, &ww);

	XFillRectangle (display, window, gc[0], xx, yy, ww, hh);
	DrawPopScrollButton (window, gc, xx, yy, ww, hh);
}

/*=============================================================================
	DrawKohoHScrArw
=============================================================================*/
void	DrawKohoHScrArw
(
	FXim_Client	*client,
	LPVWIN		lpVWin,
	int		pos
)
{
	LPVKP		lpVKp;
	LPVKPLT		lpVKplt;
	UCHAR		buf[3];
	Window		window;
	Pixmap		pixmap;
	GC		*gc;
	int		size;
	int		xx, yy, ww, hh;
	int		plus, idx;

	lpVKp	= (LPVKP   )GetVKp (client);
	lpVKplt	= (LPVKPLT )GetVKplt (lpVWin);

	window	= client->koho_window;
	gc	= lpVWin->GCList;
	pixmap	= lpVKplt->pPixBtn;
	size	= lpVKplt->wPixBtnSize;

	xx	= lpVKp->HScr.x;
	yy	= lpVKp->HScr.y;
	ww	= lpVKp->HScr.height;
	hh	= lpVKp->HScr.height;
	idx	= WIN_KOHO_HSCR_ARW_LEFT
		- WIN_KOHO_SCR_BASE;
	plus	= (pos == WIN_KOHO_HSCR_ARW_LEFT ? 1 : 0);
	XCopyArea (
		display,
		pixmap,
		window,
		gc[5],
		idx * size,
		0,
		size,
		size,
		xx + (ww-size)/2 + plus,
		yy + (hh-size)/2 + plus
	);
	if (pos == WIN_KOHO_HSCR_ARW_LEFT) {
		DrawPushScrollButton (window, gc, xx, yy, ww, hh);
	} else {
		DrawPopScrollButton  (window, gc, xx, yy, ww, hh);
	}

	xx	= lpVKp->HScr.x
		+ lpVKp->HScr.width
		- lpVKp->HScr.height;
	yy	= lpVKp->HScr.y;
	ww	= lpVKp->HScr.height;
	hh	= lpVKp->HScr.height;
	idx	= WIN_KOHO_HSCR_ARW_RIGHT
		- WIN_KOHO_SCR_BASE;
	plus	= (pos == WIN_KOHO_HSCR_ARW_RIGHT ? 1 : 0);
	XCopyArea (
		display,
		pixmap,
		window,
		gc[5],
		idx * size,
		0,
		size,
		size,
		xx + (ww-size)/2 + plus,
		yy + (hh-size)/2 + plus
	);
	if (pos == WIN_KOHO_HSCR_ARW_RIGHT) {
		DrawPushScrollButton (window, gc, xx, yy, ww, hh);
	} else {
		DrawPopScrollButton  (window, gc, xx, yy, ww, hh);
	}
}

/*=============================================================================
	DrawKohoVScrPtr
=============================================================================*/
void	DrawKohoVScrPtr
(
	FXim_Client	*client,
	LPVWIN		lpVWin
)
{
	LPVKP		lpVKp;
	UCHAR		buf[3];
	Window		window;
	GC		*gc;
	int		xx, yy, ww, hh;

	lpVKp	= (LPVKP )GetVKp (client);

	window	= client->koho_window;
	gc	= lpVWin->GCList;

	xx	= lpVKp->VScr.x;
	yy	= lpVKp->VScr.y
		+ lpVKp->VScr.width;
	ww	= lpVKp->VScr.width;
	hh	= lpVKp->VScr.height
		- lpVKp->VScr.width * 2;

	XFillRectangle (display, window, gc[1], xx, yy, ww, hh);
	GetKohoVScrPtr (client, &yy, &hh);

	XFillRectangle (display, window, gc[0], xx, yy, ww, hh);
	DrawPopScrollButton (window, gc, xx, yy, ww, hh);
}

/*=============================================================================
	DrawKohoVScrArw
=============================================================================*/
void	DrawKohoVScrArw
(
	FXim_Client	*client,
	LPVWIN		lpVWin,
	int		pos
)
{
	LPVKP		lpVKp;
	LPVKPLT		lpVKplt;
	UCHAR		buf[3];
	Window		window;
	Pixmap		pixmap;
	GC		*gc;
	int		size;
	int		xx, yy, ww, hh;
	int		plus, idx;

	lpVKp	= (LPVKP   )GetVKp (client);
	lpVKplt	= (LPVKPLT )GetVKplt (lpVWin);

	window	= client->koho_window;
	gc	= lpVWin->GCList;
	pixmap	= lpVKplt->pPixBtn;
	size	= lpVKplt->wPixBtnSize;

	xx	= lpVKp->VScr.x;
	yy	= lpVKp->VScr.y;
	ww	= lpVKp->VScr.width;
	hh	= lpVKp->VScr.width;
	idx	= WIN_KOHO_VSCR_ARW_PREV
		- WIN_KOHO_SCR_BASE;
	plus	= (pos == WIN_KOHO_VSCR_ARW_PREV ? 1 : 0);
	XCopyArea (
		display,
		pixmap,
		window,
		gc[5],
		idx * size,
		0,
		size,
		size,
		xx + (ww-size)/2 + plus,
		yy + (hh-size)/2 + plus
	);
	if (pos == WIN_KOHO_VSCR_ARW_PREV) {
		DrawPushScrollButton (window, gc, xx, yy, ww, hh);
	} else {
		DrawPopScrollButton  (window, gc, xx, yy, ww, hh);
	}

	xx	= lpVKp->VScr.x;
	yy	= lpVKp->VScr.y
		+ lpVKp->VScr.height
		- lpVKp->VScr.width;
	ww	= lpVKp->VScr.width;
	hh	= lpVKp->VScr.width;
	idx	= WIN_KOHO_VSCR_ARW_NEXT
		- WIN_KOHO_SCR_BASE;
	plus	= (pos == WIN_KOHO_VSCR_ARW_NEXT ? 1 : 0);
	XCopyArea (
		display,
		pixmap,
		window,
		gc[5],
		idx * size,
		0,
		size,
		size,
		xx + (ww-size)/2 + plus,
		yy + (hh-size)/2 + plus
	);
	if (pos == WIN_KOHO_VSCR_ARW_NEXT) {
		DrawPushScrollButton (window, gc, xx, yy, ww, hh);
	} else {
		DrawPopScrollButton  (window, gc, xx, yy, ww, hh);
	}
}

/*=============================================================================
	DrawKohoScr
=============================================================================*/
void	DrawKohoScr
(
	FXim_Client	*client,
	LPVWIN		lpVWin
)
{
	LPVKP		lpVKp;
	UCHAR		buf[3];
	Window		window;
	GC		*gc;
	int		xx, yy, ww, hh;

	lpVKp	= (LPVKP )GetVKp (client);

	window	= client->koho_window;
	gc	= lpVWin->GCList;

	if (ChKohoVScr (client)) {
		xx	= lpVKp->VScr.x;
		yy	= lpVKp->VScr.y;
		ww	= lpVKp->VScr.width;
		hh	= lpVKp->VScr.height;
		XFillRectangle (display, window, gc[0], xx, yy, ww, hh);
		DrawKohoVScrArw (client, lpVWin, -1);
		DrawKohoVScrPtr (client, lpVWin);
	}

	if (ChKohoHScr (client)) {
		xx	= DRAW_KOHO_WINDOW_BORDER;
		yy	= lpVKp->HScr.y;
		ww	= lpVKp->Win.width
			- DRAW_KOHO_WINDOW_BORDER * 2;
		hh	= lpVKp->HScr.height;
		XFillRectangle (display, window, gc[0], xx, yy, ww, hh);
		DrawKohoHScrArw (client, lpVWin, -1);
		DrawKohoHScrPtr (client, lpVWin);
	}

}

/*=============================================================================
	DrawKohoHist
=============================================================================*/
void	DrawKohoHist
(
	FXim_Client	*client,
	LPVWIN		lpVWin
)
{
	PBD		bdata;
	LPVKP		lpVKp;
	UCHAR		euc[MAX_COMP_SIZE+1];
	UCHAR		*sjis;
	UCHAR		buf[3];
	Window		window;
	FXimFont	*font;
	GC		*gc;
	ULONG		fore, back;
	int		xx, yy, ww, hh;
	int		cnt, width;

	bdata	= (PBD   )GetBDATA (client);
	lpVKp	= (LPVKP )GetVKp (client);

	window	= client->koho_window;
	font	= client->koho_font;
	gc	= lpVWin->GCList;
	fore	= lpVWin->WinForePixel;
	back	= lpVWin->WinBackPixel;

	XSetForeground (display, gc[5], fore);
	XSetBackground (display, gc[5], back);

	xx	= DRAW_KOHO_WINDOW_BORDER;
	yy	= lpVKp->Hist.y;
	ww	= lpVKp->Win.width
		- DRAW_KOHO_WINDOW_BORDER * 2;
	hh	= lpVKp->Hist.height;
	XFillRectangle (display, window, gc[0], xx, yy, ww, hh);

	xx	= lpVKp->VIdx.x;
	yy	= lpVKp->Hist.y;
	ww	= lpVKp->VIdx.width;
	hh	= lpVKp->Hist.height;
	sprintf (buf, "%d", 0);
	DrawEucTextAttrGcString (
		display,
		window,
		font,
		xx + DRAW_KOHO_WINDOW_VIDX_BORDER,
		yy + hh
		   - DRAW_KOHO_WINDOW_FONT_BORDER
		   - font->max_font_descent,
		buf,
		gc[5],
		lstrlen (buf)
	);
#if 0
	DrawPopButton1 (window, gc, xx, yy, ww, hh);
#elif 0
	DrawPushButton1(1window, gc, xx, yy, ww, hh);
#endif

	xx	= DRAW_KOHO_WINDOW_BORDER;
	if (lpVKp->Direction == 1) {
		yy	= lpVKp->Hist.y
			- DRAW_KOHO_WINDOW_DON_BORDER;
	} else {
		yy	= lpVKp->Hist.y
			+ lpVKp->Hist.height;
	}
	ww	= lpVKp->Win.width
		- DRAW_KOHO_WINDOW_BORDER * 2;
	hh	= DRAW_KOHO_WINDOW_DON_BORDER;
	DrawBorder (window, gc, xx, yy, ww, hh);

	xx	= lpVKp->Hist.x;
	yy	= lpVKp->Hist.y;
	ww	= lpVKp->Hist.width;
	hh	= lpVKp->Hist.height;
#if 0 /* 990723:KOHO-HIST */
	sjis	= (char *)bdata->XXXXXXXXXX;
	cnt	= ShiftjisToEuc (euc, sjis);
#else
	lstrcpy (euc, "TEST...");
	cnt	= lstrlen (euc);
#endif
	DrawEucTextAttrGcString (
		display,
		window,
		font,
		xx,
		yy + hh
		   - DRAW_KOHO_WINDOW_FONT_BORDER
		   - font->max_font_descent,
		euc,
		gc[5],
		lstrlen (euc)
	);
}

/*=============================================================================
	DrawKohoIndex
=============================================================================*/
void	DrawKohoIndex
(
	FXim_Client	*client,
	LPVWIN		lpVWin
)
{
	LPVKP		lpVKp;
	UCHAR		buf[3];
	Window		window;
	FXimFont	*font;
	GC		*gc;
	ULONG		fore, back;
	int		kana_w;
	int		xx, yy, ww, hh;
	int		mtx, dir;
	int		col, row, len;
	int		i;

	lpVKp		= (LPVKP )GetVKp (client);

	window		= client->koho_window;
	font		= client->koho_font;
	gc		= lpVWin->GCList;
	fore		= lpVWin->WinForePixel;
	back		= lpVWin->WinBackPixel;

	GetVKpCanMtx (client, &col, &row, &len);
	XSetForeground (display, gc[5], fore);
	XSetBackground (display, gc[5], back);

	xx		= lpVKp->VIdx.x;
	yy		= lpVKp->VIdx.y;
	ww		= lpVKp->VIdx.width;
	hh		= lpVKp->VIdx.height * row;
	XFillRectangle (display, window, gc[0], xx, yy, ww, hh);

	xx		= lpVKp->VIdx.x;
	yy		= lpVKp->VIdx.y;
	ww		= lpVKp->VIdx.width;
	hh		= lpVKp->VIdx.height;
	for (i=1; i<=row; i++) {
		sprintf (buf, "%d", i);
		DrawEucTextAttrGcString (
			display,
			window,
			font,
			xx + DRAW_KOHO_WINDOW_VIDX_BORDER,
			yy + hh
			   - DRAW_KOHO_WINDOW_FONT_BORDER
			   - font->max_font_descent,
			buf,
			gc[5],
			lstrlen (buf)
		);
#if 0
		DrawPopButton1 (window, gc, xx, yy, ww, hh);
#elif 0
		DrawPushButton1(window, gc, xx, yy, ww, hh);
#endif
		yy	+= hh;
	}

	if (col > 1) {
		kana_w	= font->ascii_kana_font_width;
		xx	= DRAW_KOHO_WINDOW_BORDER;
		yy	= lpVKp->HIdx.y;
		ww	= lpVKp->Win.width
			- DRAW_KOHO_WINDOW_BORDER * 2;
		hh	= lpVKp->HIdx.height;
		XFillRectangle (display, window, gc[0], xx, yy, ww, hh);

		xx	= lpVKp->HIdx.x;
		yy	= lpVKp->HIdx.y;
		ww	= lpVKp->HIdx.width;
		hh	= lpVKp->HIdx.height;
		for (i=1; i<=col; i++) {
			sprintf (buf, "%c", XK_A + (i - 1));
			DrawEucTextAttrGcString (
				display,
				window,
				font,
				xx + (ww - kana_w) / 2,
				yy + hh
				   - DRAW_KOHO_WINDOW_HIDX_BORDER
				   - font->ascii_kana_font_struct->descent,
				buf,
				gc[5],
				lstrlen (buf)
			);
#if 0
			DrawPopButton1 (window, gc, xx, yy, ww, hh);
#elif 0
			DrawPushButton1(window, gc, xx, yy, ww, hh);
#endif
			xx	+= ww;
		}
	}
}

/*=============================================================================
	DrawKohoBtn
=============================================================================*/
void	DrawKohoBtn
(
	FXim_Client	*client,
	LPVWIN		lpVWin
)
{
	PBD		bdata;
	LPVKP		lpVKp;
	LPVKPLT		lpVKplt;
	TCHAR		sztText[MAX_LONG_STR + MAX_LONG_STR + 1];
	Window		window;
	Pixmap		pixmap;
	GC		*gc;
	int		size;
	int		xx, yy, ww, hh;
	int		plus, idx;
	int		i, count;

	bdata	= (PBD     )GetBDATA (client);
	lpVKp	= (LPVKP   )GetVKp (client);
	lpVKplt	= (LPVKPLT )GetVKplt (lpVWin);

	window	= client->koho_window;
	gc	= lpVWin->GCList;
	pixmap	= lpVKplt->pPixBtn;
	size	= lpVKplt->wPixBtnSize;

	if (lpVKplt->wBtnPos == WIN_KOHO_BTN_MAX) {
		i	= 0;
		count	= lpVKplt->wBtnCount;
	} else {
		i	= lpVKplt->wBtnPos;
		count	= lpVKplt->wBtnPos + 1;
	}

	for (; i < count; i++) {
		xx	= lpVKp->Btn[i].x;
		yy	= lpVKp->Btn[i].y;
		ww	= lpVKp->Btn[i].width;
		hh	= lpVKp->Btn[i].height;
		plus	= GetFuncState (
				client,
				lpVKplt->wBtnFunc[i],
				sztText
			);
		idx	= lpVKplt->wBtnIndex[i]
			+ plus;
		switch (lpVKplt->wBtnFunc[i]) {
		case	F_EX_KOHO_SORT_TOP:
		case	F_EX_KOHO_SORT_END:
			XFillRectangle (
				display, window, gc[0], xx, yy, ww, hh
			);
			XCopyArea (
				display, pixmap, window, gc[5],
				idx * size, 0, size, size,
				xx + (ww-size)/2 + plus,
				yy + (hh-size)/2 + plus
			);
			if (plus) {
				DrawPushButton (window, gc, xx, yy, ww, hh);
			} else {
				DrawPopButton (window, gc, xx, yy, ww, hh);
			}
			break;
		default:
			break;
		}
	}
	return;
}

/*=============================================================================
	DrawKohoWindow
=============================================================================*/
void	DrawKohoWindow
(
	FXim_Client	*client,
	LPVWIN		lpVWin
)
{
	LPVKP		lpVKp;
	LPVKPLT		lpVKplt;
	Window		window;
	GC		*gc;
	int		xx, yy, ww, hh;
	int		mtx, dir;
	int		col, row, len;

	lpVKp		= (LPVKP   )GetVKp (client);
	lpVKplt		= (LPVKPLT )GetVKplt (lpVWin);

	window		= client->koho_window;
	gc		= lpVWin->GCList;

	GetVKpCanMtx (client, &col, &row, &len);

	xx		= 0;
	yy		= 0;
	ww		= lpVKp->Win.width;
	hh		= lpVKp->Win.height;
	DrawPanel (window, gc, xx, yy, ww, hh);

	xx		= 2;
	yy		= 2;
	ww		= lpVKp->Win.width  - xx * 2;
	hh		= lpVKp->Win.height - yy * 2;
	DrawRectangle (window, gc[0], xx, yy, ww, hh);

	xx		= 4;
	yy		= 4;
	ww		= lpVKp->Win.width  - xx * 2;
	hh		= lpVKp->Win.height - yy * 2;
	DrawRectangle (window, gc[0], xx, yy, ww, hh);

	/* Draw Border1 */
	xx		= DRAW_KOHO_WINDOW_BORDER;
	if (lpVKp->Direction == 1) {
		yy	= lpVKp->Don.y
			+ lpVKp->Don.height * row;
	} else {
		yy	= lpVKp->Don.y
			- DRAW_KOHO_WINDOW_DON_BORDER;
	}
	ww		= lpVKp->Win.width
			- DRAW_KOHO_WINDOW_BORDER * 2;
	hh		= DRAW_KOHO_WINDOW_DON_BORDER;
	DrawBorder (window, gc, xx, yy, ww, hh);

	/* Draw Border2 */
	xx		= DRAW_KOHO_WINDOW_BORDER;
	if (lpVKp->Direction == 1) {
		yy	= lpVKp->Hist.y
			+ lpVKp->Hist.height;
	} else {
		yy	= lpVKp->Hist.y
			- DRAW_KOHO_WINDOW_DON_BORDER;
	}
	ww		= lpVKp->Win.width
			- DRAW_KOHO_WINDOW_BORDER * 2;
	hh		= DRAW_KOHO_WINDOW_DON_BORDER;
	DrawBorder (window, gc, xx, yy, ww, hh);

	/* Clear Koho Btn & Title */
	xx		= DRAW_KOHO_WINDOW_BORDER;
	if (lpVKp->Direction == 1) {
		yy	= lpVKp->Code.y;
	} else {
		yy	= lpVKp->Code.y
			- DRAW_KOHO_WINDOW_TITLE_BORDER;
	}
	ww		= lpVKp->Win.width
			- DRAW_KOHO_WINDOW_BORDER * 2;
	hh		= lpVKp->Code.height
			+ DRAW_KOHO_WINDOW_TITLE_BORDER;
	XFillRectangle (display, window, gc[0], xx, yy, ww, hh);

	DrawKohoIndex (client, lpVWin);

	if (ChKohoHist (client)) {
		DrawKohoHist  (client, lpVWin);
	}

	if (ChKohoVScr (client)
	 || ChKohoHScr (client)) {
		DrawKohoScr  (client, lpVWin);
	}

	lpVKplt->wBtnPos	= WIN_KOHO_BTN_MAX;
	DrawKohoBtn (client, lpVWin);
}

/*=============================================================================
	DrawStatus
=============================================================================*/
/*990222 OnTheSpot:StatusWithAttributes*/
int	DrawStatus
(
	FXim_Client	*client,
	char		*status_message,
	char		*status_attrib
)
{
	/*990125 OnTheSpot*/
	unsigned short	outbuff[STATUS_MESSAGE_LENGTH];/*990222*/
	unsigned short	outattr[STATUS_MESSAGE_LENGTH];/*990222*/
	LPVWIN		lpVWin;
	LPVSPLT		lpVSplt;
	UCHAR		euc[STATUS_MESSAGE_LENGTH*2];
	int		xx, yy;
	int		length;
	int		j;

	lpVWin			= (LPVWIN  )GetVWin ();
	lpVSplt			= (LPVSPLT )GetVSplt (lpVWin);
	lpVSplt->wBtnPos	= WIN_STS_BTN_MAX;
	DrawStatusPalette (client, lpVWin);

	if (client->status_window == root_client->status_window) {
		return;
	}

	/*990125 OnTheSpot*/
	if (client->style & XIMStatusCallbacks) {
		/*990222 StatusDraw*/
		if (0 == lstrcmp (client->save_status_message, status_message)
		 && client->save_status_attrib == status_attrib[0]) {
			return;
		}
		strcpy (client->save_status_message, status_message);
		client->save_status_attrib	= status_attrib[0];

		XimStatusStartSend ();
#if 0/*990222 OnTheSpot:StatusWithAttributes*/
		j	= ShiftjisToUshortEuc (outbuff, status_message);
		XimStatusDrawSend (outbuff, j);
#else
		j	= ShiftjisToUshortEucWithAttr (
				outbuff, outattr,
				status_message,
				status_attrib
			);
		XimStatusDrawSend (outbuff, outattr, j);
#endif
		XimStatusDoneSend ();

	} else if (client->style & XIMStatusArea) {
		xx	= DRAW_STATUS_WINDOW_BORDER;
		yy	= (client->status->Area.height
			   - client->status_font->max_font_height
			   + 1)
			/ 2;
		XClearArea (
			display,
			client->status_window,
			xx,
			yy,
			client->status->Area.width
			- DRAW_STATUS_WINDOW_BORDER * 2,
			client->status->Area.height
			- yy * 2,
			False
		);
		yy	+= client->status_font->max_font_ascent;
	
		length	= ShiftjisToEuc (euc, status_message);
		DrawEucTextAttrNoString (
			display,
			client->status_window,
			client->status_font,
			xx,
			yy,
			euc,
			0,
			length
		);
	}
}

