/******************************************************************************
	Xkoho.C
******************************************************************************/
#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include "typedef.h"
#include "xim.h"
#include "xui.h"
#include "macro.h"
#include "fileio.h"
#include "env.h"
#include "getstr.h"
#include "strutl.h"
#include "getvcode.h"


/*=============================================================================
	Define
=============================================================================*/
#define	KOHO_TITLE_LEN		(12)	/* " XXXXX:99999"	*/
					/* "     999/999"	*/
#define	KOHO_MTRBDR_LEN		( 1)


/*=============================================================================
	LSqrt
=============================================================================*/
LONG	LSqrt
(
	LONG	x
)
{
	LONG	r;
	LONG	q;

	if (x <= 0) {
		return (0);
	}
	if (x == 1) {
		return (1);
	}
    	r = x >> 1;
	for (;;) {
		q	= x / r;
		if (q >= r) {
			return (r);
		} else {
			r	= (r + q) >> 1;
		}
	}
}

/*=============================================================================
	FuncKohoMax
=============================================================================*/
void	FuncKohoMax
(
	FXim_Client	*client,
	int		col,
	int		row
)
{
	KD	kd;
	InitKD (&kd);
	kd.lFuncOption	= MAKELONG ((WORD )col, (WORD )row);
	kd.wFunc	= F_KOHOMAX;
	FuncHandler (client, &kd);
}

/*=============================================================================
	GetVKp
=============================================================================*/
LPVKP	GetVKp
(
	FXim_Client	*client
)
{
	return ((LPVKP )&client->vkp);
}

/*=============================================================================
	EnvToVKp
=============================================================================*/
void	EnvToVKp
(
	FXim_Client	*client,
	LPVWIN		lpVWin,
	LPENV		lpEnv
)
{
	LPVKP		lpVKp;
	WORD		wFlag;
	WORD		wFunc;

	lpVKp	= (LPVKP )GetVKp (client);

	wFlag	= LOWORD (lpVWin->Param);
	wFunc	= HIWORD (lpVWin->Param);

	if (!(wFlag & (ENV_ALL|ENV_POS))) {
		return;
	}

	lpVKp->Mtx	= lpEnv->can.wMatrix;
	lpVKp->Dir	= lpEnv->can.wDirection;
	lpVKp->DonCol	= lpEnv->can.wDonCol;
	lpVKp->DonRow	= lpEnv->can.wDonRow;
	lpVKp->DonLen	= lpEnv->can.wDonLen	* DBCS_SIZE;
#if 0 /* Delete:CodeLen */
	lpVKp->CodeCol	= lpEnv->can.wCodeCol;
	lpVKp->CodeRow	= lpEnv->can.wCodeRow;
	lpVKp->CodeLen	= lpEnv->can.wCodeLen	* DBCS_SIZE;
#endif /* Delete:CodeLen */

	return;
}

/*=============================================================================
	VKpToEnv
=============================================================================*/
LPVKP	VKpToEnv
(
	FXim_Client	*client,
	LPVWIN		lpVWin,
	LPENV		lpEnv
)
{
	LPVKP		lpVKp;
	WORD		wFlag;
	WORD		wFunc;

	lpVKp	= (LPVKP )GetVKp (client);

	wFlag	= LOWORD (lpVWin->Param);
	wFunc	= HIWORD (lpVWin->Param);

	if (!(wFlag & (ENV_ALL|ENV_POS))) {
		return;
	}

	lpEnv->can.wMatrix	= lpVKp->Mtx;
	lpEnv->can.wDirection	= lpVKp->Dir;
	lpEnv->can.wDonCol	= lpVKp->DonCol;
	lpEnv->can.wDonRow	= lpVKp->DonRow;
	lpEnv->can.wDonLen	= lpVKp->DonLen		/ DBCS_SIZE;
#if 0 /* Delete:CodeLen */
	lpEnv->can.wCodeCol	= lpVKp->CodeCol;
	lpEnv->can.wCodeRow	= lpVKp->CodeRow;
	lpEnv->can.wCodeLen	= lpVKp->CodeLen	/ DBCS_SIZE;
#endif /* Delete:CodeLen */

	return;
}

/*=============================================================================
	SetVKpCanMtx
=============================================================================*/
void	SetVKpCanMtx
(
	FXim_Client	*client,
	int		col,
	int		row,
	int		len
)
{
	PBD	bdata;
	LPVKP	lpVKp;

	bdata	= (PBD      )GetBDATA (client);
	lpVKp	= (LPVKP    )GetVKp (client);

#if 0 /* Delete:CodeLen */
	if (bdata->wMainStage & PRC_CODE) {
		lpVKp->CodeCol	= col;
		lpVKp->CodeRow	= row;
		lpVKp->CodeLen	= len;
	} else {
#endif /* Delete:CodeLen */
		lpVKp->DonCol	= col;
		lpVKp->DonRow	= row;
		lpVKp->DonLen	= len;
#if 0 /* Delete:CodeLen */
	}
#endif /* Delete:CodeLen */
}

/*=============================================================================
	GetVKpCanMtx
=============================================================================*/
void	GetVKpCanMtx
(
	FXim_Client	*client,
	int		*col,
	int		*row,
	int		*len
)
{
	PBD	bdata;
	LPVKP	lpVKp;

	bdata	= (PBD      )GetBDATA (client);
	lpVKp	= (LPVKP    )GetVKp (client);

#if 0 /* Delete:CodeLen */
	if (bdata->wMainStage & PRC_CODE) {
		*col	= lpVKp->CodeCol;
		*row	= lpVKp->CodeRow;
		*len	= lpVKp->CodeLen;
	} else {
#endif /* Delete:CodeLen */
		*col	= lpVKp->DonCol;
		*row	= lpVKp->DonRow;
		*len	= lpVKp->DonLen;
#if 0 /* Delete:CodeLen */
	}
#endif /* Delete:CodeLen */
}

/*=============================================================================
	InitVKpCan
=============================================================================*/
void	InitVKpCan
(
	FXim_Client	*client
)
{
	PBD	bdata;
	LPVKP	lpVKp;
	int	col, row, len;
	int	total;

	bdata	= (PBD      )GetBDATA (client);
	lpVKp	= (LPVKP    )GetVKp (client);

	if (lpVKp->Mtx == AUTO) {
		if (bdata->wMainStage & PRC_CODE) {
#if 0 /* Delete:CodeLen */
			lpVKp->CodeCol	= MAX_CAND_COL;
			lpVKp->CodeRow	= MAX_CAND_ROW;
#else /* Delete:CodeLen */
			lpVKp->DonCol	= MAX_CAND_COL;
			lpVKp->DonRow	= MAX_CAND_ROW;
#endif /* Delete:CodeLen */
		} else {
			total	= bdata->cLBItemTotal;
			col	= (int )LSqrt ((LONG )total);
			row	= col;
			if (col * row > MAX_CAND_COL * MAX_CAND_ROW) {
				col	= MAX_CAND_COL;
				row	= MAX_CAND_ROW;
#if 0
			} else if (row > MAX_CAND_ROW) {
#else
			} else if (row >= MAX_CAND_ROW) {
#endif
				row	= MAX_CAND_ROW;
				col	= total
					/ MAX_CAND_ROW
					+((total % MAX_CAND_ROW	)
					? (1			)
					: (0			));
				if (col > MAX_CAND_COL) {
					col	= MAX_CAND_COL;
				}
			} else if (col * row < total) {
				if (lpVKp->Dir == YOKO) {
					if (++col * row < total) {
						row++;
					}
				} else {
					if (col * ++row < total) {
						col++;
					}
				}
			}
			lpVKp->DonCol	= col;
			lpVKp->DonRow	= row;
		}
	}
	return;
}

/*=============================================================================
	CalcVKpCanMtxYES
=============================================================================*/
BOOL	CalcVKpCanMtxYES
(
	FXim_Client	*client
)
{
	BOOL	rc	= FALSE;
	PBD	bdata;
	LPVKP	lpVKp;
	int	root_w, max_w;
	int	col, row, len;

	bdata	= (PBD      )GetBDATA (client);
	lpVKp	= (LPVKP    )GetVKp (client);

	GetVKpCanMtx (client, &col, &row, &len);
	root_w	= DisplayWidth (display, screen)
		- lpVKp->VIdx.width
		- lpVKp->VScr.width
		- DRAW_KOHO_WINDOW_BORDER * 2;
	max_w	= GetMaxKohoWidth (client);
	if (root_w < max_w * col) {
		if (col > 1) {
			col--;
			rc	= TRUE;
		} else {
			col	= 1;
			rc	= FALSE;
		}
	} else {
		rc	= FALSE;
	}
	SetVKpCanMtx (client, col, row, len);
	return (rc);
}

/*=============================================================================
	CalcVKpCanMtxAUTO
=============================================================================*/
BOOL	CalcVKpCanMtxAUTO
(
	FXim_Client	*client
)
{
	BOOL	rc	= FALSE;
	PBD	bdata;
	LPVKP	lpVKp;
	int	col, row, len;
	int	total;

	bdata	= (PBD      )GetBDATA (client);
	lpVKp	= (LPVKP    )GetVKp (client);
	total	= bdata->cLBItemTotal;

	GetVKpCanMtx (client, &col, &row, &len);
	if (col > 1) {
		col--;
		row	= total / col
			+((total % col	)
			? (1		)
			: (0		));
		if (row > MAX_CAND_ROW) {
			row	= MAX_CAND_ROW;
		}
		rc	= TRUE;
	} else if (MAX_CAND_ROW < total) {
		col	= 1;
		row	= MAX_CAND_ROW;
		rc	= FALSE;
	} else {
		col	= 1;
		row	= total;
		rc	= FALSE;
	}
	SetVKpCanMtx (client, col, row, len);
	return (rc);
}

/*=============================================================================
	GetVKpCanRMtx
=============================================================================*/
void	GetVKpCanRMtx
(
	FXim_Client	*client,
	int		*col,
	int		*row,
	int		*len
)
{
	PBD	bdata;
	LPVKP	lpVKp;
	int	number;

	bdata	= (PBD      )GetBDATA (client);
	lpVKp	= (LPVKP    )GetVKp (client);

	GetVKpCanMtx (client, col, row, len);

	if (!(bdata->wMainStage & PRC_CODE)) {
		number	= min (
				bdata->cLBItemTotal
				- bdata->cLBItemPageFirst
				+ 1,
				bdata->cLBItemNumMax
			);
		if (lpVKp->Dir == YOKO) {
			*col	= (number / lpVKp->DonCol	)
				? (lpVKp->DonCol		)
				: (number % lpVKp->DonCol	);
			*row	= (number / lpVKp->DonCol	)
				+((number % lpVKp->DonCol	)
				? (1				)
				: (0				));
		} else {
			*row	= (number / lpVKp->DonRow	)
				? (lpVKp->DonRow		)
				: (number % lpVKp->DonRow	);
			*col	= (number / lpVKp->DonRow	)
				+((number % lpVKp->DonRow	)
				? (1				)
				: (0				));
		}
	}
	return;
}

/*=============================================================================
	GetVKpCanRCol
=============================================================================*/
int	GetVKpCanRCol
(
	FXim_Client	*client,
	int		y
)
{
	PBD	bdata;
	LPVKP	lpVKp;
	int	col, row, len;
	int	number;

	bdata	= (PBD      )GetBDATA (client);
	lpVKp	= (LPVKP    )GetVKp (client);

	GetVKpCanRMtx (client, &col, &row, &len);

	if (!(bdata->wMainStage & PRC_CODE)) {
		number	= min (
				bdata->cLBItemTotal
				- bdata->cLBItemPageFirst
				+ 1,
				bdata->cLBItemNumMax
			);
		if (lpVKp->Dir == YOKO) {
			col	= (y >= lpVKp->DonRow		)
				? ((number - 1) % lpVKp->DonCol + 1	)
				: (lpVKp->DonCol			);
		} else {
			if (y > (number - 1) % lpVKp->DonRow + 1) {
				col	= lpVKp->DonCol - 1;
			}
		}
	}
	return (col);
}

/*=============================================================================
	GetVKpCanRRow
=============================================================================*/
int	GetVKpCanRRow
(
	FXim_Client	*client,
	int		x
)
{
	PBD	bdata;
	LPVKP	lpVKp;
	int	col, row, len;
	int	number;

	bdata	= (PBD      )GetBDATA (client);
	lpVKp	= (LPVKP    )GetVKp (client);

	GetVKpCanRMtx (client, &col, &row, &len);

	if (!(bdata->wMainStage & PRC_CODE)) {
		number	= min (
				bdata->cLBItemTotal
				- bdata->cLBItemPageFirst
				+ 1,
				bdata->cLBItemNumMax
			);
		if (lpVKp->Dir == YOKO) {
			if (x > (number - 1) % lpVKp->DonCol + 1) {
				row	= lpVKp->DonRow - 1;
			}
		} else {
			row	= (x >= lpVKp->DonCol		)
				? ((number - 1) % lpVKp->DonRow + 1	)
				: (lpVKp->DonRow			);
		}
	}
	return (row);
}

/*=============================================================================
	GetVKpCanRPos
=============================================================================*/
void	GetVKpCanRPos
(
	FXim_Client	*client,
	int		*x,
	int		*y
)
{
	PBD	bdata;
	LPVKP	lpVKp;
	int	col, row, len;
	int	number;

	bdata	= (PBD      )GetBDATA (client);
	lpVKp	= (LPVKP    )GetVKp (client);

	GetVKpCanRMtx (client, &col, &row, &len);

	if (lpVKp->Dir == YOKO) {
		number	= bdata->cLBItemNum;
		*x	= ((number - 1) % col)
			+ 1;
		*y	= (number / col )
			+((number % col )
			? (1		)
			: (0		));
	} else {
		*y	= ((number - 1) % row)
			+ 1;
		*x	= (number / row )
			+((number % row )
			? (1		)
			: (0		));
	}
	return;
}

/*=============================================================================
	GetKohoHScrPtr
=============================================================================*/
void	GetKohoHScrPtr
(
	FXim_Client	*client,
	int		*x,
	int		*w
)
{
	LPVKP	lpVKp;
	int	all, cur;
	int	xx, ww, mm;

	lpVKp	= (LPVKP )GetVKp (client);
	all	= lpVKp->HScrAllPage;
	cur	= lpVKp->HScrCurPage;
	xx	= *x;
	ww	= *w;

	*x	= xx + ww * (cur - 1) / all;
	if (all == cur) {
		*w	= xx + ww - *x;
	} else {
		*w	= ww / all;
	}
	if (*w <= DRAW_KOHO_WINDOW_MIN_SCROLL_PTR_SIZE) {
		mm	= DRAW_KOHO_WINDOW_MIN_SCROLL_PTR_SIZE
			- *w;
		*x	= xx + (ww - mm) * (cur - 1) / all;
		if (all == cur) {
			*x	= xx
				+ ww
				- DRAW_KOHO_WINDOW_MIN_SCROLL_PTR_SIZE;
		}
		*w	= DRAW_KOHO_WINDOW_MIN_SCROLL_PTR_SIZE;
	}
}

/*=============================================================================
	GetKohoHScrPageNo
=============================================================================*/
int	GetKohoHScrPageNo
(
	FXim_Client	*client,
	int		xx,
	int		ww,
	int		x
)
{
	LPVKP	lpVKp;
	int	all, cur;

	lpVKp	= (LPVKP )GetVKp (client);
	all	= lpVKp->HScrAllPage;

	cur	=  (x - xx) * all / ww + 1;

	return (cur);
}

/*=============================================================================
	GetKohoVScrPtr
=============================================================================*/
void	GetKohoVScrPtr
(
	FXim_Client	*client,
	int		*y,
	int		*h
)
{
	LPVKP	lpVKp;
	int	all, cur;
	int	yy, hh, mm;

	lpVKp	= (LPVKP )GetVKp (client);
	all	= lpVKp->VScrAllPage;
	cur	= lpVKp->VScrCurPage;
	yy	= *y;
	hh	= *h;

	*y	= yy + hh * (cur - 1) / all;
	if (all == cur) {
		*h	= yy + hh - *y;
	} else {
		*h	= hh / all;
	}
	if (*h <= DRAW_KOHO_WINDOW_MIN_SCROLL_PTR_SIZE) {
		mm	= DRAW_KOHO_WINDOW_MIN_SCROLL_PTR_SIZE
			- *h;
		*y	= yy + (hh - mm) * (cur - 1) / all;
		if (all == cur) {
			*y	= yy
				+ hh
				- DRAW_KOHO_WINDOW_MIN_SCROLL_PTR_SIZE;
		}
		*h	= DRAW_KOHO_WINDOW_MIN_SCROLL_PTR_SIZE;
	}
}

/*=============================================================================
	GetKohoVScrPageNo
=============================================================================*/
int	GetKohoVScrPageNo
(
	FXim_Client	*client,
	int		yy,
	int		hh,
	int		y
)
{
	LPVKP	lpVKp;
	int	all, cur;

	lpVKp	= (LPVKP )GetVKp (client);
	all	= lpVKp->VScrAllPage;

	cur	=  (y - yy) * all / hh + 1;

	return (cur);
}

/*=============================================================================
	ChKohoHScr
=============================================================================*/
BOOL	ChKohoHScr
(
	FXim_Client	*client
)
{
	PBD	bdata;
	LPVKP	lpVKp;
	int	min_w;

	bdata	= (PBD   )GetBDATA (client);
	lpVKp	= (LPVKP )GetVKp (client);
	min_w	= GetMinKohoWidth (client);

	if (!(bdata->wMainStage & PRC_CODE)) {
		if (max (min_w, lpVKp->KohoWidth) > lpVKp->Don.width) {
			return (TRUE);
		}
	}
	return (FALSE);
}

/*=============================================================================
	ChKohoVScr
=============================================================================*/
BOOL	ChKohoVScr
(
	FXim_Client	*client
)
{
	PBD	bdata;
	LPVKP	lpVKp;
	int	don_h;
	int	scr_h;

	bdata	= (PBD      )GetBDATA (client);
	lpVKp	= (LPVKP    )GetVKp (client);

	if (!(bdata->wMainStage & PRC_CODE)) {
		scr_h	= DRAW_KOHO_WINDOW_SCROLL_BAR_SIZE * 2
			+ DRAW_KOHO_WINDOW_MIN_SCROLL_PTR_SIZE;
		don_h	= lpVKp->Don.height * lpVKp->DonRow;
		if (don_h < scr_h) {
			return (FALSE);
		}
#if 0 /* VSCR-DISPLAY */
		return (TRUE);
#else
		if (bdata->cLBItemTotal > lpVKp->DonCol * lpVKp->DonRow) {
			return (TRUE);
		}
#endif
	}
	return (FALSE);
}

/*=============================================================================
	ChKohoHist
=============================================================================*/
BOOL	ChKohoHist
(
	FXim_Client	*client
)
{
#if 1 /* 990723:KOHO-HIST */
	return (FALSE);
#else
	return (TRUE);
#endif
}

/*=============================================================================
	GetCyumokuToWidth
=============================================================================*/
int	GetCyumokuToWidth
(
	FXim_Client	*client
)
{
	PBD	bdata;
	UCHAR	sjis[MAX_COMP_SIZE+1];
	UCHAR	euc[MAX_COMP_SIZE+1];
	UCHAR 	ch;
	int	cnt, width;
	int	i;

	bdata	= (PBD )GetBDATA (client);

	sjis[0]	= 0;
	for (i=0; i < (int)bdata->cchDisplay; i++) {
		ch	= (UCHAR )*(bdata->lpattrDisplay + i);
		if (ch ==  CL_HENKAN_1
		 || ch == (CL_HENKAN_1|CL_NODISP)) {
			lstrncpy (sjis, bdata->lpchDisplay, i);
			break;
		}
	}
	cnt	= ShiftjisToEuc (euc, sjis);
	width	= FXChTextWidth (client->preedit_font, euc, cnt);

	return (width);		
}

/*=============================================================================
	GetMinKohoWidth
=============================================================================*/
int	GetMinKohoWidth
(
	FXim_Client	*client
)
{
	PBD	bdata;
	LPVKP	lpVKp;
	LPVWIN	lpVWin;
	LPVKPLT	lpVKplt;
	UCHAR	euc[MAX_COMP_SIZE+1];
	UCHAR	*sjis;
	int	base_w;
	int	root_w1, root_w2;
	int	koho_w, min_w;
	int	col, row, len;
	int	cnt, tmp_w;
	int	max_w, kanji_w, kana_w;
	int	i;

	bdata	= (PBD      )GetBDATA (client);
	lpVKp	= (LPVKP    )GetVKp (client);
	lpVWin	= (LPVWIN   )GetVWin ();
	lpVKplt	= (LPVKPLT  )GetVKplt (lpVWin);

	GetVKpCanMtx (client, &col, &row, &len);

	koho_w	= 0;
	kanji_w	= client->koho_font->kanji_font_width;
	kana_w	= client->koho_font->ascii_kana_font_width;
	max_w	= (kanji_w > kana_w	)
		? (kanji_w		)
		: (kana_w		);
	base_w	= lpVKp->Btn[0].width * lpVKplt->wBtnCount
		- lpVKp->VIdx.width
		- lpVKp->VScr.width;

	min_w	= base_w
		+ kana_w * KOHO_TITLE_LEN;

#if 0 /* debug */
	root_w1	= DisplayWidth (display, screen) / 4;
#else
	root_w1	= DisplayWidth (display, screen);
#endif
	root_w2	= root_w1
		- lpVKp->VIdx.width
		- lpVKp->VScr.width
		- DRAW_KOHO_WINDOW_BORDER * 2;

	if (ChKohoHist (client)) {
		tmp_w	= base_w;
#if 0 /* 990723:KOHO-HIST */
		sjis	= (char *)bdata->XXXXXXXXXX;
		cnt	= ShiftjisToEuc (euc, sjis);
#else
		lstrcpy (euc, "TEST...");
		cnt	= lstrlen (euc);
#endif
		tmp_w	+= FXChTextWidth (client->koho_font, euc, cnt);
		if (min_w < tmp_w) {
			min_w	= tmp_w;
		}
	}

	if (lpVKp->Mtx == NO) {
		koho_w	= min_w;
	} else if (lpVKp->Mtx == YES) {
		min_w	= (min_w / col	)
			+((min_w % col	)
			? (1		)
			: (0		));
		koho_w	= max_w  * len / DBCS_SIZE
			+ kana_w * KOHO_MTRBDR_LEN;
		if (koho_w < min_w) {
			koho_w	= min_w;
		}
		if (koho_w > root_w2) {
			koho_w = root_w2;
		}
	} else {
		min_w	= (min_w / col	)
			+((min_w % col	)
			? (1		)
			: (0		));
		koho_w	= max_w
			+ kana_w * KOHO_MTRBDR_LEN;
		if (koho_w < min_w) {
			koho_w	= min_w;
		}
	}

	return (koho_w);
}

/*=============================================================================
	GetMaxKohoWidth
=============================================================================*/
int	GetMaxKohoWidth
(
	FXim_Client	*client
)
{
	LPVKP	lpVKp;
	int	col, row, len;
	int	koho_w;
	int	root_w1, root_w2;
	int	min_w, max_w;
	int	kanji_w, kana_w;

	lpVKp	= (LPVKP    )GetVKp (client);

	GetVKpCanMtx (client, &col, &row, &len);

	kanji_w	= client->koho_font->kanji_font_width;
	kana_w	= client->koho_font->ascii_kana_font_width;
	max_w	= (kanji_w > kana_w	)
		? (kanji_w		)
		: (kana_w		);
	min_w	= GetMinKohoWidth (client);

#if 0 /* debug */
	root_w1	= DisplayWidth (display, screen) / 4;
#else
	root_w1	= DisplayWidth (display, screen);
#endif
	root_w2	= root_w1
		- lpVKp->VIdx.width
		- lpVKp->VScr.width
		- DRAW_KOHO_WINDOW_BORDER * 2;

	if (lpVKp->Mtx == NO) {
		koho_w	= root_w2;
	} else if (lpVKp->Mtx == YES) {
		koho_w	= max_w * len / DBCS_SIZE;
		if (koho_w < min_w) {
			koho_w	= min_w;
		}
		if (koho_w > root_w2) {
			koho_w = root_w2;
		}
	} else {
		koho_w	= root_w2 / col;
	}

	return (koho_w);
}

/*=============================================================================
	GetKohoWidth
=============================================================================*/
int	GetKohoWidth
(
	FXim_Client	*client
)
{
	PBD	bdata;
	LPVKP	lpVKp;
	UCHAR	euc[MAX_COMP_SIZE+1];
	UCHAR	*sjis;
	int	koho_w;
	int	index, length;
	int	cnt, tmp_w;
	int	max_w, kanji_w, kana_w;

	bdata	= (PBD      )GetBDATA (client);
	lpVKp	= (LPVKP    )GetVKp (client);

	FuncKohoMax (client, bdata->cLBItemTotal, 1);

	kanji_w	= client->koho_font->kanji_font_width;
	kana_w	= client->koho_font->ascii_kana_font_width;
	max_w	= (kanji_w > kana_w	)
		? (kanji_w		)
		: (kana_w		);
	koho_w	= 0;

	for (index=0; index < (int)bdata->cchLBListLen;) {
		length	= lstrlen ((char *)&bdata->lpchLBList[index]);
		sjis	= (char *)&bdata->lpchLBList[index];
		cnt	= ShiftjisToEuc (euc, sjis);
		tmp_w	= FXChTextWidth (client->koho_font, euc, cnt);
		if (koho_w < tmp_w) {
			koho_w	= tmp_w;
		}
		index	+= length + 1;
	}
	if (lpVKp->Mtx == AUTO
	 || lpVKp->Mtx == YES) {
		koho_w	+= kana_w * KOHO_MTRBDR_LEN;
	}

	return (koho_w);
}

/*=============================================================================
	KohoWindowOpen
=============================================================================*/
void	KohoWindowOpen
(
	FXim_Client	*client
)
{
	PBD	bdata;
	LPVKP	lpVKp;
	LPVWIN	lpVWin;
	LPVKPLT	lpVKplt;
	Window	r_win, win;
	int	spotx, spoty;
	int	startx, starty, endx, endy;
	UINT	width, height, b_width, depth;
	int	don_w, max_w, min_w;
	int	font_h, kana_w, kana_h;
	int	ascent, w_cyu;
	int	col, row, len;
	int	all_btn_w, btn_w, btn_h;
	int	i;

	bdata	= (PBD      )GetBDATA (client);
	lpVKp	= (LPVKP    )GetVKp (client);
	lpVWin	= (LPVWIN   )GetVWin ();
	lpVKplt	= (LPVKPLT  )GetVKplt (lpVWin);

	font_h	= client->koho_font->max_font_height;
	kana_w	= client->koho_font->ascii_kana_font_width;
	kana_h	= client->koho_font->ascii_kana_font_struct->ascent
		+ client->koho_font->ascii_kana_font_struct->descent;
	ascent	= client->preedit->LineSpacing;

	InitVKpCan (client);

	lpVKp->KohoWidth	= GetKohoWidth (client);

KohoWindowOpen_AutoMatrix_Retry:;

	GetVKpCanMtx (client, &col, &row, &len);

	lpVKp->Don.height	= font_h
				+ DRAW_KOHO_WINDOW_FONT_BORDER * 2;

	lpVKp->VIdx.width	= kana_w
				+ DRAW_KOHO_WINDOW_VIDX_BORDER * 2;
	lpVKp->VIdx.height	= lpVKp->Don.height;

	btn_w			= max (
					lpVKplt->wPixBtnSize
					+ DRAW_KOHO_WINDOW_BUTTON_BORDER * 2,
					kana_h
				);
	btn_h			= btn_w;
	for (i=0; i < lpVKplt->wBtnCount; i++) {
		lpVKp->Btn[i].width	= btn_w;
		lpVKp->Btn[i].height	= btn_h;
	}
	all_btn_w		= btn_w * lpVKplt->wBtnCount;

	if (ChKohoVScr (client)) {
		lpVKp->VScr.width	= DRAW_KOHO_WINDOW_SCROLL_BAR_SIZE;
		lpVKp->VScr.height	= lpVKp->Don.height * row;
	} else {
		lpVKp->VScr.width	= 0;
		lpVKp->VScr.height	= 0;
	}

	max_w	= GetMaxKohoWidth (client);
	min_w	= GetMinKohoWidth (client);

	if (lpVKp->Mtx == NO) {
		don_w	= max (min_w, min (lpVKp->KohoWidth, max_w));
	} else if (lpVKp->Mtx == YES) {
		if (CalcVKpCanMtxYES (client)) {
			goto	KohoWindowOpen_AutoMatrix_Retry;
		}
		don_w	= max (min_w, min (lpVKp->KohoWidth, max_w));
	} else {
		don_w	= max (min_w, lpVKp->KohoWidth);
		if (don_w > max_w) {
			if (CalcVKpCanMtxAUTO (client)) {
				goto	KohoWindowOpen_AutoMatrix_Retry;
			}
			don_w	= max_w;
		}
	}
	lpVKp->Don.width	= don_w;

	FuncKohoMax (client, col, row);

	if (col > 1) {
		lpVKp->HIdx.width	= lpVKp->Don.width;
		lpVKp->HIdx.height	= kana_h
					+ DRAW_KOHO_WINDOW_HIDX_BORDER * 2;
	} else {
		lpVKp->HIdx.width	= 0;
		lpVKp->HIdx.height	= 0;
	}

	if (ChKohoHScr (client)) {
		lpVKp->HScr.width	= lpVKp->Don.width * col;
		lpVKp->HScr.height	= DRAW_KOHO_WINDOW_SCROLL_BAR_SIZE;
	} else {
		lpVKp->HScr.width	= 0;
		lpVKp->HScr.height	= 0;
	}

	if (ChKohoHist (client)) {
		lpVKp->Hist.width	= lpVKp->Don.width * col;
		lpVKp->Hist.height	= lpVKp->Don.height;
	} else {
		lpVKp->Hist.width	= 0;
		lpVKp->Hist.height	= 0;
	}

	lpVKp->Code.width	= lpVKp->Don.width * col
				+ lpVKp->VIdx.width
				+ lpVKp->VScr.width
				- all_btn_w;
	lpVKp->Code.height	= btn_h;

	lpVKp->Win.width	= DRAW_KOHO_WINDOW_BORDER
				+ lpVKp->VIdx.width
				+ lpVKp->Don.width * col
				+ lpVKp->VScr.width
				+ DRAW_KOHO_WINDOW_BORDER;
	lpVKp->Win.height	= DRAW_KOHO_WINDOW_BORDER
				+ lpVKp->HIdx.height
				+ DRAW_KOHO_WINDOW_DON_BORDER
				+ lpVKp->Don.height * row
				+ lpVKp->HScr.height
				+ DRAW_KOHO_WINDOW_DON_BORDER
				+ lpVKp->Hist.height
				+((lpVKp->Hist.height > 0	)
				? (DRAW_KOHO_WINDOW_DON_BORDER	)
				: (0				))
				+ DRAW_KOHO_WINDOW_TITLE_BORDER
				+ btn_h
				+ DRAW_KOHO_WINDOW_BORDER;

	if (client->style & XIMPreeditPosition
	 || client->style & XIMPreeditNothing) {
		win	= client->preedit_window;
		spotx	= 0;
		spoty	= ascent;
		endx	= DisplayWidth (display, screen);

	} else if (client->style & XIMPreeditArea) {
		win	= client->window;
		spotx	= client->preedit->Area.x;
		spoty	= client->preedit->Area.y
			+ ascent
			+ (client->preedit->Area.height -
			   client->preedit_font->max_font_height + 1) / 2;
		endx	= DisplayWidth (display, screen);

	/*990125 OnTheSpot*/
	} else if (client->style & XIMPreeditCallbacks) {
		win	= client->focus_window;
		XGetGeometry (
			display, win, &r_win,
			(int *)&startx, (int *)&starty,
			&width, &height, &b_width, &depth
		);
		spotx	= 0;
		spoty	= height;
		endx	= DisplayWidth (display, screen);

	} else {
		win	= root_client->preedit_window;
		spotx	= 0;
		spoty	= 0;
		endx	= DisplayWidth (display, screen);
	}

	XTranslateCoordinates (
		display, win, RootWindow (display, 0),
		spotx, spoty, &startx, &starty, &r_win
	);

	if (!(bdata->wMainStage & PRC_CODE)) {
		w_cyu	= GetCyumokuToWidth (client);
		if (client->style & XIMPreeditPosition ||
		    client->style & XIMPreeditNothing) {
			for (i=0; i < PREEDIT_MAX; i++) {
				if (w_cyu < client->preedit_area[i].width) {
					startx	+= client->preedit_area[i].x
						 - client->preedit_area[0].x;
					break;
				}
				w_cyu	-= client->preedit_area[i].width;
				starty	+= ascent;
			}
		}
		startx	       += w_cyu;
	}
	if (!(client->style & XIMPreeditCallbacks)) {
		startx	-= lpVKp->VIdx.width
			 + DRAW_KOHO_WINDOW_BORDER;
		if (startx < 0) {
			startx	= 0;
		}
	}

	endx	= DisplayWidth  (display, screen);
	if (startx + lpVKp->Win.width  > endx) {
		startx	= endx - lpVKp->Win.width;
	}
	endy	= DisplayHeight (display, screen);
	if (starty + lpVKp->Win.height > endy) {
		starty	= starty - ascent - lpVKp->Win.height;
		lpVKp->Direction	= 1;
	} else {
		lpVKp->Direction	= 0;
	}

	lpVKp->VIdx.x	= DRAW_KOHO_WINDOW_BORDER;
	lpVKp->HIdx.x	= lpVKp->VIdx.x
			+ lpVKp->VIdx.width;
	lpVKp->Don.x	= lpVKp->HIdx.x;
	lpVKp->VScr.x	= lpVKp->Don.x
			+ lpVKp->Don.width * col;
	lpVKp->HScr.x	= lpVKp->Don.x;
	lpVKp->Hist.x	= lpVKp->Don.x;
	lpVKp->Btn[0].x	= lpVKp->VIdx.x;
	for (i=0; i < lpVKplt->wBtnCount - 1; i++) {
		lpVKp->Btn[i+1].x	= lpVKp->Btn[i].x
					+ lpVKp->Btn[i].width;
	}
	lpVKp->Code.x	= lpVKp->Btn[i].x
			+ lpVKp->Btn[i].width;
	lpVKp->Win.x	= startx;
	if (lpVKp->Direction == 1) {
		lpVKp->HIdx.y	= lpVKp->Win.height
				- lpVKp->HIdx.height
				- DRAW_KOHO_WINDOW_BORDER;
		lpVKp->VIdx.y	= lpVKp->HIdx.y
				- lpVKp->Don.height * row
				- DRAW_KOHO_WINDOW_DON_BORDER;
		lpVKp->Don.y	= lpVKp->VIdx.y;
		lpVKp->HScr.y	= lpVKp->Don.y
				- lpVKp->HScr.height;
		lpVKp->VScr.y	= lpVKp->Don.y;
		lpVKp->Hist.y	= lpVKp->HScr.y
				- lpVKp->Hist.height
				- DRAW_KOHO_WINDOW_DON_BORDER;
		lpVKp->Btn[0].y	= lpVKp->Hist.y
				- lpVKp->Btn[0].height
				- DRAW_KOHO_WINDOW_TITLE_BORDER;
		if (ChKohoHist (client)) {
			lpVKp->Btn[0].y	-= DRAW_KOHO_WINDOW_DON_BORDER;
		}
		for (i=0; i < lpVKplt->wBtnCount - 1; i++) {
			lpVKp->Btn[i+1].y	= lpVKp->Btn[i].y;
		}
		lpVKp->Code.y	= lpVKp->Btn[0].y;
	} else {
		lpVKp->HIdx.y	= DRAW_KOHO_WINDOW_BORDER;
		lpVKp->VIdx.y	= lpVKp->HIdx.y
				+ lpVKp->HIdx.height
				+ DRAW_KOHO_WINDOW_DON_BORDER;
		lpVKp->Don.y	= lpVKp->VIdx.y;
		lpVKp->HScr.y	= lpVKp->Don.y
				+ lpVKp->Don.height * row;
		lpVKp->VScr.y	= lpVKp->Don.y;
		lpVKp->Hist.y	= lpVKp->HScr.y
				+ lpVKp->HScr.height
				+ DRAW_KOHO_WINDOW_DON_BORDER;
		lpVKp->Btn[0].y	= lpVKp->Hist.y
				+ lpVKp->Hist.height
				+ DRAW_KOHO_WINDOW_TITLE_BORDER;
		if (ChKohoHist (client)) {
			lpVKp->Btn[0].y	+= DRAW_KOHO_WINDOW_DON_BORDER;
		}
		for (i=0; i < lpVKplt->wBtnCount - 1; i++) {
			lpVKp->Btn[i+1].y	= lpVKp->Btn[i].y;
		}
		lpVKp->Code.y	= lpVKp->Btn[0].y;
	}
	lpVKp->Win.y	= starty;

	lpVKp->VScrAllPage	= bdata->cLBItemTotal
				/ (row * col)
				+((bdata->cLBItemTotal % (row * col)	)
				? (1					)
				: (0					));
	lpVKp->VScrCurPage	= (bdata->cLBItemPageFirst - 1)
				/ (row * col)
				+ 1;
	lpVKp->HScrAllPage	= lpVKp->KohoWidth
				/ lpVKp->Don.width
				+((lpVKp->KohoWidth % lpVKp->Don.width	)
				? (1					)
				: (0					));
	lpVKp->HScrCurPage	= 1;

	XMoveResizeWindow (
		display,
		client->koho_window,
		startx,
		starty,
		lpVKp->Win.width,
		lpVKp->Win.height
	); 

	if (lpVKp->Visible != 1) {
		lpVKp->save_lpchLBTitle[0]	= (UCHAR )0;
		lpVKp->save_lpchLBList[0]	= (UCHAR )0;
		lpVKp->save_cLBItemNum		= 0;
		lpVKp->DrawFlag			= VKP_DRAW_NONE;
	}
	lpVKp->Visible	= 1;
}

/*=============================================================================
	KohoWindowClose
=============================================================================*/
void	KohoWindowClose
(
	FXim_Client	*client
)
{
	LPVKP	lpVKp;

	lpVKp	= (LPVKP )GetVKp (client);

	XUnmapWindow (display, client->koho_window);

	lpVKp->save_lpchLBTitle[0]	= (UCHAR )0;
	lpVKp->save_lpchLBList[0]	= (UCHAR )0;
	lpVKp->save_cLBItemNum		= 0;
	lpVKp->DrawFlag			= VKP_DRAW_NONE;
	lpVKp->Visible			= 0;
}

/*=============================================================================
	GetKohoPageNo
=============================================================================*/
int	GetKohoPageNo
(
	FXim_Client	*client,
	UCHAR		*euc
)
{
	PBD	bdata;
	UCHAR	sjis[MAX_COMP_SIZE+1];
	char	buf[10];
	int	cnt, size;

	bdata	= (PBD )GetBDATA (client);

	if (bdata->wMainStage & PRC_CODE) {
		;
	} else {
		sprintf (sjis, "%d/%d",
			bdata->cLBItemPageFirst
			+ bdata->cLBItemNum
			- 1,
			bdata->cLBItemTotal
		);
	}
	cnt	= ShiftjisToEuc (euc, sjis);
	size	= FXChTextWidth (client->koho_font, euc, cnt);

	return (size);
}

/*=============================================================================
	GetKohoTitle
=============================================================================*/
int	GetKohoTitle
(
	FXim_Client	*client,
	UCHAR		*euc
)
{
	PBD	bdata;
	UCHAR	sjis[MAX_COMP_SIZE+1];
	char	buf[10];
	int	cnt, size;

	bdata	= (PBD )GetBDATA (client);

	if (bdata->wMainStage & PRC_CODE) {
		if (bdata->wHenModFlag & MOD_SJIS) {
			strcpy (buf, "SJIS:");
		} else if (bdata->wHenModFlag & MOD_JIS) {
			strcpy (buf, "JIS:");
#if 0
		} else if (bdata->wHenModFlag & MOD_UNICODE) {
			strcpy (buf, "UNICODE:");
#endif
		} else if (bdata->wHenModFlag & MOD_EUC) {
			strcpy (buf, "EUC:");
		} else {
#if 1
			strcpy (buf, "KUTEN:");
#else
			buf[0]	= 0x8b; buf[1]	= 0xe6;
			buf[2]	= 0x93; buf[3]	= 0x5f;
			buf[4]	= ':';
			buf[5]	= 0;
#endif
		}
		sprintf (sjis, "%s%s", buf, bdata->lpchLBTitle);
	} else {
		sprintf (sjis, "%s", bdata->lpchLBTitle);
	}
	cnt	= ShiftjisToEuc (euc, sjis);
	size	= FXChTextWidth (client->koho_font, euc, cnt);

	return (size);
}

/*=============================================================================
	DisplayKohoTitle
=============================================================================*/
void	DisplayKohoTitle
(
	FXim_Client	*client,
	LPVWIN		lpVWin
)
{
	PBD	bdata;
	LPVKP	lpVKp;
	UCHAR	euc[MAX_COMP_SIZE+1];
	GC	*gc;
	ULONG	fore, back;
	int	size, attr;
	int	x, y, w, h;
	int	kana_h;

	bdata	= (PBD   )GetBDATA (client);
	lpVKp	= (LPVKP )GetVKp (client);

	gc	= lpVWin->GCList;
	fore	= lpVWin->WinForePixel;
	back	= lpVWin->WinBackPixel;

	x	= lpVKp->Code.x;
	y	= lpVKp->Code.y;
	w	= lpVKp->Code.width;
	h	= lpVKp->Code.height;

	attr	= 2;
	kana_h	= client->koho_font->ascii_kana_font_struct->ascent
		+ client->koho_font->ascii_kana_font_struct->descent;

	XSetForeground (display, gc[5], fore);
	XSetBackground (display, gc[5], back);

	if (bdata->wMainStage & PRC_CODE) {
		size	= GetKohoTitle (client, euc);
		DrawEucTextAttrGcString (
			display,
			client->koho_window,
			client->koho_font,
			x + (w - size) / 2,
			y + h
			  - client->koho_font->ascii_kana_font_struct->descent
			  - (h - kana_h) / 2,
			euc,
			gc[5],
			strlen (euc)
		);
	} else {
		size	= GetKohoPageNo (client, euc);
		XFillRectangle (
			display,
			client->koho_window,
			gc[0],
			x, y, w, h
		);
		DrawEucTextAttrGcString (
			display,
			client->koho_window,
			client->koho_font,
			x + w - size,
			y + h
			  - client->koho_font->ascii_kana_font_struct->descent
			  - (h - kana_h) / 2,
			euc,
			gc[5],
			strlen (euc)
		);
	}
}

/*=============================================================================
	MakeKohoString
=============================================================================*/
void	MakeKohoString
(
	FXim_Client	*client,
	int		number,
	char		*out,
	char		*in
)
{
	LPVKP	lpVKp;
	UCHAR	buf[MAX_COMP_SIZE+1];
	int	jlen, cnt, size;
	int	col, row, len;
	int	don_w, cur;
	int	i, j, k, l;

	lpVKp	= (LPVKP    )GetVKp (client);
	don_w	= lpVKp->Don.width;
	cur	= lpVKp->HScrCurPage;

	GetVKpCanMtx (client, &col, &row, &len);

	k	= 0;
	for (i=1; i < cur; i++) {
		l	= 0;
		jlen	= EucJstrLen (in + k);
		for (j=1; j <= jlen; j++) {
			cnt	= EucJstrCpy (out, in + k, j);
			size	= FXChTextWidth (
					client->koho_font,
					out,
					cnt
				);
			if (size > don_w) {
				break;
			}
			l	= cnt;
		}
		k	+= l;
	}
	jlen	= EucJstrLen (in + k);
	if (jlen > 0) {
		for (j=jlen; j >= 1; j--) {
			cnt	= EucJstrCpy (out, in + k, j);
			size	= FXChTextWidth (
					client->koho_font,
					out,
					cnt
				);
			if (size <= don_w) {
				break;
			}
		}
	} else {
		*(out + 0)	= 0x20;
		*(out + 1)	= 0x00;
	}
}

/*=============================================================================
	DisplayKohoString
=============================================================================*/
int	DisplayKohoString
(
	FXim_Client	*client,
	int		index,
	int		i,
	int		j,
	int		k
)
{
	PBD	bdata;
	LPVKP	lpVKp;
	UCHAR	euc[MAX_COMP_SIZE+1];
	UCHAR	buf[MAX_COMP_SIZE+1];
	int	length, count, attr;

	bdata	= (PBD   )GetBDATA (client);
	lpVKp	= (LPVKP )GetVKp (client);

	if (index >= (int)bdata->cchLBListLen) {
		return (0);
	}

	length	= lstrlen ((char *)&bdata->lpchLBList[index]);
	count	= ShiftjisToEuc (euc, (char *)&bdata->lpchLBList[index]);

	if (bdata->cLBItemNum == k) {
		attr	= CL_EX_LISTCUR;
	} else if (lpVKp->DrawFlag == VKP_DRAW_WIN_ALL
		|| lpVKp->DrawFlag == VKP_DRAW_DON_ALL
		|| lpVKp->save_cLBItemNum == k) {
		attr	= CL_EX_LIST;
	} else {
		attr	= -1;
	}

	if (attr != -1) {
		SetVclrAttr (
			display,
			client->koho_font->normal_gc,
			attr,
			1
		);
		XFillRectangle (
			display,
			client->koho_window,
			client->koho_font->normal_gc,
			lpVKp->Don.x + lpVKp->Don.width  * (j-1),
			lpVKp->Don.y + lpVKp->Don.height * (i-1),
			lpVKp->Don.width,
			lpVKp->Don.height
		);
		SetVclrAttr (
			display,
			client->koho_font->normal_gc,
			attr,
			0
		);
		MakeKohoString (client, k, buf, euc);
		DrawEucTextAttrString (
			display,
			client->koho_window,
			client->koho_font,
			lpVKp->Don.x + lpVKp->Don.width  * (j-1),
			lpVKp->Don.y + lpVKp->Don.height * i
			  - DRAW_KOHO_WINDOW_FONT_BORDER
			  - client->koho_font->max_font_descent,
			buf,
			attr,
			lstrlen (buf)
		);
	}

	return (length);
}

/*=============================================================================
	ClearKohoString
=============================================================================*/
void	ClearKohoString
(
	FXim_Client	*client,
	int		i,
	int		j
)
{
	LPVKP	lpVKp;
	int	attr;

	lpVKp	= (LPVKP )GetVKp (client);

	attr	= CL_EX_LIST;

	SetVclrAttr (
		display,
		client->koho_font->normal_gc,
		attr,
		1
	);
	XFillRectangle (
		display,
		client->koho_window,
		client->koho_font->normal_gc,
		lpVKp->Don.x + lpVKp->Don.width  * (j-1),
		lpVKp->Don.y + lpVKp->Don.height * (i-1),
		lpVKp->Don.width,
		lpVKp->Don.height
	);
}

/*=============================================================================
	KohoWindowDisplay
=============================================================================*/
void	KohoWindowDisplay
(
	FXim_Client	*client
)
{
	PBD	bdata;
	LPVWIN	lpVWin;
	LPVKP	lpVKp;
	int	index, length;
	int	col, row, len;
	int	i, j, k, l;


	bdata	= (PBD      )GetBDATA (client);
	lpVWin	= (LPVWIN   )GetVWin ();
	lpVKp	= (LPVKP    )GetVKp (client);

	if (lpVKp->Visible == 0) {
		return;
	}

	if (bdata->wMainStage & PRC_CODE) {
		if (0 != lstrcmp ((LPTSTR )lpVKp->save_lpchLBList,
				  (LPTSTR )&bdata->lpchLBList[0])) {
			lpVKp->DrawFlag	= VKP_DRAW_WIN_ALL;
		} else if (lpVKp->DrawFlag == VKP_DRAW_WIN_ALL
			|| lpVKp->DrawFlag == VKP_DRAW_DON_ALL) {
			;
		} else if (lpVKp->DrawFlag == VKP_DRAW_VSCR
			|| lpVKp->DrawFlag == VKP_DRAW_HSCR) {
			lpVKp->DrawFlag	= VKP_DRAW_DON_ALL;
		} else if (lpVKp->save_cLBItemNum != bdata->cLBItemNum) {
			lpVKp->DrawFlag	= VKP_DRAW_DON_CUR;
		} else {
			lpVKp->DrawFlag	= VKP_DRAW_NONE;
			return;
		}
	} else {
		if (bdata->cLBItemNumMax == DOU_MAX) {
			lpVKp->DrawFlag	= VKP_DRAW_WIN_ALL;
			KohoWindowOpen (client);
		} else if (0 != lstrcmp ((LPTSTR )lpVKp->save_lpchLBTitle,
					 (LPTSTR )&bdata->lpchLBTitle[0])) {
			lpVKp->DrawFlag	= VKP_DRAW_WIN_ALL;
			KohoWindowOpen (client);
		} else if (0 != lstrcmp ((LPTSTR )lpVKp->save_lpchLBList,
					 (LPTSTR )&bdata->lpchLBList[0])) {
			lpVKp->DrawFlag	= VKP_DRAW_DON_ALL;
		} else if (lpVKp->DrawFlag == VKP_DRAW_WIN_ALL
			|| lpVKp->DrawFlag == VKP_DRAW_DON_ALL) {
			;
		} else if (lpVKp->DrawFlag == VKP_DRAW_VSCR
			|| lpVKp->DrawFlag == VKP_DRAW_HSCR) {
			lpVKp->DrawFlag	= VKP_DRAW_DON_ALL;
		} else if (lpVKp->save_cLBItemNum != bdata->cLBItemNum) {
			lpVKp->DrawFlag	= VKP_DRAW_DON_CUR;
		} else {
			lpVKp->DrawFlag	= VKP_DRAW_NONE;
			return;
		}
	}

	GetVKpCanMtx (client, &col, &row, &len);
	lpVKp->VScrAllPage	= bdata->cLBItemTotal
				/ (row * col)
				+((bdata->cLBItemTotal % (row * col)	)
				? (1					)
				: (0					));
	lpVKp->VScrCurPage	= (bdata->cLBItemPageFirst - 1)
				/ (row * col)
				+ 1;
	lpVKp->HScrAllPage	= lpVKp->KohoWidth
				/ lpVKp->Don.width
				+((lpVKp->KohoWidth % lpVKp->Don.width	)
				? (1					)
				: (0					));
/* 990219
 * ΥɥʤȤ˸¤̤ʸɽ롣
 * 䥦ɥʤ뤳ȤǸʤ㤤ʤ̤ʸ뤿ᡣ
 * оʸ̤ʸȤ롣
 * PreeditCallbacksоݳ
 */
	if (!(client->style & XIMPreeditCallbacks)) {
		if (lpVKp->Win.width < client->koho_width) {
			Clear_NODISP (client);
			client->input_flag	= FINF_MODY;
			DisplayOutputBuff ();
		}
		client->koho_width	= lpVKp->Win.width;
	}
/**/

#if 1 /* 991217:First-Display */
	XMapRaised (display, client->koho_window);
#endif

	k	= 1;
	index	= 0;
	if (lpVKp->Dir == YOKO) {
		for (i=1; i <= row; i++) {
		for (j=1; j <= col; j++) {
			length	= DisplayKohoString (client, index, i, j, k);
			if (!length) {
				for (; i <= row; i++, j=1) {
				for (; j <= col; j++) {
					ClearKohoString (client, i, j);
				} /* j */
				} /* i */
			} else {
				index	+= length + 1;
				k	+= 1;
			}
		} /* j */
		} /* i */
	} else {
		for (j=1; j <= col; j++) {
		for (i=1; i <= row; i++) {
			length	= DisplayKohoString (client, index, i, j, k);
			if (!length) {
				for (; j <= col; j++, i=1) {
				for (; i <= row; i++) {
					ClearKohoString (client, i, j);
				} /* i */
				} /* j */
			} else {
				index	+= length + 1;
				k	+= 1;
			}
		} /* i */
		} /* j */
	}

	switch (lpVKp->DrawFlag) {
	case VKP_DRAW_WIN_ALL:
		DrawKohoWindow (client, lpVWin);
		break;
	case VKP_DRAW_DON_ALL:
		if (ChKohoVScr (client)) {
			DrawKohoVScrPtr (client, lpVWin);
		}
		if (ChKohoHScr (client)) {
			DrawKohoHScrPtr (client, lpVWin);
		}
		break;
	}
	DisplayKohoTitle (client, lpVWin);

	lstrcpy (
		(LPTSTR )lpVKp->save_lpchLBTitle,
		(LPTSTR )&bdata->lpchLBTitle[0]
	);
	lstrcpy (
		(LPTSTR )lpVKp->save_lpchLBList,
		(LPTSTR )&bdata->lpchLBList[0]
	);
	lpVKp->save_cLBItemNum	= bdata->cLBItemNum;
	lpVKp->DrawFlag		= VKP_DRAW_NONE;

	XMapRaised (display, client->koho_window);

	XFlush (display);
}

/*=============================================================================
	InitKohoWindow
=============================================================================*/
BOOL	InitKohoWindow
(
	LPVKPLT	lpVKplt,
	LPENV	lpEnv
)
{
	BOOL	rc;

	rc	= CreateVKpltPixBtnList (lpVKplt);
	if (!rc) {
		return (FALSE);
	}
}
