#if !USE_XPM
/******************************************************************************
	Xpixmap.C
******************************************************************************/
#define		XGRAB_SERVER	1
#define		XK_KATAKANA
#include	<X11/keysym.h>
#include	<X11/cursorfont.h>
#include	"typedef.h"
#include	"xim.h"
#include	"xui.h"
#include	"xpixmap.h"
#include	"macro.h"
#include	"fileio.h"
#include	"env.h"
#include	"getstr.h"
#include	"strutl.h"

/*=============================================================================
	XPixmapExtractString
=============================================================================*/
int	XPixmapExtractString (TCHAR dst[], TCHAR src[])
{
	USHORT	i;
	for (i=0; src[i] && src[i] != 0x20 && src[i] != 0x09; i++);
	lstrncpy (dst, src, i);
	dst[i]	= (TCHAR )0;
	return (i);
}

/*=============================================================================
	XPixmapSkipString
=============================================================================*/
LPTCHAR	XPixmapSkipString (TCHAR src[])
{
	USHORT	i;
	for (i=0; src[i] && src[i] != 0x20 && src[i] != 0x09; i++);
	return (&src[i]);
}

/*=============================================================================
	XPixmapSkipSpace
=============================================================================*/
LPTCHAR	XPixmapSkipSpace (TCHAR src[])
{
	USHORT	i;
	for (i=0; src[i] && (src[i] == 0x20 || src[i] == 0x09); i++);
	return (&src[i]);
}

/*=============================================================================
	XPixmapCheckBackground
=============================================================================*/
BOOL	XPixmapCheckBackground (TCHAR str[])
{
	LPTCHAR	addr;

	addr	= &str[0];
	addr	= XPixmapSkipSpace (++addr);
	while (*addr) {
		if (*addr == 's') {
			return (TRUE);
		}
		addr	= XPixmapSkipString (addr);
		addr	= XPixmapSkipSpace (addr);
		addr	= XPixmapSkipString (addr);
		addr	= XPixmapSkipSpace (addr);
	}
	return (FALSE);
}

/*=============================================================================
	XPixmapExtractColor
=============================================================================*/
BOOL	XPixmapExtractColor (TCHAR ch, TCHAR name[], TCHAR str[])
{
	LPTCHAR	addr;

	addr	= &str[0];
	addr	= XPixmapSkipSpace (++addr);
	while (*addr) {
		if (*addr != ch) {
			addr	= XPixmapSkipString (addr);
			addr	= XPixmapSkipSpace (addr);
			addr	= XPixmapSkipString (addr);
			addr	= XPixmapSkipSpace (addr);
		} else {
			addr	= XPixmapSkipString (addr);
			addr	= XPixmapSkipSpace (addr);
			XPixmapExtractString (name, addr);
			break;
		}
	}
	return (*addr ? TRUE : FALSE);
}

/*=============================================================================
	XPixmapParseHeader
=============================================================================*/
void	XPixmapParseHeader
(
	int		*width,
	int		*height,
	int		*cnum,
	int		*cpp,
	LPTCHAR		xpm[]
)
{
	sscanf (xpm[0], "%d %d %d %d", width, height, cnum, cpp);
}

/*=============================================================================
	CPixMemoryFree
=============================================================================*/
void	CPixMemoryFree
(
	LPCPIXDATA	cpix
)
{
	XFree (cpix->clr);
	XDestroyImage (cpix->image);
}

/*=============================================================================
	CPixXImageToPixmap
=============================================================================*/
Pixmap	CPixXImageToPixmap
(
	Display		*display,
	Window		window,
	LPCPIXDATA	cpix,
	LPTCHAR		xpm[],
	LPCPIXATTR	attr
)
{
	Pixmap	pixmap;
	GC	gc;

	pixmap	= XCreatePixmap (
			display, window,
			cpix->image->width,
			cpix->image->height,
			cpix->image->depth
		);
	if (!pixmap) {
		return ((Pixmap )0);
	}

	gc	= XCreateGC (display, window, 0, 0);

	XPutImage (
		display, pixmap, gc,
		cpix->image,
		0, 0, 0, 0,
		cpix->image->width,
		cpix->image->height
	);

	XFreeGC (display, gc);

	return ((Pixmap )pixmap);
}

/*=============================================================================
	CPixMakeImagePixels8
=============================================================================*/
void	CPixMakeImagePixels8
(
	Display		*display,
	Window		window,
	LPCPIXDATA	cpix,
	LPTCHAR		xpm[],
	LPCPIXATTR	attr
)
{
	UINT		start;
	UINT		end;
	LPTCHAR		ptr;
	XColor		*clr;
	UINT		i, j;

	start	= cpix->cnum + 1;
	end	= start + cpix->height;
	ptr	= cpix->image->data;

	for (i=start; i < end; i++) {
		for (j=0; j < cpix->width; j++) {
			ptr[0]	= cpix->clr[*(xpm[i] + j)].pixel;
			ptr++;
		}
	}
}

/*=============================================================================
	CPixMakeImagePixels16
=============================================================================*/
void	CPixMakeImagePixels16
(
	Display		*display,
	Window		window,
	LPCPIXDATA	cpix,
	LPTCHAR		xpm[],
	LPCPIXATTR	attr
)
{
	UINT		start;
	UINT		end;
	LPTCHAR		ptr;
	XColor		*clr;
	UINT		i, j;

	start	= cpix->cnum + 1;
	end	= start + cpix->height;
	ptr	= cpix->image->data;

	for (i=start; i < end; i++) {
		for (j=0; j < cpix->width; j++) {
			clr	= &cpix->clr[*(xpm[i] + j)];
			ptr[0]	= clr->pixel >> 8;
			ptr[1]	= clr->pixel;
			ptr	+= 2;
		}
	}
}

/*=============================================================================
	CPixMakeImagePixels32
=============================================================================*/
void	CPixMakeImagePixels32
(
	Display		*display,
	Window		window,
	LPCPIXDATA	cpix,
	LPTCHAR		xpm[],
	LPCPIXATTR	attr
)
{
	UINT		start;
	UINT		end;
	LPTCHAR		ptr;
	XColor		*clr;
	UINT		i, j;

	start	= cpix->cnum + 1;
	end	= start + cpix->height;
	ptr	= cpix->image->data;

	for (i=start; i < end; i++) {
		for (j=0; j < cpix->width; j++) {
			clr	= &cpix->clr[*(xpm[i] + j)];
			ptr[0]	= clr->pixel >> 24;
			ptr[1]	= clr->pixel >> 16;
			ptr[2]	= clr->pixel >>  8;
			ptr[3]	= clr->pixel;
			ptr	+= 4;
		}
	}
}

/*=============================================================================
	CPixMakeImage
=============================================================================*/
int	CPixMakeImage
(
	Display		*display,
	Window		window,
	LPCPIXDATA	cpix,
	LPTCHAR		xpm[],
	LPCPIXATTR	attr
)
{
	if (cpix->image->bits_per_pixel == 32) {
		CPixMakeImagePixels32 (display, window, cpix, xpm, attr);
	} else if (cpix->image->bits_per_pixel == 16) {
		CPixMakeImagePixels16 (display, window, cpix, xpm, attr);
	} else {
		CPixMakeImagePixels8 (display, window, cpix, xpm, attr);
	}
	return (CPIX_SUCCESS);
}

/*=============================================================================
	CPixMakeColor
=============================================================================*/
int	CPixMakeColor
(
	Display		*display,
	Window		window,
	LPCPIXDATA	cpix,
	LPTCHAR		xpm[],
	LPCPIXATTR	attr
)
{
	int		sts;
	Colormap	cmap;
	UINT		start;
	UINT		end;
	LPTCHAR		ptr;
	XColor		*clr;
	TCHAR		name[32];
	TCHAR		ch;
	int		i;
	BOOL		rc;

	cpix->clr	= (XColor *)XtMalloc (sizeof (XColor) * MAX_MARK);
	if (!cpix->clr) {
		return (CPIX_COLOR_MALLOC_ERROR);
	}

#if XGRAB_SERVER
	XSync (display, 0);
	XGrabServer (display);
	XSync (display, 0);
#endif

	sts	= CPIX_SUCCESS;
	cmap	= DefaultColormap (display, DefaultScreen (display));
	start	= 1;
	end	= start + cpix->cnum;

	for (i=start; i < end; i++) {
		ptr	= xpm[i];
		clr	= &cpix->clr[*ptr];
		if (attr->valuemask & CPIX_BACKGROUND) {
			rc	= XPixmapCheckBackground (ptr);
			if (rc) {
				clr->pixel	= attr->background;
				XQueryColor (display, cmap, clr);
				continue;
			}
		}
		if (!(attr->valuemask & CPIX_COLOR_KEY)) {
			ch	= 'c';
		} else {
			if (attr->color_key == CPIX_MONO) {
				ch	= 'm';
			} else if (attr->color_key == CPIX_GRAY) {
				ch	= 'g';
			} else {
				ch	= 'c';
			}
		}
		rc	= XPixmapExtractColor (ch, name, ptr);
		if (!rc) {
			sts	= CPIX_COLOR_NAME_ERROR;
			break;
		}
		if (0 == lstrcmp (name, "None")) {
			continue;
		}
		rc	= XParseColor (display, cmap, name, clr);
		if (!rc) {
			sts	= CPIX_COLOR_XPARSE_ERROR;
			break;
		}
		rc	= XAllocColor (display, cmap, clr);
		if (!rc) {
			if (cpix->image->bits_per_pixel > 8) {
				sts	= CPIX_COLOR_XALLOC_ERROR;
				break;
			}
			rc	= QueryColorDepth8 (
					display,
					DefaultScreen (display),
					cmap,
					clr
				);
			if (!rc) {
				sts	= CPIX_QUERY_COLOR_ERROR;
				break;
			}
			rc	= XAllocColor (display, cmap, clr);
			if (!rc) {
				sts	= CPIX_QUERY_COLOR_ERROR;
				break;
			}
		}
	}

#if XGRAB_SERVER
	XSync (display, 0);
	XUngrabServer (display);
	XSync (display, 0);
#endif

	return (sts);
}

/*=============================================================================
	CPixMakeXImage
=============================================================================*/
int	CPixMakeXImage
(
	Display		*display,
	Window		window,
	LPCPIXDATA	cpix,
	LPTCHAR		xpm[],
	LPCPIXATTR	attr
)
{
	Visual		*visual;
	UINT		depth;
	int		format;
	int		offset;
	UINT		width;
	UINT		height;
	int		bitmap_pad;
	int		bytes_per_line;

	visual		= DefaultVisual (display, DefaultScreen (display));
	depth		= DefaultDepth  (display, DefaultScreen (display));
	format		= ZPixmap;
	offset		= 0;
	width		= cpix->width;
	height		= cpix->height;
	bitmap_pad	= ((depth <= 8) ? 8 : ((depth <= 16) ? 16 : 32));
	bytes_per_line	= 0;

	cpix->image	=
		XCreateImage (
			display, visual, depth,
			format, offset, 0, width, height,
			bitmap_pad, bytes_per_line
		);
	if (!cpix->image) {
		return (CPIX_CREATE_XIMAGE_ERROR);
	}

	cpix->image->data	=
		(LPTCHAR )XtMalloc (
			height * cpix->image->bytes_per_line
		);
	if (!cpix->image->data) {
		XDestroyImage (cpix->image);
		return (CPIX_XIMAGE_MALLOC_ERROR);
	}

	cpix->image->byte_order	= MSBFirst;

	return (CPIX_SUCCESS);
}

/*=============================================================================
	CreatePixmapFromXpmData
=============================================================================*/
int	CreatePixmapFromXpmData
(
	Display		*display,
	Window		window,
	Pixmap		*pixmap,
	LPTCHAR		xpm[],
	LPCPIXATTR	attr
)
{
	Pixmap		pix;
	CPIXDATA	cpix;
	int		sts;

	XPixmapParseHeader (
		&cpix.width, &cpix.height, &cpix.cnum, &cpix.cpp, xpm
	);

	sts	= CPixMakeXImage (display, window, &cpix, xpm, attr);
	if (sts != CPIX_SUCCESS) {
		return (sts);
	}

	sts	= CPixMakeColor (display, window, &cpix, xpm, attr);
	if (sts != CPIX_SUCCESS) {
		return (sts);
	}

	sts	= CPixMakeImage (display, window, &cpix, xpm, attr);
	if (sts != CPIX_SUCCESS) {
		return (sts);
	}

	pix	= CPixXImageToPixmap (display, window, &cpix, xpm, attr);
	if (!pix) {
		return (CPIX_CREATE_PIXMAP_ERROR);
	}

	*pixmap		= pix;
	attr->width	= cpix.width;
	attr->height	= cpix.height;

	CPixMemoryFree (&cpix);

	return (CPIX_SUCCESS);
}
#endif /* !USE_XPM */
