/******************************************************************************
	Xui.H
******************************************************************************/

/*=============================================================================
	Include
=============================================================================*/
#include	<X11/Xlib.h>
#include	<typedef.h>

/*=============================================================================
	Define
=============================================================================*/
/*
   A. Status Window 
      +------------------------------+  
      |           2)                 |
      |  +------------------+        |
      |  |        3)        |        |
      |2)|3)3)3)3)3)| 2) + 1)|
      |  |        3)        |        |
      |  +------------------+        |
      |           2)                 |
      +------------------------------+  
*/
#define DRAW_STATUS_WINDOW_TITLE_WIDTH			( 0) /* 1) */
#define DRAW_STATUS_WINDOW_BORDER			( 4) /* 2) */
#define DRAW_STATUS_WINDOW_BUTTON_BORDER		( 2) /* 3) */

/*
   B. PopUpMenu Window 
      +----------------------------------+  
      |               1)                 |
      |               3)                 |
      |      أأأأأأأأأأ      |
      |               3)                 |
      |               3)                 |
      |      =========2)===========      |
      |               3)                 |
      |               3)                 |
      |      أأأأأأأأأأ      |
      |1)3)4)         3)           5)3)1)|
      |               3)                 |
      |      أأأأأأأأأأ      |
      |               3)                 |
      |               3)                 |
      |      أأأأأأأأأأ      |
      |               3)                 |
      |               1)                 |
      +----------------------------------+  
*/
#define DRAW_POPUP_MENU_BORDER			( 4) /* 1) */
#define DRAW_POPUP_MENU_MESSAGE_BORDER		( 2) /* 2) */
#define DRAW_POPUP_MENU_FONT_BORDER		( 2) /* 3) */
#define DRAW_POPUP_MENU_CHECK_MARK		(12) /* 4) */
#define DRAW_POPUP_MENU_CHILD_MARK		(12) /* 5) */

/*
   C. Koho Window
      +--------------------------------+  
      |             2)                 |
      |    +--------6)----------+      |
      |    |        X           |      |
      |    +--------------------+      |
      |  ===========3)===============  |
      |  +-+        4)          +---+  |
      |  5)|أأأأأأأأأ|   |  |
      |  +-+        4)          |   |  |
      |  +-+        4)          |   |  |
      |  |X|أأأأأأأأأ|   |  |
      |  +-+        4)          |   |  |
      |  +-+        4)          |   |  |
      |  |X|أأأأأأأأأ|   |  |
      |  +-+        4)          |   |  |
      |  +-+        4)          |   |  |
      |  |X|أأأأأأأأأ|   |  |
      |  +-+        4)          |   |  |
      |  +-+        4)          |   |  |
      |2)|X|أأأأأأأأأ| 8)|2)|
      |  +-+        4)          |   |  |
      |  +-+        4)          |   |  |
      |  |X|أأأأأأأأأ|   |  |
      |  +-+        4)          |   |  |
      |  +-+        4)          |   |  |
      |  |X|أأأأأأأأأ|   |  |
      |  +-+        4)          |   |  |
      |  +-+        4)          |   |  |
      |  |X|أأأأأأأأأ|   |  |
      |  +-+        4)          |   |  |
      |  +-+        4)          |   |  |
      |  |X|أأأأأأأأأ|   |  |
      |  +-+        4)          +---+  |
      |  +----------------------+      |
      |  |          8)          |      |
      |  +----------------------+      |
      |  ===========3)===============  |
      |  +-+        4)                 |
      |  |X|أأأأأأأأأ       |
      |  +-+        4)                 |
      |  ===========3)===============  |
      |             1)                 |
      |  7)-++--+                      |
      |  |  ||  | أأأأأأ X/XX  |
      |  +--++--+                      |
      |             2)                 |
      +--------------------------------+
*/
#define DRAW_KOHO_WINDOW_TITLE_BORDER			( 3) /* 1) */
#define DRAW_KOHO_WINDOW_BORDER				( 6) /* 2) */
#define DRAW_KOHO_WINDOW_DON_BORDER			( 4) /* 3) */
#define DRAW_KOHO_WINDOW_FONT_BORDER			( 1) /* 4) */
#define DRAW_KOHO_WINDOW_VIDX_BORDER			( 4) /* 5) */
#define DRAW_KOHO_WINDOW_HIDX_BORDER			( 0) /* 6) */
#define DRAW_KOHO_WINDOW_BUTTON_BORDER			( 2) /* 7) */
#define DRAW_KOHO_WINDOW_SCROLL_BAR_SIZE		(16) /* 8) */
#define DRAW_KOHO_WINDOW_MIN_SCROLL_PTR_SIZE	\
		(DRAW_KOHO_WINDOW_SCROLL_BAR_SIZE / 2)	     /* 8)/2 */

/*
   D. Preedit Root Window

      +------------------------------------------------------------+
      |                   2)                                       |
      |                                                            |
      |    +----------------------------------+                    |
      |    |    3)                            |                    |
      | 2) |3)أأأأأ                  3)| 2) +      1)       |
      |    |    3)                            |                    |
      |    +----------------------------------+                    |
      |                                                            |
      |                   2)                                       |
      +------------------------------------------------------------+
 */
#define DRAW_PREEDIT_ROOT_WINDOW_TITLE_WIDTH		(15) /* 1) */
#define DRAW_PREEDIT_ROOT_WINDOW_BORDER			( 6) /* 2) */
#define DRAW_PREEDIT_ROOT_WINDOW_FONT_BORDER		( 2) /* 3) */
#define DRAW_PREEDIT_ROOT_WINDOW_BORDER_ALL		\
		(DRAW_PREEDIT_ROOT_WINDOW_BORDER	\
		+ DRAW_PREEDIT_ROOT_WINDOW_FONT_BORDER)	     /* 2)+3) */
#if 1
#define DRAW_PREEDIT_ROOT_WINDOW_MIN_LENGTH	(5)
#else
#define DRAW_PREEDIT_ROOT_WINDOW_MIN_LENGTH	(NUM_CHAR / PREEDIT_MAX)
#endif

#define MAX_CELLS		(256)

#define PM_WIN_MAX		(2)

#define	PM_KWD_NON		(0)
#define	PM_KWD_SKP		(1)
#define	PM_KWD_BDR		(2)
#define	PM_KWD_CHK		(3)
#define	PM_KWD_PRC		(4)
#define	PM_KWD_ACC		(5)
#define	PM_KWD_CMD		(6)

#define	PM_OFF			(0)
#define	PM_ON			(1)

#define	PH_OFF			(0)
#define	PH_ON_STS		(1)
#define	PH_ON_KOHO		(2)

#define	VC_COLOR		(0)
#define	VC_GRAY			(1)
#define	VC_MONO			(2)

#define	CLR_LIGHT_YELLOW	(0x01)
#define	CLR_RED			(0x08)
#define	CLR_CYAN		(0x0c)
#define	CLR_MAZENTA		(0x0f)
#define	CLR_YELLOW		(0x11)
#define	CLR_GREEN		(0x12)
#define	CLR_BLUE		(0x1c)
#define	CLR_BLACK		(0x28)
#define	CLR_GRAY		(0x2d)
#define	CLR_TICKER_GRAY		(0x2e)
#define	CLR_WHITE		(0x2f)

#define	CLR_WFORE		(CLR_BLACK)
#define	CLR_WBACK		(CLR_GRAY)
#define	CLR_WBDR		(CLR_BLACK)
#define	CLR_TFORE		(CLR_BLACK)
#define	CLR_TBACK		(CLR_WHITE)

#define	CUR_NM			(0)
#define	CUR_GB			(1)
#define	CUR_TB			(2)
#define	CUR_LR			(3)
#define	CUR_TL			(4)
#define	CUR_TR			(5)
#define	CUR_BL			(6)
#define	CUR_BR			(7)
#define	CUR_MAX			(8)

#define	STATUS_WINDOW_TYPE_MIN	(0)
#define	STATUS_WINDOW_TYPE_MAX	(1)

#define WIN_BASE		(0)

#define WIN_KOHO_HISTORY	(WIN_BASE - 16)
#define WIN_KOHO_HSCR_PTR	(WIN_BASE - 15)
#define WIN_KOHO_HSCR_BAR_LEFT	(WIN_BASE - 14)
#define WIN_KOHO_HSCR_BAR_RIGHT	(WIN_BASE - 13)
#define WIN_KOHO_VSCR_PTR	(WIN_BASE - 12)
#define WIN_KOHO_VSCR_BAR_NEXT	(WIN_BASE - 11)
#define WIN_KOHO_VSCR_BAR_PREV	(WIN_BASE - 10)

#define WIN_LOWER_RIGHT		(WIN_BASE -  9)
#define WIN_UPPER_RIGHT		(WIN_BASE -  8)
#define WIN_LOWER_LEFT		(WIN_BASE -  7)
#define WIN_UPPER_LEFT		(WIN_BASE -  6)
#define WIN_RIGHT		(WIN_BASE -  5)
#define WIN_LEFT		(WIN_BASE -  4)
#define WIN_BOTTOM		(WIN_BASE -  3)
#define WIN_TOP			(WIN_BASE -  2)
#define WIN_NONE		(WIN_BASE -  1)

#define WIN_STS_BASE		(WIN_BASE)

#define WIN_STS_BTN_BASE	(WIN_STS_BASE)
#define WIN_STS_BTN_MAX		(128)
#define WIN_STS_BTN_END		(WIN_STS_BTN_MAX)

#define WIN_STS_END		(WIN_STS_BTN_END)

#define WIN_KOHO_BASE			(WIN_STS_END + 1)

#define WIN_KOHO_SCR_BASE		(WIN_KOHO_BASE)
#define WIN_KOHO_VSCR_ARW_NEXT		(WIN_KOHO_SCR_BASE + 0)
#define WIN_KOHO_VSCR_ARW_PREV		(WIN_KOHO_SCR_BASE + 1)
#define WIN_KOHO_HSCR_ARW_LEFT		(WIN_KOHO_SCR_BASE + 2)
#define WIN_KOHO_HSCR_ARW_RIGHT		(WIN_KOHO_SCR_BASE + 3)
#define WIN_KOHO_SCR_END		(WIN_KOHO_SCR_BASE + 4)

#define WIN_KOHO_BTN_BASE		(WIN_KOHO_SCR_END)
#define WIN_KOHO_BTN_MAX		(WIN_KOHO_BTN_BASE + VKP_BTN_MAX)
#define WIN_KOHO_BTN_END		(WIN_KOHO_BTN_MAX)

#define WIN_KOHO_ITEM_BASE		(WIN_KOHO_BTN_END)

#define	ITR_WI			(1)
#define	ITR_WD			(2)
#define	ITR_HI			(3)
#define	ITR_HD			(4)
#define	ITR_WIHI		(5)
#define	ITR_WDHD		(6)
#define	ITR_WIHD		(7)
#define	ITR_WDHI		(8)
#define	ITR_HIWI		(9)
#define	ITR_HDWD		(10)
#define	ITR_HIWD		(11)
#define	ITR_HDWI		(12)

#define STS_BTN_SIZE_MIN	(16 - DRAW_STATUS_WINDOW_BUTTON_BORDER * 2)
#define STS_BTN_SIZE_MAX	(32 - DRAW_STATUS_WINDOW_BUTTON_BORDER * 2)
#define STS_BTN_SIZE_SMALL	(16 - DRAW_STATUS_WINDOW_BUTTON_BORDER * 2)
#define STS_BTN_SIZE_BIG	(24 - DRAW_STATUS_WINDOW_BUTTON_BORDER * 2)

#define STS_BTN_MODE_NML	(0)
#define STS_BTN_MODE_POP	(1)
#define STS_BTN_MODE_PSH	(2)

#define STS_BTN_OFF_NML		(0)
#define STS_BTN_OFF_MIN		(1)
#define STS_BTN_OFF_HID		(2)

#define ENV_ALL			(0x10)
#define ENV_MODE		(0x02)
#define ENV_POS			(0x01)

/*=============================================================================
	Enum
=============================================================================*/
enum	FC_ID
{
	FC_0 ,	FC_1 ,	FC_2 ,	FC_3 ,
	FC_4 ,	FC_5 ,	FC_6 ,	FC_7 ,
	FC_8 ,	FC_9 ,	FC_10,	FC_11,
	FC_12,	FC_13,	FC_14,	FC_15,
	FC_16,	FC_17,	FC_18,	FC_19,
	FC_20,	FC_21,	FC_22,	FC_23
};

/*=============================================================================
	Typedef
=============================================================================*/
typedef struct _tagVCLR {
	USHORT		usUse;
	XColor		xColor;
} VCLR, *LPVCLR;

typedef struct _tagVCUR {
	USHORT		usUse;
	Cursor		cCursor;
	USHORT		usSymbol;
} VCUR, *LPVCUR;

typedef struct _tagVCMD {
	WORD		wFunc;
	LPTSTR		lptFile;
} VCMD, *LPVCMD;

typedef struct _tagVCWI {
	WORD		wWinX;
	WORD		wWinY;
	WORD		wRootX;
	WORD		wRootY;
	WORD		wRootW;
	WORD		wRootH;
	WORD		wStepW;
	WORD		wStepH;
	WORD		wMinW;
	WORD		wMinH;
} VCWI, *LPVCWI;

typedef struct _tagVPM {
	WORD		wKeyword;
	WORD		wCheck;
	WORD		wVisible;
	WORD		wChild;
	LPTSTR		lptText;
	WORD		wFunc;
} VPM, *LPVPM;

typedef struct _tagVPML {
	WORD		wCount;
	LPVPM		vMenu;
} VPML, *LPVPML;

typedef struct  _tagVKBD {
	WORD		wComma;
	WORD		wPeriod;
	WORD		wSlash;
	WORD		wHyphen;
	WORD		wBracket;
	WORD		wTen;
	WORD		wMaru;
} VKBD, *LPVKBD;

typedef struct	_tagVSPLT {
	WORD		wOffStatus;

	WORD		wBtnMode;
	WORD		wBtnPos;
	WORD		wBtnPosPrev;
	WORD		wBtnPosRoot;

	WORD		wBtnType;
	WORD		wBtnCount;
	WORD		wBtnFunc[WIN_STS_BTN_MAX];
	WORD		wBtnIndex[WIN_STS_BTN_MAX];
	WORD		wBtnX[WIN_STS_BTN_MAX];
	WORD		wBtnY[WIN_STS_BTN_MAX];
	WORD		wBtnSize;
	WORD		wBtnRow;
	WORD		wBtnCol;

	WORD		wMaxBtnPosRoot;
	WORD		wMaxBtnCount;
	WORD		wMaxBtnFunc[WIN_STS_BTN_MAX];
	WORD		wMaxBtnIndex[WIN_STS_BTN_MAX];
	WORD		wMaxBtnRow;
	WORD		wMaxBtnCol;

	Pixmap		pPixBtn;
	Pixmap		pPixBtnOn;
	Pixmap		pPixBtnOff;
	WORD		wPixBtnSize;
	WORD		wPixBtnBgClr;

	WORD		wPopHelp;

#if 1 /* 20000217 MOVE-WINDOW */
	XPoint		XButton;
#endif
} VSPLT, *LPVSPLT;

typedef struct	_tagVKPLT {
	WORD		wBtnPos;
	WORD		wBtnCount;
	WORD		wBtnFunc[WIN_KOHO_BTN_MAX];
	WORD		wBtnIndex[WIN_KOHO_BTN_MAX];

	Pixmap		pPixBtn;
	WORD		wPixBtnSize;
	WORD		wPixBtnBgClr;

	WORD		wPopHelp;
} VKPLT, *LPVKPLT;

typedef struct  _tagVWIN {
	int		First;

	int		Gravity;
	SHORT		StatusNW;
	SHORT		StatusNH;
	SHORT		PreeditNW;
	SHORT		PreeditNH;

	SHORT		StatusWinType;
	XRectangle	StatusR;
	XRectangle	StatusRMin;
	XRectangle	StatusRMax;
	XRectangle	StatusM;
	XRectangle	StatusMMin;
	XRectangle	StatusMMax;

	XRectangle	PreeditR;
	XRectangle	PreeditM;

	XRectangle	PopUpMenu[PM_WIN_MAX];
	SHORT		PopUpMenuPos[PM_WIN_MAX];
	SHORT		PopUpMenuIdx;
	SHORT		PopUpMenuFlg;

	XRectangle	PopUpHelp;
	SHORT		PopUpHelpPos;
	SHORT		PopUpHelpFlg;

	char		*BaseFontName;
	FXimFont	BaseFont;

	ULONG		WinForePixel;
	ULONG		WinBackPixel;
	ULONG		WinBorderPixel;
	ULONG		WinShadowPixel;
	ULONG		WinHiglightPixel;
	ULONG		TextForePixel;
	ULONG		TextBackPixel;

	GC 		GCList[6];
	GC		XorGC;
	XGCValues	MoveGCV;

	VSPLT		vsplt;
	VKPLT		vkplt;
	VKBD		vkbd;

	LPARAM		Param;

} VWIN, *LPVWIN;

